;;*****************************************************************************
;;
;; File: ThrustLoader.asm
;;
;;       Don't need this unless not using compressor
;;
;;*****************************************************************************
        PROCESSOR 6502

;
; Constants
;
LOADER_LOCATION EQU $0333

SCREEN_ORIGIN   EQU $0C00

MAIN_PROG_START EQU $100D

PTR1            EQU $03
PTR2            EQU PTR1+2
TEMP            EQU PTR2+2

FROM_ADDR       EQU PTR1
TO_ADDR         EQU PTR2

        INCLUDE CbmBasicTokens.asm
        INCLUDE C16KernalEntryPoints.asm

;;;
;;; 2009-09: MY: DASM doesn't have the CBM macro to declare strings as arrays
;;;              of Commodore screen codes.  This is a poor-man's version that
;;;              works for most letters numbers and punctuation symbols.
;;;
;;;              Replace this:
;;;                CBM "SOME STRING"
;;;              with this:
;;;                DV.B ASCII2CBM 'S, 'O, 'M, 'E, ' , 'S, 'T, 'R, 'I, 'N, 'G
;;;
;;;              Note that spaces on the end of lines can be a problem as many
;;;              editors remove them, so they are best done as follows -- eg
;;;              replace this:
;;;                CBM "X "
;;;              with this:
;;;                DV.B ASCII2CBM 'X, " "
;;;
ASCII2CBM   EQM [.. & $3F]


        ;
        ; BASIC lines to run code below and then load & run main application
        ;
        ORG $1001
BASIC_L10
        WORD BASIC_END ; Address of next line?
        WORD 10 ; Line number
        BYTE BASIC_TKN_SYS ; SYS command
        BYTE " "
        BYTE [CODE_START/1000 + $30] ; Address (in ASCII)
        BYTE [[CODE_START%1000]/100 + $30]
        BYTE [[CODE_START%100]/10 + $30]
        BYTE [[CODE_START%10] + $30]
        BYTE $00 ; End of line
BASIC_END
        WORD $0000

CODE_START      SEI
                LDX #$FF
                TXS
                AND #$00
                PHA
                PLP ; Clear all flags
                SEI
                STA $FF15 ; Background
                STA $FF19 ; Border

                ORA #$02
FILL_STACK      STA $0100,X
                DEX
                BNE FILL_STACK
                STA $FF3E ; ROM select

                ; Clear the screen
                LDA #$20
CLEAR_SCREEN    STA $0C00,X
                STA $0D00,X
                STA $0E00,X
                STA $0F00,X
                DEX
                BNE CLEAR_SCREEN

                JSR PRINT_LOAD_MES

                ;
                ; Copy the loader code to where it won't get overwritten by the
                ; main program as it's loading in.
                ;
                ; We could copy a little less code (eg file open etc could be
                ; done here), but it's a little neater to keep all the loading
                ; stuff together.
                ;
                LDA #<LOADER_CODE
                STA FROM_ADDR
                LDA #>LOADER_CODE
                STA FROM_ADDR+1
                LDA #<LOADER_LOCATION
                STA TO_ADDR
                LDA #>LOADER_LOCATION
                STA TO_ADDR+1
                LDX #>[LOADER_CODE_END - LOADER_CODE]
                LDA #<[LOADER_CODE_END - LOADER_CODE]
                JSR COPY_MEMORY
                CLI

                JMP LOADER_LOCATION


;*******************************************************************************
;* Copy a block of memory.  Assumes that blocks do not overlap.
;*
;* Inputs: FROM_ADDR
;*         TO_ADDR
;*         X-register - number of whole pages to copy
;*         Accumulator - size of last partial page to copy
;*
;*******************************************************************************
COPY_MEMORY    LDY #0
               CPX #0
               BEQ CM_ONLY_LAST_PG
               PHA
CM_INNER_LOOP  LDA (FROM_ADDR),Y
               STA (TO_ADDR),Y
               INY
               BNE CM_INNER_LOOP ; Copy whole pages until we reach the last one
               INC FROM_ADDR+1
               INC TO_ADDR+1
               DEX
               BNE CM_INNER_LOOP
               PLA
               BEQ CM_DONE
CM_ONLY_LAST_PG
               TAX
CM_LAST_LOOP   LDA (FROM_ADDR),Y
               STA (TO_ADDR),Y
               INY
               DEX
               BNE CM_LAST_LOOP
CM_DONE        RTS


;
;
; Print loading page - similar to code used in game to print title page
; DAT2 USED FOR SCREEN
; CL1 USED FOR COLOUR
;
PRINT_LOAD_MES  LDA #<LOAD_MESSAGES
                STA PTR1
                LDA #>LOAD_MESSAGES
                STA PTR1+1
                LDY #$FF
PR_LD_MS1       INY
                TYA
                BEQ PR_LD_MS1B
                CLC
                ADC PTR1
                STA PTR1
                BCC PR_LD_MS1A
                INC PTR1+1
PR_LD_MS1A      LDY #0
PR_LD_MS1B      LDA (PTR1),Y
                TAX
                INY
                LDA (PTR1),Y
                BEQ PR_LD_MS_DONE
                STX PR_LD_MS_SCR+1
                STX PR_LD_MS_COL+1
                STA PR_LD_MS_SCR+2
                SEC
                SBC #4
                STA PR_LD_MS_COL+2
                LDX #0
PR_LD_MS_RD_COL INY
                LDA (PTR1),Y ; Colour
                STA TEMP
PR_LD_MS2       INY
                LDA (PTR1),Y
                BEQ PR_LD_MS1
                BMI PR_LD_MS_RD_COL
PR_LD_MS_SCR    STA $0C00,X ; smc
                LDA TEMP
PR_LD_MS_COL    STA $0800,X ; smc
                INX
                BNE PR_LD_MS2
                INC PR_LD_MS_SCR+2
                INC PR_LD_MS_COL+2
                BNE PR_LD_MS2 ; = JMP
PR_LD_MS_DONE   RTS

LOAD_MESSAGES
                WORD SCREEN_ORIGIN+[1*40]+10
                BYTE $77 ;YELLOW
                DV.B ASCII2CBM 'C, 'O, 'M, 'M, 'O, 'D, 'O, 'R, 'E
                DV.B ASCII2CBM ' , '1, '6, ' , 'T, 'H, 'R, 'U, 'S, 'T
                BYTE 0

                WORD SCREEN_ORIGIN+[3*40]+2
                BYTE $53 ;CYAN
                DV.B ASCII2CBM 'R, 'E, 'C, 'O, 'M, 'P, 'I, 'L, 'E, 'D
                DV.B ASCII2CBM ' , 'F, 'R, 'O, 'M, ' , 'S, 'O, 'U, 'R, 'C, 'E
                DV.B ASCII2CBM ' , 'C, 'O, 'D, 'E, ' , 'I, 'N, ' , '2, '0, '0, '9
                BYTE 0

                WORD SCREEN_ORIGIN+[4*40]+6
                BYTE $53 ;CYAN
                DV.B ASCII2CBM '(, 'W, 'I, 'T, 'H
                DV.B ASCII2CBM ' , 'S, 'C, 'R, 'O, 'L, 'L, 'I, 'N, 'G
                DV.B ASCII2CBM ' , 'B, 'U, 'G, ' , 'F, 'I, 'X, 'E, 'D, '!, ')
                BYTE 0

                WORD SCREEN_ORIGIN+[8*40]+0
                BYTE $36 ;BLUE
                DV.B ASCII2CBM 'T, 'I, 'T, 'L, 'E, ' , 'S, 'C, 'R, 'E, 'E, 'N
                DV.B ASCII2CBM ' , 'C, 'H, 'E, 'A, 'T, ' , 'K, 'E, 'Y, 'S, ':
                BYTE 0

                WORD SCREEN_ORIGIN+[10*40]+2
                BYTE $65 ;GREEN
                DV.B ASCII2CBM 'L
                BYTE $FF, $56 ;BLUE
                DV.B ASCII2CBM ' , '-, ' , 'T, 'O, 'G, 'G, 'L, 'E, 'S
                DV.B ASCII2CBM ' , 'I, 'N, 'F, 'I, 'N, 'I, 'T, 'E, ' , 'L, 'I, 'V, 'E, 'S
                BYTE 0

                WORD SCREEN_ORIGIN+[12*40]+2
                BYTE $65 ;GREEN
                DV.B ASCII2CBM 'F
                BYTE $FF, $56 ;BLUE
                DV.B ASCII2CBM ' , '-, ' , 'T, 'O, 'G, 'G, 'L, 'E, 'S
                DV.B ASCII2CBM ' , 'I, 'N, 'F, 'I, 'N, 'I, 'T, 'E, ' , 'F, 'U, 'E, 'L
                BYTE 0

                WORD SCREEN_ORIGIN+[14*40]+2
                BYTE $65 ;GREEN
                DV.B ASCII2CBM 'S
                BYTE $FF, $56 ;BLUE
                DV.B ASCII2CBM ' , '-, ' , 'I, 'N, 'C, 'R, 'E, 'A, 'S, 'E, 'S
                DV.B ASCII2CBM ' , 'T, 'H, 'E, ' , 'S, 'T, 'A, 'R, 'T, 'I, 'N, 'G
                DV.B ASCII2CBM ' , '(, 'L, 'A, 'N, 'D, '), 'S, 'C, 'A, 'P, 'E
                BYTE 0

                WORD SCREEN_ORIGIN+[16*40]+2
                BYTE $65 ;GREEN
                DV.B ASCII2CBM 'V
                BYTE $FF, $56 ;BLUE
                DV.B ASCII2CBM ' , '-, ' , 'T, 'O, 'G, 'G, 'L, 'E, 'S
                DV.B ASCII2CBM ' , 'I, 'N, 'V, 'U, 'L, 'N, 'E, 'R, 'A, 'B, 'I, 'L, 'I, 'T, 'Y, '!
                BYTE 0

MS_LOADING_POS  EQU SCREEN_ORIGIN+[20*40]+8
                WORD MS_LOADING_POS
                BYTE $77 ;YELLOW
MS_LOADING_START
                DV.B ASCII2CBM 'L, 'O, 'A, 'D, 'I, 'N, 'G, '., '., '.
MS_LOADING_END  BYTE 0
MS_LOADING_LEN  EQU MS_LOADING_END - MS_LOADING_START

MS_CONTIN_POS   EQU SCREEN_ORIGIN+[21*40]+8
                WORD MS_CONTIN_POS
                BYTE $00 ; Black
MS_CONTIN_START DV.B ASCII2CBM 'P, 'R, 'E, 'S, 'S, ' , 'S, 'P, 'A, 'C, 'E
                DV.B ASCII2CBM ' , 'T, 'O, ' , 'C, 'O, 'N, 'T, 'I, 'N, 'U, 'E
MS_CONTIN_END   BYTE 0
MS_CONTIN_LEN   EQU MS_CONTIN_END - MS_CONTIN_START
                BYTE 0, 0

LOADER_CODE
;;******** vvv Relocated code vvv *********************************************
                ;
                ; This code is going to be copied to LOADER_LOCATION and run there
                ;
                RORG LOADER_LOCATION

                ; Turn off control and error messages
                LDA #$00
                JSR C16_KERN_SETMSG

                ; Set logical file number, device address, secondary address (command number)
                LDA #$01 ; logical file number
                TAY      ; secondary address (command number)
                LDX #$08 ; device number
                JSR C16_KERN_SETLFS

                ; Set the name of the file we want to load in
                LDA #[MAIN_PROG_NAME_END - MAIN_PROG_NAME]
                LDX #<MAIN_PROG_NAME
                LDY #>MAIN_PROG_NAME
                JSR C16_KERN_SETNAM

                ; Load the file
                LDA #0
                JSR C16_KERN_LOADSP

                ; Close logical file 8
                LDA #$01
                JSR C16_KERN_CLOSE

                JSR C16_KERN_CLALL ; Close all files

                ; Black the "Loading..." message
                LDX #MS_LOADING_LEN
                LDA #$00 ; Black
BLK_MES_COLR    STA [MS_LOADING_POS-$0400-1],X
                DEX
                BNE BLK_MES_COLR

                ; Unblack the "Press Space to Continue" message
                LDX #MS_CONTIN_LEN
                LDA #$72 ; Colour
SET_MES_COLR    STA [MS_CONTIN_POS-$0400-1],X
                DEX
                BNE SET_MES_COLR

                LDX #$7F ;WAIT FOR SPACE BAR
                LDY #$10
CHKY_NB_PRESS   TYA
                JSR CHKKY
                BNE CHKY_NB_PRESS
CHKY_NB_RELEASE TYA
                JSR CHKKY
                BEQ CHKY_NB_RELEASE

                ; Blank the screen, because we're about to overwrite it with
                ; code when we decompress.  The raster interrupt in the main
                ; program will turn it on again.
                LDA #$78
                STA $FF06

                JMP MAIN_PROG_START

                ;
                ;CHECK FOR A KEY
                ;SEND ROW IN XR, COL IN AR
                ;ZERO FLG SET: KEY PRESSED
CHKKY           STX $FD30
                STX $FF08
                AND $FF08
                RTS


MAIN_PROG_NAME  BYTE "THRUST-MAIN"
MAIN_PROG_NAME_END

                REND
;;******** ^^^ Relocated code ^^^ *********************************************
LOADER_CODE_END



