;;*****************************************************************************
;; File: Thrust.asm
;;       Source code for Commodore 16 / Plus 4 version of Thrust
;;
;; Build: To compile on Windows from command line in same directory as this
;;        file, run:
;;        ..\dasm\dasm Thrust.asm -f1 -othrust-main.prg -sthrust.sym.txt
;;        ..\exo20beta7\win32\exomizer.exe -t4 sfx $0c00 thrust.prg -p 1 -o c16thrust.prg -x1
;;
;; Notes: Converted from PDS (Programmer's Development System) to the Cadaver /
;;        Olaf variant of DASM (see http://cadaver.homeftp.net/tools.htm)
;;        downloaded from http://cadaver.homeftp.net/tools/dasm.zip.
;;
;;        Summary of changes made to get this to compile with DASM:
;;
;;          - Merge all files into this one, converting CR to NL, removing other
;;            control characters (except tab) and removing non-ASCII characters.
;;            Eg use the following command line in Cygwin:
;;              $ echo "" > merged.asm
;;              $ ls -1 X?.C16 | sort | while read ii
;;              > do cat $ii | tr -d "\000-\010\013\014\177-\377" | tr "\015" "\012" | tr "\016-\037" "\040" >> merged.asm
;;              > done
;;              $
;;          - Remove remaining non-text portions of PDS files by sight.  (The
;;            method above gets rid of most but not all of the binary data.)
;;            What needs to be removed is usually just the garbage lines before
;;            the "PDS Pc3.a :0000:  (C)  1985-7 P.D Systems" at the start of
;;            each file.
;;          - Comment out PDS instructions for uploading code from PC to C64
;;          - Change "DB" pseudop (declare byte data) to "BYTE" (Could have used
;;            "DC" or "DC.B" but these don't get syntax highlighted in
;;            SlickEdit on Windows.)
;;            Take care not to change "$DB" strings or "$xxDB" strings or DB
;;            inside HEX strings.
;;          - Change "DW" pseudop (declare word data) to "WORD"
;;          - Add "\tPROCESSOR 6502" to the beginning of the file
;;          - Change () to [] in expressions following mnemonics
;;          - Change regular expression "ASL A$" to "ASL ; = ASL A"
;;                                      "LSR A$" to "LSR ; = LSR A"
;;                                      "ROL A$" to "ROL ; = ROL A"
;;                                      "ROR A$" to "ROR ; = ROR A"
;;
;; Cheats:
;;        Pressing 'L' on the start screen toggles infinite lives
;;        Pressing 'F' on the start screen toggles infinite fuel
;;        Pressing 'S' on the start screen increases the starting (Land)scape
;;        Pressing 'V' on the start screen toggles invulnerability!
;;
;;*****************************************************************************

                PROCESSOR 6502

;
; File X0.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;C16 THRUST
                ;BY MATTHEW F. YOUNG
                ;V 8709252135
                ;PDS VERSION
                ;
;               SEND COMPUTER1
                ;CONSTANTS...
BEEPSND         EQU 900
GN1SND          EQU 50
GN2SND          EQU 9
GNSSND          EQU 9
FR1SND          EQU 500
FR2SND          EQU 100
FRSSND          EQU 50
EX1SND          EQU 400
EX2SND          EQU 70
EXSSND          EQU 6
THRSND          EQU 780
SH1SND          EQU 0
SH2SND          EQU 8
NUMCOL          EQU $7F ;MISSN# COL ETC
BLHYCL          EQU $65 ;BLL.HYP.COL.:GRN
SHHYCL          EQU $67 ;SHP.HYP.COL.:YEL
NUMSCP          EQU 6           ;NUMBER OF SCAPES
BUBCHR          EQU 0 ;BUBBLES CHAR
SPACE           EQU 32 ;SPACE CHAR
SHIPCOL         EQU $71 ;WHITE
SCRCOL          EQU $79 ;SCOREBOX COLOUR
FUELSCR         EQU $30
BFLSCR          EQU $15
GUNSCR          EQU $75
BWID            EQU 5
BHGT            EQU BWID
SHWID           EQU $0B
SHHGT           EQU $0A
HYPDELAY        EQU $19
SECLEN          EQU 60 ;# FRAMES EQU 1 'SEC' (FOR COUNTDOWN)
                ;
                ;'VARS'
                ;VARIABLES: MOSTLY ZERO PAGE
                ;

                SEG.U VARIABLES
;
; Zero page variables
;
                ORG $02             ;start of zero page

                ;
                ;TOTALLY PERMANENT VARS
                ;
SHPBUF          DS.B 64
JMPVEC          DS.B 2 ; Jump vector for raster interrupt
SNDCTR          DS.B 1 ; Used for sound timing
FRAMEYSMTH      DS.B 1
MC1COL          DS.B 1
MC2COL          DS.B 1
TITLE           DS.B 1
SCORE           DS.B 3

;;;
;;; 2009-10: MY: Added this for "cheats"
;;;
;;;              Bit 0 set = infinite lives
;;;              Bit 1 set = infinite fuel
;;;              Bit 2 set = invulnerable
;;;
CHEAT_MODE     DS.B 1
CHEAT_INFINITE_LIVES  EQU $01
CHEAT_INFINITE_FUEL   EQU $02
CHEAT_INVULNERABLE    EQU $04
CHEAT_START_SCAPE DS.B 1

;FINVPERM         ;MUST BE LAST PERM VAR
                ;
                ;GAME-PERMANENT VARS
GAMPERM         ;EQU FINVPERM
FUEL            DS.B 3
CANTH           DS.B 1 ; "Can thrust": 0 Means have fuel left, so can thrust and use shield
NOFUEL          DS.B 1
LIVES           DS.B 1
DIFFIC          DS.B 1
XTRDIF          DS.B 1
CMESNO          DS.B 1 ; Current message number
MISSN           DS.B 1 ; Mission number
VISFLG          DS.B 1 ; 0 = visible landscape, $FF = invisible
FRAME           DS.B 1
SCAPNO          DS.B 1
GRVFLG          DS.B 1
DIFFC2          DS.B 1
RANNUM          DS.B 1 ; Random number store
;FINGAMP
                ;
                ;SCAPE-PERMANENT VARS
SCPPERM         ;EQU FINGAMP
GRAV            DS.B 2
LEDSTL          DS.B 1
LEDSTH          DS.B 1
LEDRCL          DS.B 1
LEDRCH          DS.B 1
REDSTL          DS.B 1
REDSTH          DS.B 1
REDRCL          DS.B 1
REDRCH          DS.B 1
REACDM          DS.B 1 ; Reactor damage
REACNT          DS.B 1 ; Cycles before reactor unstunned
NUMGUN          DS.B 1 ; Number of "guns" on this scape
TPLAYL          DS.B 1 ; \_ Y position of top of play area in _characters_,
TPLAYH          DS.B 1 ; /  measured downwards from top of scape.
PICKUP          DS.B 1 ; Whether ball being carried or not. 0 = not carried
;FINSCPP
                ;
                ;NON-PERMANENT VARS (FRAME-PERM.)
NONPERM         ;EQU FINSCPP
THCTR           DS.B 1
PFRCTR          DS.B 1
RESULT          DS.B 2
HIRESL          DS.B 2
ACC1            DS.B 1
ACC2            DS.B 2
TEMPORARY       DS.B 2
CON1            DS.B 1
CONNECT         DS.B 1
SHPOR           DS.B 1 ; Ship orientation (ie direction ship is pointing) 0-31
PROR            DS.B 2 ; Pull Rope orientation
; X coordinates are measured left-to-right
; Y coordinates are measured top-to-bottom
; Centre of mass is same as ship position when no load is being towed
CMX             DS.B 2 ; Centre of Mass x position
CMY             DS.B 2 ; Centre of Mass y position
PRANVL          DS.B 2 ; Pull Rope angular velocity
CMXVL           DS.B 2 ; Centre of Mass x velocity
CMYVL           DS.B 2 ; Centre of Mass y velocity
SHPX            DS.B 2 ; Ship x position
SHPY            DS.B 2 ; Ship y position
LOADX           DS.B 2 ; Load x position
LOADY           DS.B 2 ; Load y position
CORD1           DS.B 2
CORD2           DS.B 2
CORD3           DS.B 4
XFLG            DS.B 1
YFLG            DS.B 1
XDIST           DS.B 1
YDIST           DS.B 1
CH1             DS.B 2
CH2             DS.B 2
PLTFLG          DS.B 1
NEWCHR          DS.B 1
TBBPTR          DS.B 1
SC1             DS.B 2
TEMP1           DS.B 1
TEMP2           DS.B 1
CL1             DS.B 2
DAT1            DS.B 2
DAT2            DS.B 2
BUFFER          DS.B 4
SHST            DS.B 1
TBST            DS.B 1
SHSL            DS.B 1
SHSH            DS.B 1
BLSL            DS.B 1
BLSH            DS.B 1
VDPL            DS.B 1
VDPH            DS.B 1
HDPLT           DS.B 1
HDPRT           DS.B 1
DTPTL           DS.B 1
DTPTR           DS.B 1
CRLVD           DS.B 1
CRRVD           DS.B 1
LDIREC          DS.B 1
RDIREC          DS.B 1
SCPPTR          DS.B 1
OLDLDC          DS.B 1
OLDRDC          DS.B 1
SMTHY           DS.B 1 ; Smooth Y screen position (ie pixel offset _subtracted_ from TPLAYL/TPLAYH)
SMTHX           DS.B 1 ; Smooth X screen position (ie pixel offset _subtracted_ from PLAYX)
PLAYX           DS.B 1 ; X-position of left of screen in _characters_
COLFLG          DS.B 1
SCXVL           DS.B 1 ; Screen x-velocity - +ve means ship going right so PLAYX increasing
SCYVL           DS.B 1 ; Screen y-velocity - +ve means ship going down so TPLAYL/TPLAYH increasing
TEMPX           DS.B 1
TEMPY           DS.B 1
GOR             DS.B 1 ; Gun orientation
LOPCNT          DS.B 1
CHAR            DS.B 1
OTBST           DS.B 1
COLDET          DS.B 1 ; Collision detection flag. 0 = no collision
GSTAT           DS.B 1 ; Current "gun" status
LENX            DS.B 1
LENY            DS.B 1
GATCNT          DS.B 1 ; 0 = gate closed, non-0 = fully/partly open
CNTDWN          DS.B 1 ; "Seconds" before reactor death. >$80 = reactor alive
KLLSHP          DS.B 1 ; 0 = alive, +ve = dying (no control, ship/ball explode etc), -ve = dead
RETFLG          DS.B 1 ; Stop auto-repeat on fire
FIRFLG          DS.B 1 ; 0 = don't (try to) shoot
BLLCNT          DS.B 1 ; Current bullet (0-3)
SHIFT           DS.B 1 ; 0 means SHIFT not pressed (so not thrusting), 1 means thrust on
V2SNDI          DS.B 1
V1SNDI          DS.B 1
BONUS           DS.B 1
WRAPFLG         DS.B 1
MCONFLG         DS.B 1
FLSHCNT         DS.B 1
DVAR1           DS.B 1
DVAR2           DS.B 1
REACX           DS.B 1
REACY           DS.B 1
UNPLOT          DS.B 1
PLAYING         DS.B 1
NEWTEMP         DS.B 1
GATPOS          DS.B 1
SGLDRV          DS.B 1

;FINNONP
                ;
;END             ;EQU FINNONP
ENDZP          ;END OF ZERO PAGE VARS

               IF ENDZP>$00FF
               ECHO "ZERO-PAGE OVERFLOW !!!"
               ERR
               ENDIF

                ;
                ORG $0200
BLXLO           DS.B 32
BLXHI           DS.B 32
BLYLO           DS.B 32
BLYHI           DS.B 32
BLXV            DS.B 32
BLYV            DS.B 32
BULLFG          DS.B 32
BLTYPE          DS.B 32
TBCS            DS.B 10
TBSL            DS.B 10
TBSH            DS.B 10
SHCS            DS.B 28
BLCS            DS.B 9
LLBUF           DS.B 23
LEBUF           DS.B 23
RLBUF           DS.B 23
REBUF           DS.B 23
BLCHS           DS.B 32
GUNFGS          DS.B 19
TANKS           DS.B 12
BLSPL           DS.B 32
BLSPH           DS.B 32
LINBUF          DS.B 40
;FIN

                ORG $0500
SHLDDT          DS.B $3F
SHPDAT          DS.B $240
;ENGRFS

                SEG CODE
                ;
                ;
                ;
                ;part "6":COPY GRAPHICS DOWN MEM AND RUN GAME
                ORG $0C00
                SEI
                LDX #$00
COPGRFS         LDA $0D00,X
                STA $0500,X
                LDA $0E00,X
                STA $0600,X
                LDA $0F00,X
                STA $0700,X
                INX
                BNE COPGRFS
                LDA #$1B
                STA $FF06 ;SCREEN ON
                JMP Q
ENPT6           ;

                ORG $0D00 ;$0500
                ;SHIELD DATA
                HEX 000000000000007C
                HEX 0001830002008004
                HEX 0040080020080020
                HEX 1000101000101000
                HEX 1010001010001008
                HEX 0020080020040040
                HEX 020080018300007C
                HEX 00000000000000

                ;SHIP DATA
                HEX 0000000000000010
                HEX 0000280000280000
                HEX 4400004400008200
                HEX 0082000101000101
                HEX 000600C008002004
                HEX 0040020080023880
                HEX 0145000082000000
                HEX 0000000000000000
                HEX 0000000000000004
                HEX 00000A0000120000
                HEX 2200002200004100
                HEX 0081000081000701
                HEX 000801000400C004
                HEX 0020020040027080
                HEX 0189000006000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000030000050000
                HEX 0900003100004100
                HEX 0081000701000801
                HEX 0008010004010004
                HEX 008002004002E080
                HEX 0113000014000008
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000018000068000
                HEX 188000208006C080
                HEX 0901000801000801
                HEX 0008010008020007
                HEX 0100008080004180
                HEX 004E000030000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 000000000001C000
                HEX 0E40023040054080
                HEX 0480800400800801
                HEX 0008010008020006
                HEX 0400010200008100
                HEX 008E000070000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000000000000003
                HEX 03E002BC20044040
                HEX 0400400400800800
                HEX 8008010006020001
                HEX 0200008400008200
                HEX 008200007C000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000000001000002
                HEX 8000047FE0040020
                HEX 0800401000800C01
                HEX 0002010002020002
                HEX 0400040800030800
                HEX 00C8000030000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000800001400002
                HEX 4000043E000801E0
                HEX 0800100400200200
                HEX 4002018002020004
                HEX 0C00041000031000
                HEX 00D0000020000000
                HEX 0000000000000000
                HEX 0000000000000000
                HEX 0000400000A00003
                HEX 2000041800080600
                HEX 0401800200600200
                HEX 1002006004018008
                HEX 0600041800032000
                HEX 00A0000040000000
                HEX 0000000000000000

;;;
;;; 2009-09: MY: DASM doesn't have the CBM macro to declare strings as arrays
;;;              of Commodore screen codes.  This is a poor-man's version that
;;;              works for most letters numbers and punctuation symbols.
;;;
;;;              Replace this:
;;;                CBM "SOME STRING"
;;;              with this:
;;;                DV.B ASCII2CBM 'S, 'O, 'M, 'E, ' , 'S, 'T, 'R, 'I, 'N, 'G
;;;
;;;              Note that spaces on the end of lines can be a problem as many
;;;              editors remove them, so they are best done as follows -- eg
;;;              replace this:
;;;                CBM "X "
;;;              with this:
;;;                DV.B ASCII2CBM 'X, " "
;;;
ASCII2CBM   EQM [.. & $3F]


;*
;* There are 64 keys on the C16 / Plus/4 keyboard.  To read a key press, write
;* the latch value to $FD30 then to $FD08.  The next read from $FD08 shows
;* which of the group of 8 keys selected by the latch were pressed.
;*
;* In the table below, the row heading is the value to store in $FD30 and $FD08
;* to select the group of keys in that row.  The column heading is the bit that
;* will be _cleared_ in output read from $FD08 if the corresponding key in the
;* selected row is pressed.
;*
;*       ---------------------------------------------------------------------------------
;*       | $80        | $40   | $20       | $10   | $08     | $04   | $02      | $01     |
;* ------+------------+-------+-----------+-------+---------+-------+----------+----------
;* | $7F | Run/Stop   | q     | Commodore | Space | 2       | Ctrl  | Clr/Home | 1 (one) |
;* | $BF | /          | +     | =         | Esc   | Right   | ]     | *        | Left    |
;* | $DF | <          | -     | [         | >     | Up      | l     | p        | Down    |
;* | $EF | n          | o     | k         | m     | 0       | j     | i        | 9       |
;* | $F7 | v          | u     | h         | b     | 8       | g     | y        | 7       |
;* | $FB | x          | t     | f         | c     | 6       | d     | r        | 5       |
;* | $FD | Shift-Keys | e     | s         | z     | 4       | a     | w        | 3       |
;* | $FE | @sign      | F3/F6 | F2/F5     | F1/F4 | Help/F7 | Pound | Return   | ins/del |
;* ---------------------------------------------------------------------------------------
;*
;* The constants and CALL_CHKKY / CALL_CHKKY_NB macros below allow symbolic names to be used in
;* calls to CHKKY.  Eg:
;*
;*    CALL_CHKKY C16_KEY_SPACE
;*
;* is expanded to:
;*
;*    LDX #$7F
;*    LDA #$10
;*    JSR CHKKY
;*
C16_KEY_INS_DEL   EQU 0
C16_KEY_RETURN    EQU 1
C16_KEY_POUND     EQU 2
C16_KEY_F7        EQU 3
C16_KEY_F1        EQU 4
C16_KEY_F2        EQU 5
C16_KEY_F3        EQU 6
C16_KEY_AT_SIGN   EQU 7
C16_KEY_3         EQU 8
C16_KEY_W         EQU 9
C16_KEY_A         EQU 10
C16_KEY_4         EQU 11
C16_KEY_Z         EQU 12
C16_KEY_S         EQU 13
C16_KEY_E         EQU 14
C16_KEY_SHIFT     EQU 15
C16_KEY_5         EQU 16
C16_KEY_R         EQU 17
C16_KEY_D         EQU 18
C16_KEY_6         EQU 19
C16_KEY_C         EQU 20
C16_KEY_F         EQU 21
C16_KEY_T         EQU 22
C16_KEY_X         EQU 23
C16_KEY_7         EQU 24
C16_KEY_Y         EQU 25
C16_KEY_G         EQU 26
C16_KEY_8         EQU 27
C16_KEY_B         EQU 28
C16_KEY_H         EQU 29
C16_KEY_U         EQU 30
C16_KEY_V         EQU 31
C16_KEY_9         EQU 32
C16_KEY_I         EQU 33
C16_KEY_J         EQU 34
C16_KEY_0         EQU 35
C16_KEY_M         EQU 36
C16_KEY_K         EQU 37
C16_KEY_O         EQU 38
C16_KEY_N         EQU 39
C16_KEY_DOWN      EQU 40
C16_KEY_P         EQU 41
C16_KEY_L         EQU 42
C16_KEY_UP        EQU 43
C16_KEY_>         EQU 44
C16_KEY_[         EQU 45
C16_KEY_MINUS     EQU 46
C16_KEY_<         EQU 47
C16_KEY_LEFT      EQU 48
C16_KEY_STAR      EQU 49
C16_KEY_]         EQU 50
C16_KEY_RIGHT     EQU 51
C16_KEY_ESC       EQU 52
C16_KEY_EQUALS    EQU 53
C16_KEY_PLUS      EQU 54
C16_KEY_SLASH     EQU 55
C16_KEY_1         EQU 56
C16_KEY_CLR_HOME  EQU 57
C16_KEY_CTRL      EQU 58
C16_KEY_2         EQU 59
C16_KEY_SPACE     EQU 60
C16_KEY_COMMODORE EQU 61
C16_KEY_Q         EQU 62
C16_KEY_RUN_STOP  EQU 63

                ; Loads X and Acc with appropriate values to check for the key
                ; specified as the parameter.  Then calls CHKKY.
                MAC CALL_CHKKY
                LDX #[$FF - [1 << [{1} >> 3]]]
                LDA #[1 << [{1} & $07]]
                JSR CHKKY
                ENDM

                ; Loads XR and YR with appropriate values to check for the key
                ; specified as the parameter.  Then calls CHKKY_NO_BOUNCE.
                MAC CALL_CHKKY_NB
                LDX #[$FF - [1 << [{1} >> 3]]]
                LDY #[1 << [{1} & $07]]
                JSR CHKKY_NO_BOUNCE
                ENDM

                ;
                ;'SETUP'
                ;
                ;
                ORG $0F9E
                ;SETUP
Q               SEI
                LDX #$FF
                TXS
                STX $FF3F ;RAM SELECT
                ;COPY STACK ROUTINES DOWN FROM CHARSET
                INX ;=0
                STX CHEAT_MODE
                STX CHEAT_START_SCAPE
COPYSTACK       LDA STACK,X
                STA $0100,X
                INX
                BNE COPYSTACK
                ;SETUP IRQ/RESET VECTORS
                LDX #5
SETVECS         LDA VECTORS,X
                STA $FFFA,X
                DEX
                BPL SETVECS
                LDX #RSNM0&255
                LDY #RSNM0/256
                STX JMPVEC
                STY JMPVEC+1
                LDA #2
                STA $FF0A
                LDA #$00
                STA $FF0B ; Raster interrupt register
                STA $FF15 ;BACKGROUND
                STA $FF19 ;BORDER
                PHA
                PLP ;CLEARS I ASWELL
                LDA #$C8
                STA $FF12 ;CHARS ETC FROM RAM
                LDA #$38
                STA $FF13 ;CHARBASE=$3800
                LDA #$9B
                STA $FF07 ;FULL CHARSET ON
                ;
                ;
                ;CLEAR OUT SCORE
                LDX #SCORE
                JSR CLEARSUB ;ALSO CLEARS MOST OF REST OF ZP
                ;
                ;TITLE PAGE,START GAME
TITPAG          LDA #1
                STA TITLE
                BNE SETUP ;JMP
TITTEXT         JSR PRTITLES
WAITSPC         LDX #$7F ;CHECK FOR SPACE BAR
                LDA #$10
                JSR CHKKY
                ;Could replace 3 lines above CALL_CHKKY C16_KEY_SPACE, but macro needs to be above this code
                BEQ STARTSPC

                ; 2009-11: MY: Uncomment this call and the corresponding
                ;              function to allow the screen smooth scroll
                ;              position on the title page to be adjusted by the
                ;              arrow keys.  The X and Y smooth scroll offsets
                ;              are shown to the left of the lives number.
                ;
;                JSR XX_SCROLL_TEST

                ;
                ; Disable cheats in the "unenhanced" version, but in such a way
                ; that they can be added back in by modifying the uncompressed
                ; binary
                ;
                IFCONST BUILD_ENHANCED_VERSION
                JSR CHECK_CHEAT_KEYS
                ELSE
                JSR CHEAT_KEYS_END
                ENDIF


                LDX #$FB ;CHECK FOR FIRE (JOYSTICK PORT 1)
                LDA #$40
                JSR CHKKY
                BNE WAITSPC
STARTFIRE       LDX #JOYSTICK&255
                LDY #JOYSTICK/256
                BNE STRCTRL ;JMP
STARTSPC        LDX #KEYS&255
                LDY #KEYS/256
STRCTRL         STX CTRLJUMP+1
                STY CTRLJUMP+2
                DEC TITLE
                ;JMP SETUP-FALLS THRU
                ;
                ;
                ;GENERAL SET UP VARS
                ;
                ;
SETUP           ;MAIN ENTRY POINT
                ;
                ;CLEAR NEARLY ALL VARS ON ZP
                LDX #GAMPERM
                JSR CLEARSUB
                ;
                JSR SCRBOX
                ;
                ;
STRTGM          ;START GAME
                ;
                LDA TITLE
                BNE NTSTSCORE
                LDX #2
ZSCFLL          AND #0
                STA FUEL,X
                STA SCORE,X
                DEX
                BPL ZSCFLL
                ;
                LDX #1
                STX DIFFIC
                ;
                LDA #$10
                STA FUEL+1 ;BCD
NTSTSCORE       JSR PRFUEL
                JSR NOINLV ;PRINTS SCORE
                ;
                LDA #$03
                STA LIVES
                JSR PRINT_LIVES
                ;
                ORA #$FF
                STA FRAME
                LDX CHEAT_START_SCAPE
                DEX
                STX SCAPNO
                ;
NXTFRM          ;NEXT FRAME
                ;
                LDX #SCPPERM
                JSR CLEARSUB
                ;
                INC FRAME
                INC SCAPNO
                LDA SCAPNO
                CMP #NUMSCP ;NUMBER OF SCAPES SEE TS...
                BCC NTNXSQ
                ;
                ;BACK TO FIRST FRAME
                AND #0
                STA SCAPNO
                LDA GRVFLG
                EOR #$FF
                STA GRVFLG
                BNE NOTINV
                LDA VISFLG
                EOR #$FF
                STA VISFLG
                JMP NOTRG
                ;
NOTINV          LDX #RGOFF ;'REVERSE GRAVITY'
                JSR PRMESS
                LDY #150
                JSR LONGPAUSE
                JMP NOTIL
                ;
                ;
NOTRG
                ;'INVISIBLE LANDSCAPE'
                LDX #ILOFF
                JSR PRMESS
                LDY #150
                JSR LONGPAUSE
                ;
NOTIL
                ;
NTNXSQ
                ;
                SED
                CLC
                LDA MISSN
                ADC #1
                STA MISSN
                CLD
                ;
                TAX
                LDA #1
                CPX #3
                BCC DIFFOK
                INC DIFFIC
                LDA DIFFIC
                CMP #35
                BCC DIFFOK
                LDA #35
                STA DIFFIC
DIFFOK          CLC
                ADC XTRDIF
                STA DIFFC2
                ;
                AND #0
                STA XTRDIF
                ;
                ;
                ;SET UP VARS
                AND #0
                STA PICKUP
                ;
                LDA #50
                STA REACDM
                LDX SCAPNO
                LDA GRVTAB,X
                STA GRAV
                LDA LDLTAB,X
                STA LEDSTL
                LDA LRLTAB,X
                STA LEDRCL
                LDA LDHTAB,X
                STA LEDSTH
                LDA LRHTAB,X
                STA LEDRCH
                LDA RDLTAB,X
                STA REDSTL
                LDA RRLTAB,X
                STA REDRCL
                LDA RDHTAB,X
                STA REDSTH
                LDA RRHTAB,X
                STA REDRCH
                ;
                LDA MULT1,X
                STA MC1COL ;($FF16)
                LDA MULT2,X
                STA MC2COL ;($FF17)
                ;
                LDY #0
                LDA GRVFLG
                BEQ NOREVG
                LDA GRAV
                EOR #255
                STA GRAV
                LDY #255
NOREVG          STY GRAV+1
                ;
                ;SELF MODIFYING BIT.....
                TXA
                ASL ; = ASL A   ;*2
                TAX
                LDA GUNXD,X
                STA SMDGNX+1
                LDA GUNXD+1,X
                STA SMDGNX+2
                LDA GUNYLD,X
                STA SMDGYL+1
                LDA GUNYLD+1,X
                STA SMDGYL+2
                LDA GUNORD,X
                STA SMDGOR+1
                LDA GUNORD+1,X
                STA SMDGOR+2
                ;
                ;
                LDX #31
                AND #0
ENGUNS          STA GUNFGS,X
                DEX
                BPL ENGUNS
                LDX SCAPNO
                LDY GUNTOT,X
                STY NUMGUN
                LDA #1
ONGUNS          STA GUNFGS-1,Y
                DEY
                BNE ONGUNS
                ;
                LDX #11
                LDA #26
TANKLP          STA TANKS,X
                DEX
                BPL TANKLP
                ;
                ;
RESTRT
                ;RESUMPTION
                ;
                JSR CLS
                ;
                LDX #NONPERM
                JSR CLEARSUB
                ;
                LDX #31
                AND #0
CLEARBLS        STA BULLFG,X
                DEX
                BPL CLEARBLS
                ;
                LDX SCAPNO
                LDA YCRDL,X
                STA DAT1
                LDA YCRDH,X
                STA DAT1+1
                LDA YDND,X ;# OF STRTS
                STA TEMP1
                CLC
                ADC DAT1
                STA DAT2
                LDA DAT1+1
                ADC #0
                STA DAT2+1
                ;
                LDY #0
CHKSTY          LDA (DAT1),Y
                CMP TPLAYL
                LDA (DAT2),Y
                SBC TPLAYH
                BCS CKSTYO
                INY
                CPY TEMP1
                BNE CHKSTY
                ;
                AND #0
                STA PICKUP
                ;
CKSTYO          CPY #0
                BEQ SUTSPY
                DEY
                ;
SUTSPY          LDA (DAT1),Y
                STA TPLAYL
                CLC
                LDA (DAT2),Y
                STA TPLAYH
                TYA
                ADC TEMP1
                TAY
                LDA (DAT2),Y
                STA CMY+1
                TYA
                ADC TEMP1
                TAY
                LDA (DAT2),Y
                STA PLAYX
                TYA
                ADC TEMP1
                TAY
                LDA (DAT2),Y
                STA CMX+1
                JSR POSNS
                ;
                ;
                LDY #1
                LDA PICKUP
                BEQ NOHPU
                DEY ;=0
                LDX #1
                LDA GRAV+1
                BPL NORPO
                LDX #17
NORPO           STX PROR
NOHPU           LDA GRAV+1
                BPL NORSO
                LDA #16
                STA SHPOR
NORSO           STY GUNFGS ;1ST 'GUN' IS ALWAYS PEDISTAL
                JSR DOSHBF ;COPY SHIP DATA
                ;
                LDX #$FF
                STX TBBPTR
                STX CNTDWN
                TXS
                ;
                JSR WHOLED
                DEC UNPLOT ;=$FF
                JSR PLOTOB
                LDA TITLE
                BEQ DOGAME
                JMP TITTEXT
DOGAME          JSR HYPSPC
                JSR WHOLED ;HYPSPC MAY BUGGER SCAPE
                JSR UNDRD1
                ;
ZLOOP           JSR RDKBRD
                JMP ZLOOP

;******************************************************
;* Process the "cheat" keys
;******************************************************
                ; Toggle the cheat mode specified in the parameter
                ; Modifies AR
                MAC TOGGLE_CHEAT
                LDA CHEAT_MODE
                EOR #{1}
                STA CHEAT_MODE
                ENDM
CHECK_CHEAT_KEYS
                ; Pressing 'L' on the start screen toggles infinite lives
                CALL_CHKKY_NB C16_KEY_L
                BNE CHK_CHEAT_NOT_LIVES
                TOGGLE_CHEAT CHEAT_INFINITE_LIVES
                JSR PRINT_LIVES

                ; Pressing 'F' on the start screen toggles infinite fuel
CHK_CHEAT_NOT_LIVES
                CALL_CHKKY_NB C16_KEY_F
                BNE CHK_CHEAT_NOT_FUEL
                TOGGLE_CHEAT CHEAT_INFINITE_FUEL
                JSR PRFUEL

                ; Pressing 'S' on the start screen increases the starting (Land)scape
CHK_CHEAT_NOT_FUEL
                CALL_CHKKY_NB C16_KEY_S
                BNE CHK_CHEAT_NOT_SCAPE
                LDY CHEAT_START_SCAPE
                INY
                CPY #NUMSCP
                BNE CHK_CHEAT_NOT_MX
                LDY #0
CHK_CHEAT_NOT_MX
                STY CHEAT_START_SCAPE
CHK_CHEAT_NOT_SCAPE
                ;
                ; Need to write the start scape to the screen regardless of
                ; whether it changed as otherwise will display wrong value
                ; after a game has been played starting from scape >1
                ;
                ; Note that we use the underlined digits, as we don't have non-
                ; underlined versions of all the digits in the character set --
                ; just the ones that were needed for the original title screen,
                ; for "16" and "1987"!
                ;
                CLC
                LDA CHEAT_START_SCAPE
                ADC #$31
                STA [SPMESS_POS + SPMESS_LEN - 1]

                ; Pressing 'V' on the start screen toggles invulnerability!
                CALL_CHKKY_NB C16_KEY_V
                BNE CHK_CHEAT_NOT_INVULNERABLE
                TOGGLE_CHEAT CHEAT_INVULNERABLE
                JSR SCRBOX
                JSR PRINT_LIVES
                JSR PRFUEL
CHK_CHEAT_NOT_INVULNERABLE

CHEAT_KEYS_END  RTS


                ;
                ;
                ;
                ;DEATH...
DIE             INC MCONFLG ;MUL.COL. OFF IN PLAY AREA
                INC PLAYING ;STOP COUNTDOWN
                JSR CLS
                LDA NOFUEL
                BNE OUTOFFUEL
                JSR DCLVES
                CPX #255
                BEQ GAMEOVER
                LDA CNTDWN
                BPL FAILED
                JMP RESTRT
                ;
                ;
                ;
                ;ESCAPE PLANET...
LPLAN           INC MCONFLG ;MUL.COL. OFF IN PLAY AREA
                INC PLAYING ;STOP COUNTDOWN
                ;
                LDY #5 ;<??
                JSR LONGPAUSE
                JSR HYPSPC
                LDY #20
                JSR LONGPAUSE
                JSR CLS
                LDA NOFUEL
                BNE OUTOFFUEL
                LDA PICKUP
                BNE MCOM
                ;
                ;INCOMPLETE
                LDA CNTDWN
                BMI NOTFAILED
                ;
                ;FAILED
FAILED          LDA #8
                STA XTRDIF
                ;
                JSR PRPDMESS
                JSR PRMIMESS
                JSR PRMISNUM
                LDX #FAOFF
                JSR PRMESS
                LDX #NBOFF
                JSR PRMESS
                LDY #120
                JSR LONGPAUSE
                JMP NXTFRM
                ;
OUTOFFUEL       LDX #FUOFF
                JSR PRMESS
                ;
GAMEOVER        LDX #GOOFF
                JSR PRMESS
                LDY #90
                JSR LONGPAUSE
                JMP TITPAG
                ;
NOTFAILED       JSR PRMIMESS
                LDX #INOFF
                JSR PRMESS
                JSR PRCMMESS
                LDY #60
                JSR LONGPAUSE
                JMP RESTRT
                ;
                ;COMPLETE MISSION
MCOM            JSR PRMIMESS
                JSR PRMISNUM
                JSR PRCMMESS
                ;
                AND #0
                STA BONUS ;LO-BYTE=0
                LDA SCAPNO
                CLC
                ADC #5
                LDY CNTDWN
                BMI NOXTRA
                PHA
                JSR PRPDMESS
                PLA
                CLC
                ADC #5 ;FOR REACTOR
NOXTRA          STA TEMPX ;A*400=BONUS
BONLP           LDA #$40
                JSR ADDSCR
                LDA BONUS
                SED
                CLC
                ADC #4
                STA BONUS
                CLD
                DEC TEMPX
                BNE BONLP
                ;
                LDX #BOOFF
                JSR PRMESS
                LDA BONUS
                JSR MK2DNUM
                BCC BNLDEZ ;SPACE ALREADY PRINTED
                STY BOSCRP
BNLDEZ          STX BOSCRP+1
                LDA #NUMCOL
                STA BOSCRP-$0400
                STA BOSCRP-$03FF ;COL.
                ;
                ;
                LDY #100
                JSR LONGPAUSE
                JMP NXTFRM
                ;
                ;
                ;
                ;
PAUSE           DEX
                BNE PAUSE
PAUSE1          DEX
                BNE PAUSE1
                DEY
                BNE PAUSE
                RTS
                ;
                ;
LONGPAUSE       TYA
                CLC
                ADC SNDCTR
LNGPSL          CMP SNDCTR
                BNE LNGPSL
                RTS
                ;
                ;
CLEARSUB        AND #0
CLRZP           STA $00,X
                INX
                BNE CLRZP
                RTS
                ;
                ;
                ;
                ;SETUP DATA
                ;EACH ITEM SHOULD HAVE AN
                ;ENTRY FOR EACH LANDSCAPE
                ;
                ;# OF START POSNS FOR EACH SCAPE
YDND            BYTE 1,1,3,3,4,5
                ;
YCRD0           BYTE 149 ;TPLAYL
                BYTE 0 ;<<TPLAYH
                BYTE 66 ;YPOS OF SHIP-NO HI BYTE
                BYTE 39 ;<PLAYX
                BYTE 68 ;XPOS OF SHIP
                ;
YCRD1           BYTE 146
                BYTE 0
                BYTE 89
                BYTE 43
                BYTE 108
                ;
YCRD2           BYTE 144,178,202
                BYTE 0,0,0
                BYTE 66,111,115
                BYTE 43,50,17
                BYTE 98,114,72
                ;
YCRD3           BYTE 146,168,196
                BYTE 0,0,0
                BYTE 109,116,116
                BYTE 43,43,66
                BYTE 108,130,130
                ;
YCRD4           BYTE 146,199,227,250
                BYTE 0,0,0,0
                BYTE 109,74,70,70
                BYTE 43,41,38,52
                BYTE 108,146,107,121
                ;
YCRD5           BYTE 153,195,227,250,20
                BYTE 0,0,0,0,1
                BYTE 77,96,100,106,112
                BYTE 41,66,62,50,70
                BYTE 76,114,104,120,120
                ;
YCRDL           BYTE YCRD0&255
                BYTE YCRD1&255
                BYTE YCRD2&255
                BYTE YCRD3&255
                BYTE YCRD4&255
                BYTE YCRD5&255
                ;
YCRDH           BYTE YCRD0/256
                BYTE YCRD1/256
                BYTE YCRD2/256
                BYTE YCRD3/256
                BYTE YCRD4/256
                BYTE YCRD5/256
                ;
GUNXD           WORD GUNX0,GUNX1,GUNX2
                WORD GUNX3,GUNX4,GUNX5
GUNYLD          WORD GUNYL0,GUNYL1,GUNYL2
                WORD GUNYL3,GUNYL4,GUNYL5
GUNORD          WORD GUNOR0,GUNOR1,GUNOR2
                WORD GUNOR3,GUNOR4,GUNOR5
                ;
                ;
MULT1           BYTE $32,$35,$33,$3E,$38,$34
MULT2           BYTE $52,$55,$53,$5E,$58,$54
                ;
GRVTAB          BYTE 5,7,9,11,12,13
                ;
LDLTAB          BYTE LEDIS0&255
                BYTE LEDIS1&255
                BYTE LEDIS2&255
                BYTE LEDIS3&255
                BYTE LEDIS4&255
                BYTE LEDIS5&255
LDHTAB          BYTE LEDIS0/256
                BYTE LEDIS1/256
                BYTE LEDIS2/256
                BYTE LEDIS3/256
                BYTE LEDIS4/256
                BYTE LEDIS5/256
LRLTAB          BYTE LEDIR0&255
                BYTE LEDIR1&255
                BYTE LEDIR2&255
                BYTE LEDIR3&255
                BYTE LEDIR4&255
                BYTE LEDIR5&255
LRHTAB          BYTE LEDIR0/256
                BYTE LEDIR1/256
                BYTE LEDIR2/256
                BYTE LEDIR3/256
                BYTE LEDIR4/256
                BYTE LEDIR5/256
RDLTAB          BYTE REDIS0&255
                BYTE REDIS1&255
                BYTE REDIS2&255
                BYTE REDIS3&255
                BYTE REDIS4&255
                BYTE REDIS5&255
RDHTAB          BYTE REDIS0/256
                BYTE REDIS1/256
                BYTE REDIS2/256
                BYTE REDIS3/256
                BYTE REDIS4/256
                BYTE REDIS5/256
RRLTAB          BYTE REDIR0&255
                BYTE REDIR1&255
                BYTE REDIR2&255
                BYTE REDIR3&255
                BYTE REDIR4&255
                BYTE REDIR5&255
RRHTAB          BYTE REDIR0/256
                BYTE REDIR1/256
                BYTE REDIR2/256
                BYTE REDIR3/256
                BYTE REDIR4/256
                BYTE REDIR5/256
                ;
                ;
                ;


;
; File X1.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;'MISC'
                ;MISCELANEOUS ROUTINES
                ;
                ;
                ;CLEAR PLAY AREA,SET COLOUR
CLS             LDY #$DC
CLS1            LDA #SPACE
                STA $0C77,Y
                STA $0D53,Y
                STA $0E2F,Y
                STA $0F0B,Y
                LDA #SHIPCOL+8
                STA $0877,Y
                STA $0953,Y
                STA $0A2F,Y
                STA $0B0B,Y
                DEY
                BNE CLS1
                RTS
                ;
                ;
                ;SET UP SC1 FROM CO-ORDS IN CORD1
SETCRD          LDA CORD1+1 ;Y-CO
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A ;/8=CHARS
                TAY
                LDX SCREENHI,Y
                LDA CORD1 ;X-CO
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A ;/8=CHARS
                CLC
                ADC SCREENLO,Y
SETCRDEX        STA SC1
                BCC SCD1
                INX
SCD1            STX SC1+1
                RTS
                ;
SCREENLO        BYTE $7C,$A4,$CC,$F4,$1C,$44,$6C,$94,$BC,$E4,$0C
                BYTE $34,$5C,$84,$AC,$D4,$FC,$24,$4C,$74,$9C,$C4
OFFSETHI        BYTE $0C,$0C,$0C
SCREENHI        BYTE $0C,$0C,$0C,$0C,$0D,$0D,$0D,$0D,$0D,$0D,$0E
                BYTE $0E,$0E,$0E,$0E,$0E,$0E,$0F,$0F,$0F,$0F,$0F
OFFSETLO        BYTE $00,$28,$50
SCREENLO2       BYTE $78,$A0,$C8,$F0,$18,$40,$68,$90,$B8,$E0,$08
                BYTE $30,$58,$80,$A8,$D0,$F8,$20,$48,$70,$98,$C0
                ;
                ;
                ;SET UP SC1 FROM CHAR CORDS IN
                ;CORD2
STCCRD          LDA CORD2+1 ;Y-CO
                TAY
                LDX SCREENHI,Y
                LDA CORD2 ;X-CO
                CLC
                ADC SCREENLO2,Y
                JMP SETCRDEX
                ;
                ;
                ;COPY CHAR DATA ETC FOR BULLET
                ;CO-RDS (XR IS OFFSET)
                ;RETURN SCR/COL POS IN DAT1/2
STCD2           LDA BLYHI,X
                TAY
                LDA SCREENHI,Y
                STA DAT1+1
                LDA BLXHI,X
                CLC
                ADC SCREENLO2,Y
                STA DAT1
                STA DAT2
                STA BLSPL,X
                BCC STCD2A
                INC DAT1+1
STCD2A          LDA DAT1+1
                STA BLSPH,X
                SEC
                SBC #$04
                STA DAT2+1
                RTS
                ;
                ;
                ;SET UP CHR PTRS++CHR FROM IN AR
                ;1ST CHR TO IN TEMP1++CHR # IN XR
                ;ALSO COPY CHAR DATA
SETCHR          STA CH1
                AND #0
                ASL CH1
                ROL ; = ROL A
                ASL CH1
                ROL ; = ROL A
                ASL CH1
                ROL ; = ROL A
                ADC #$38 ;CARRY CLR AS LDA #0 ^
                STA CH1+1 ;SAVE CHAR DATA PTR
                TXA
                CLC
                ADC TEMP1
                STA CH2
                AND #0
                ASL CH2
                ROL ; = ROL A
                ASL CH2
                ROL ; = ROL A
                ASL CH2
                ROL ; = ROL A
                ADC #$38 ;CARRY CLR AS LDA #0 ^
                STA CH2+1
                ;
                ;COPY
                LDY #7
COPSUB          LDA (CH1),Y
                STA (CH2),Y
                DEY
                BPL COPSUB
                RTS

                ;
                ; Check for a key
                ;
                ; Send: row in XR, col in YR
                ;
                ; Return: zero flag set: key pressed
                ;         zero flag clear: key pressed and released
                ;
                ; Modifies: AR
                ;
                ; NB: If the key is pressed, this routine does not return until
                ; it is released.  So use only on title screen, not during
                ; game.
                ;
CHKKY_NO_BOUNCE TYA
                JSR CHKKY
                BNE CHKY_NB_DONE
CHKY_NB_RELEASE TYA
                JSR CHKKY
                BEQ CHKY_NB_RELEASE
                AND #0
CHKY_NB_DONE    RTS

                ;
                ;CHECK FOR A KEY
                ;SEND ROW IN XR, COL IN AR
                ;ZERO FLG SET: KEY PRESSED
CHKKY           STX $FD30
                STX $FF08
                AND $FF08
                RTS


RDKBRD          INC LOPCNT ;DELAYS ETC
                LDA KLLSHP
                BNE RDKFRZ
CTRLJUMP        JMP KEYS ;SMC
                ;
KEYS            ;READ KEYBOARD - SHIP CONTROLS
                ;
                ;LEFT (A)
                ;LDX #$FD
                ;LDA #$04
                ;JSR CHKKY
                CALL_CHKKY C16_KEY_A
                BNE RDKRT
                DEC SHPOR
                JMP FIXSHP
                ;
                ;RIGHT (S)
RDKRT           LDA #$20
                JSR CHKKY ;X ALREADY RIGHT VALUE
                BNE RDKTH
                INC SHPOR
                ;
                ;KEEP SHPOR IN RANGE
FIXSHP          LDA SHPOR
                AND #31
                STA SHPOR
                ;+DO DATA
                JSR DOSHBF
                ;
                ;THRUST (SHIFT)
RDKTH           ;LDX #$FD
                ;LDA #$80
                ;JSR CHKKY
                CALL_CHKKY C16_KEY_SHIFT
;                CALL_CHKKY C16_KEY_Z

                BEQ RDKTH2
                ;
                ;TURN OFF THRUST SOUND
THRSNDOFF       AND #0
                STA SHIFT
                BEQ RDKSHD ;JMP
                ;
                ;DO THRUST & SOUND
RDKTH2          LDA CANTH
                BNE THRSNDOFF
                LDA #1
                STA SHIFT
                JSR THRUST
                JSR DCFUEL
                ;
                ;SHIELD (SPACE)
RDKSHD          AND #0
                STA SHST
                LDA CANTH
                BNE RDKFIR
                LDX #$7F
                LDA #$10
                JSR CHKKY
                BNE RDKFIR
                INC SHST
                JSR DCFUEL
                ;TBEAM ETC ELSEWHERE
                ;
                ;FIRE (RETURN)
RDKFIR          LDX #$FE
                LDA #$02
                JSR CHKKY
                BEQ FIRCHK
                STA RETFLG
                BNE RDKFRZ ;JMP
FIRCHK          LDX RETFLG
                BEQ RDKFRZ
                STX FIRFLG
                AND #0
                STA RETFLG
                ;
                ;CHECK FREEZE (F 3)
RDKFRZ          LDX #$FE
                LDA #$40
                JSR CHKKY
                BNE RDKEX
                ;
                ;DO PAUSE
                ;
                INC PLAYING ;STOP SOUND/COUNTDOWN
                ;WAIT FOR RESTART (HELP)
                LDX #$FE
PAUSEL          LDA #$08
                JSR CHKKY ;YR/XR NOT CORRUPTED
                BNE PAUSEL
                DEC PLAYING ;=0,RESTORE VOLUME ETC
                ;
RDKEX           JMP MOVMNT ;MOVE/DISP/SCRLL
                ;+EXIT
                ;
JOYSTICK        ;READ JOYSTICK - SHIP CONTROLS
                ;
                ;LEFT
                LDA #$FF
                STA $FD30 ;KEYBOARD OFF
                LDA #$FB
                STA $FF08 ;SELECT PORT 1
                LDA $FF08
                AND #$04 ;LEFT
                BNE JRDKRT
                DEC SHPOR
                JMP JFIXSHP
                ;
                ;RIGHT
JRDKRT          LDA $FF08
                AND #$08
                BNE JRDKTH
                INC SHPOR
                ;
                ;KEEP SHPOR IN RANGE
JFIXSHP         LDA SHPOR
                AND #31
                STA SHPOR
                ;+DO DATA
                JSR DOSHBF
                ;
                ;THRUST (BUTTON)
JRDKTH          LDA $FF08
                AND #$40
                BEQ JRDKTH2
                ;
                ;TURN OFF THRUST SOUND
JTHRSNDOFF      AND #0
                STA SHIFT
                BEQ JRDKSHD ;JMP
                ;
                ;DO THRUST & SOUND
JRDKTH2         LDA CANTH
                BNE JTHRSNDOFF
                LDA #1
                STA SHIFT
                JSR THRUST
                JSR DCFUEL
                ;
                ;SHIELD (BACK)
JRDKSHD         AND #0
                STA SHST
                LDA CANTH
                BNE JRDKFIR
                LDA $FF08
                AND #$02
                BNE JRDKFIR
                INC SHST
                JSR DCFUEL
                ;TBEAM ETC ELSEWHERE
                ;
                ;FIRE (FORWARD)
JRDKFIR         LDA $FF08
                AND #$01
                BEQ JFIRCHK
                STA RETFLG
                BNE RDKFRZ ;JMP
JFIRCHK         LDX RETFLG
                BEQ CHFRZJ
                STX FIRFLG
                AND #0
                STA RETFLG
CHFRZJ          JMP RDKFRZ ;CHECK FOR FREEZE AND CONTINUE
                ;
                ;
                ;PLOT WHOLE PLAY AREA (AT START)
WHOLED          JSR UNCOMP
                JSR BOTSCP
WHLDSP          DEX
                BMI WHLDX
                SEC
                LDA DAT1
                SBC #40
                STA DAT1
                BCS WHLDS1
                DEC DAT1+1
WHLDS1          JSR CLRIT
                JMP WHLDSP
WHLDX           JMP GATES

                ;
                ; PUT SCORE BOX ON SCREEN
                ;
SCRBOX
                ;
                ;SET COLOUR TO WHITE (MCM ON)
                LDA #SCRCOL
                LDY #$78 ;`40*3
SCBXL1          STA $07FF,Y
                DEY
                BNE SCBXL1
                ;
                ;TOP LINE
                LDA #$1F
                STA $0C00
                LDY #$26 ;=`38
                LDA #$1C
SCBXL2          STA $0C00,Y
                DEY
                BNE SCBXL2
                ;
                ;MIDDLE LINE
                LDY #$2A ;=`42 (2 CHR OVERLAP)
SCBXL3          LDA MIDSDT-1,Y
                STA $0C26,Y
                TAX
                LDA COLTAB,X
                STA $0826,Y ;SOME CHRS WONT BE MC
                DEY
                BNE SCBXL3
                ;
                ;BOTTOM LINE
                LDA #$25
                LDY #$26 ;=`38
SCBXL4          STA $0C50,Y
                DEY
                BNE SCBXL4
                LDA #$24
                STA $0C77
                LDA #$30 ;LAST ZERO OF SCORE
                STA $0C75
                ;

                ;
                ; Print the INVULN! message if we are invulnerable
                ;
                CLC
                LDY #SHIPCOL
                LDA CHEAT_MODE
                AND #CHEAT_INVULNERABLE
                BNE INVULNERABLE_ON
                LDA #$1E
                LDY #SCRCOL
                SEC
INVULNERABLE_ON LDX #[INVULN_MES_LEN - 1]
CHK_CHEAT_IV_M0 BCS CHK_CHEAT_IV_M1
                LDA INVULN_MES,X
CHK_CHEAT_IV_M1 STA INVULN_MES_POS,X
                DEX
                BPL CHK_CHEAT_IV_M0
                TYA
                LDX #[INVULN_MES_LEN - 1]
CHK_CHEAT_IV_M2 STA [INVULN_MES_POS - $0400],X
                DEX
                BPL CHK_CHEAT_IV_M2

                RTS

INVULN_MES_POS  EQU SCREEN_ORIGIN+[1*40]+24
INVULN_MES      DV.B ASCII2CBM 'I, 'N, 'V, 'U, 'L, 'N, '!
INVULN_MES_END
INVULN_MES_LEN  EQU INVULN_MES_END - INVULN_MES


                ;
MIDSDT          ;DATA IN DECIMAL
                BYTE 27
                BYTE 29,30,30,6,21,5,12,30,30,30
                BYTE 30,30,30,30,30,30,30,12,9,22
                BYTE 5,19,30,30,30,30,30,30,30,30
                BYTE 30,30,30,19,3,15,18,5,30,38
                BYTE 35
                ;
                ;
                ;GENERAL STUFF
PRINT_LIVES     LDA CHEAT_MODE
                AND #CHEAT_INFINITE_LIVES
                BNE DC_LVES_INF
                LDA #SCRCOL
                JSR SET_LIVES_COLR
                LDA LIVES
                JMP PLIV0

DCLVES          LDA CHEAT_MODE
                AND #CHEAT_INFINITE_LIVES
                BEQ DC_LVES_NORMAL

DC_LVES_INF     LDA #SHIPCOL
                JSR SET_LIVES_COLR
                LDY #["I" & $3F]
                LDX #["N" & $3F]
                LDA #["F" & $3F]
                BNE PLIV2 ; JMP

DC_LVES_NORMAL  LDA LIVES
                BEQ NMRLVS
                SED
                SEC
                SBC #1
                JMP PLIVES
NMRLVS          LDX #255
                RTS
                ;
INLVES          LDA LIVES
                SED
                CLC
                ADC #1
                ;
PLIVES          CLD
                STA LIVES
PLIV0           JSR MK2DNUM
                BCS PLIV1
                LDY #$25
PLIV1           LDA #$25
PLIV2           STY $0C62 ;UNDER 'I' OF 'LIVES'
                STX $0C63 ;COLOURS ALREADY OK
                STA $0C64
                RTS
SET_LIVES_COLR  STA $0862
                STA $0863
                STA $0864
                RTS

                ;
                ;
                ;SEPARATE 2-DIGIT NUMBER IN AR INTO XR/YR
                ;ON RETURN:CARRY CLR LEADING DIGIT (YR)='0'
MK2DNUM         TAX
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                CLC
                BEQ MK2DN1
                SEC
MK2DN1          ORA #$30
                TAY
                TXA
                AND #$0F
                ORA #$30
                TAX
                RTS
                ;
                ;
ADDSCR          ;AMOUNT TO ADD IN AR
                TAY
                LDA SCORE+1
                AND #$F0
                STA TEMP1
                TYA
                SED
                CLC
                ADC SCORE
                STA SCORE
                AND #0
                ADC SCORE+1
                STA SCORE+1
                TAX
                AND #0
                ADC SCORE+2
                STA SCORE+2
                CLD
                TXA
                AND #$F0
                CMP TEMP1
                BEQ NOINLV
                JSR INLVES
                ;
NOINLV          LDX #[SCORE&255]+2 ;HI-BYTE ALWAYS $00
                LDY #$0C6F&255 ;HI-BYTE ALWAYS $0C
                ;
                ;JMP PNUM (FALLS THRU ANYWAY)
                ;
PNUM            STX DAT1
                AND #0
                STA DAT1+1
                STY DAT2
                LDA #$0C
                STA DAT2+1
                ;
                LDA #2
                STA TEMP1
                LDY #0
                LDX #0 ;LEADING ZEROS FLAG
PSCLP           LDA (DAT1),Y
                PHA
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                JSR PRNTNM
                PLA
                AND #$0F
                JSR PRNTNM
                DEC DAT1 ;ALWAYS ON Z.P.
                DEC TEMP1
                BPL PSCLP
                RTS
                ;
                ;
                ;
PRNTNM          BNE PRNTN2
                CPX #0
                BNE PRNTN2
                ;
PRNTN1          LDA #$25 ;UNDERLINED SPACE
                BNE PRNTN3 ;JMP PRNTN3
                ;
PRNTN2          ORA #$30
                TAX
PRNTN3          STA (DAT2),Y
                INC DAT2 ;DOESN'T REACH PAGE BNDRY
                RTS


SCRN_LOCN_FUEL  EQU $0C51
                ;
                ;
                ;
INFUEL          LDA #$11
                LDX #0
                BEQ DODF ;JMP DODF
                ;
DCFUEL          AND #0
                STA CANTH
                LDA FUEL
                ORA FUEL+1
                ORA FUEL+2
                BNE DODCFU
ZRFUEL          LDA #$30 ;'0'
                STA SCRN_LOCN_FUEL+5 ;LS DIGIT OF FUEL
                ORA #$FF
                STA NOFUEL
                STA CANTH
                RTS
                ;
                ; Decrement fuel by one -- by adding 999999 to it!
DODCFU          LDX #$99
                LDA CHEAT_MODE
                AND #CHEAT_INFINITE_FUEL
                BEQ DODCFU_NO_CHEAT
                LDX #0
DODCFU_NO_CHEAT TXA
                ; Fall through

                ; Increment fuel by amount in accumulator, adding the X register to the high bytes
DODF            SED
                CLC
                ADC FUEL
                STA FUEL
                TXA
                ADC FUEL+1
                STA FUEL+1
                TXA
                ADC FUEL+2
                STA FUEL+2
                CLD
                ORA FUEL+1
                ORA FUEL
                BEQ ZRFUEL
                ;
PRFUEL          LDX #SCRCOL
                LDA CHEAT_MODE
                AND #CHEAT_INFINITE_FUEL
                TAY
                BEQ PRFUEL_COLOUR
                LDX #SHIPCOL
PRFUEL_COLOUR   TXA
                LDX #2
PRFUEL_COLOUR1  STA [SCRN_LOCN_FUEL + 3 - $0400],X
                DEX
                BPL PRFUEL_COLOUR1
                TYA
                BEQ PRFUEL_ACTUAL

                LDX #2
                LDA #$25 ;UNDERLINED SPACE
PRFUEL_INF1     STA SCRN_LOCN_FUEL,X
                DEX
                BPL PRFUEL_INF1
                LDA #["I" & $3F]
                STA SCRN_LOCN_FUEL+3
                LDA #["N" & $3F]
                STA SCRN_LOCN_FUEL+4
                LDA #["F" & $3F]
                STA SCRN_LOCN_FUEL+5
                RTS

PRFUEL_ACTUAL   LDX #[FUEL&255]+2
                LDY #<SCRN_LOCN_FUEL
                JMP PNUM
                ;
                ;
                ;PRINTING SUBROUTINES
                ;
PRINT           STA (DAT2),Y
                CMP #$30
                BCC NOTNUM
                CMP #$3A
                BCS NOTNUM
                LDA #NUMCOL
                BNE PDOCOL ;JMP
NOTNUM          LDA TEMPX
PDOCOL          STA (CL1),Y
                ;JMP BPMSPTS-FALLS THRU
                ;
BPMSPTS         INC DAT2
                INC CL1
                BNE PRMESBA
                INC DAT2+1
                INC CL1+1
PRMESBA         RTS
                ;
                ;
                ;PRINT MESSAGE FROM XR
                ;MESSAGES START @ MST
                ;DAT2 USED FOR SCREEN
                ;CL1 USED FOR COLOUR
                ;
PRMESS          LDA MST,X
                BEQ PRMES1
                STA DAT2+1
                SEC
                SBC #4
                STA CL1+1
                INX
                LDA MST,X
                STA DAT2
                STA CL1
                DEX
                ;
PRMES1          INX
                INX
                LDA MST,X
                STA TEMPX ;COL.
                INX
                ;
                LDY #0
                ;
PRMESL          LDA MST,X
                BEQ PRMESE
                JSR PRINT
                INX
                BNE PRMESL
                ;
PRMESE          RTS
                ;
                ;
PRMIMESS        LDX #MIOFF
                BEQ PRMESS ;JMP
                ;
PRCMMESS        LDX #CMOFF
                BNE PRMESS ;JMP
                ;
PRPDMESS        LDX #PDOFF
                BNE PRMESS ;JMP
                ;
PRMISNUM        LDA MISSN
                JSR MK2DNUM
                TYA
                LDY #1
                STA (DAT2),Y
                LDA #NUMCOL
                STA (CL1),Y
                TXA
                INY
                STA (DAT2),Y
                LDA #NUMCOL
                STA (CL1),Y
                LDY #4
PMSNML          JSR BPMSPTS
                DEY
                BNE PMSNML
PRTITE          RTS
                ;
                ;
                ;PRINT TITLE PAGE
                ;DAT2 USED FOR SCREEN
                ;CL1 USED FOR COLOUR
                ;
PRTITLES        LDX #0
PRTITIN         LDA MS2ST,X
                CMP #$FF
                BEQ PRTITE
                STA DAT2+1
                SEC
                SBC #4
                STA CL1+1
                INX
                LDA MS2ST,X
                STA DAT2
                STA CL1
                ;
                INX
                LDA MS2ST,X
                STA TEMPX ;COL.
                INX
                ;
                LDY #0
                ;
PRTITL          LDA MS2ST,X
                BEQ PRTITLP
                JSR PRINT
                INX
                BNE PRTITL
                ;
PRTITLP         INX
                BNE PRTITIN ;JMP
                ;
                ;
                ;
RANDOM          LDA RANNUM
                EOR $FF1E
                CLC
                ADC $FF02
                STA RANNUM
                RTS
                ;HYPERSPACE SUBROUTINE
                ;
CALCCHAR        CLC
                ADC TEMPX
                STX NEWTEMP
                SEC
                SBC NEWTEMP
                RTS
                ;
                ;
                ;DO HYPERSPACE:SHIP & BALL
                ;UNPLOT THEM (+ROPE) BEFORE CALLING
                ;
HYPSPC          LDA SHPX+1
                STA CORD1
                LDA SHPY+1
                STA CORD1+1
                JSR SETCRD
                LDA SC1
                STA DAT1
                ;STA SC1 ^
                STA CORD3
                STA CL1
                LDA SC1+1
                STA DAT1+1
                ;STA SC1+1 ^
                SEC
                SBC #4
                STA CORD3+1
                STA CL1+1
                LDA SC1
                SEC
                SBC #205 ;=5*40+5
                STA DAT2
                STA CH2
                STA CORD2
                STA CORD1
                LDA SC1+1
                SBC #0
                STA DAT2+1
                STA CH2+1
                SEC
                SBC #4
                STA CORD2+1
                STA CORD1+1
                ;
                LDX #BLHYCL ;LOAD HYP COL
                LDY #1
                LDA PICKUP
                BNE BALLPT
                LDX #SHHYCL ;SHIP HYP COL
                LDY #0
BALLPT          STX TEMP2
                STY TEMPY
                TYA
                BNE BLLPT1
                JMP HYPNBL
                ;
BLLPT1          LDA LOADX+1 ;DO BALL DATA PTRS
                STA CORD1
                LDA LOADY+1
                STA CORD1+1
                JSR SETCRD
                LDA SC1
                ;STA SC1 ^
                STA CL1
                LDA SC1+1
                ;STA SC1+1 ^
                SEC
                SBC #4
                STA CL1+1
                LDA SC1
                SEC
                SBC #205
                STA CH2
                STA CORD1
                LDA SC1+1
                SBC #0
                STA CH2+1
                SEC
                SBC #4
                STA CORD1+1
                ;
HYPNBL          LDX #0
HYPLA           STX TEMPX
                ;NEW CHAR
HYPLA1          LDY YOFF1,X
                LDA #SHHYCL
                STA (CORD2),Y
                LDA TEMP2
                STA (CORD1),Y ;COLS
                LDA #186
                JSR CALCCHAR
                STA (DAT2),Y
                STA (CH2),Y
                ;
                LDY YOFF2,X
                LDA #SHHYCL
                STA (CORD3),Y
                LDA TEMP2
                STA (CL1),Y ;COLS
                LDA #201
                JSR CALCCHAR
                STA (DAT1),Y
                STA (SC1),Y
                ;
                LDY YOFF3,X
                LDA #SHHYCL
                STA (CORD3),Y
                LDA TEMP2
                STA (CL1),Y ;COLS
                LDA #191
                JSR CALCCHAR
                STA (DAT1),Y
                STA (SC1),Y
                ;
                LDY YOFF4,X
                LDA #SHHYCL
                STA (CORD2),Y
                LDA TEMP2
                STA (CORD1),Y ;COLS
                LDA #196
                JSR CALCCHAR
                STA (DAT2),Y
                STA (CH2),Y
                ;
                ;UPDATE
HYPLB           DEX
                BPL HYPLA1
                ;
HYPLC           LDY #HYPDELAY
                JSR PAUSE
                LDX TEMPX
                INX
                CPX #5
                BCC HYPLA
                ;
                DEX
HYPLD           STX TEMPX
                ;
HYPLD1          LDY YOFF1,X
                LDA #186-1
                JSR CALCCHAR
                CMP #186
                BCS OK1
                LDA #$20 ;SPACE
OK1             STA (DAT2),Y
                STA (CH2),Y
                ;
                LDY YOFF2,X
                LDA #201-1
                JSR CALCCHAR
                CMP #201
                BCS OK2
                LDA #$20 ;SPACE
OK2             STA (DAT1),Y
                STA (SC1),Y
                ;
                LDY YOFF3,X
                LDA #191-1
                JSR CALCCHAR
                CMP #191
                BCS OK3
                LDA #$20 ;SPACE
OK3             STA (DAT1),Y
                STA (SC1),Y
                ;
                LDY YOFF4,X
                LDA #196-1
                JSR CALCCHAR
                CMP #196
                BCS OK4
                LDA #$20 ;SPACE
OK4             STA (DAT2),Y
                STA (CH2),Y
                ;COLOURS DON'T MATTER
                ;
                DEX
                BPL HYPLD1
                ;
HYPLF           LDY #HYPDELAY
                JSR PAUSE
                ;
                LDX TEMPX
                DEX
                BPL HYPLD
                ;
                ;RESTORE COLOUR
                LDX #4
                LDA #SHIPCOL+8
RPHCOL          LDY YOFF1,X
                STA (CORD2),Y
                STA (CORD1),Y
                LDY YOFF4,X
                STA (CORD2),Y
                STA (CORD1),Y
                LDY YOFF2,X
                STA (CORD3),Y
                STA (CL1),Y
                LDY YOFF3,X
                STA (CORD3),Y
                STA (CL1),Y
                DEX
                BPL RPHCOL
                ;
                ;FINISHED
                ;
                RTS
                ;
                ;OFFSETS
YOFF1           BYTE 165,125,85,45,5
YOFF2           BYTE 1,2,3,4,5
YOFF3           BYTE 40,80,120,160,200
YOFF4           BYTE 204,203,202,201,200
                ;
                ;
                ;'UNCOMP/DRAWSC'
                ;;UNCOMPRESS SCAPE DATA INTO
                ;LHS & RHS BUFFERS
                ;
                ;
UNCOMP          LDX #0
                ;STX SCPPTR (DONE BELOW [LDX #0])
                STX VDPH ;HOW FAR DOWN SCAPE
                STX VDPL
                STX HDPRT
                STX HDPLT
                STX DTPTL
                STX DTPTR
                STX LDIREC
                STX RDIREC
                ;
                ;
                DEX ;=$FF :SCPPTR-HOW FAR DOWN SCREEN
                JSR GETLTU
                JSR GETRTU
                ;
                ;MAIN LOOP (LEFT ALWAYS FIRST)
                ;
                ;
                ;CHECK WHETHER ON PLAY AREA
UNCOMPLP        LDA VDPL
                CMP TPLAYL
                LDA VDPH
                SBC TPLAYH
                BCC UNCPADDL
                ;
                INX
                ;
UNCPADDL        LDA LDIREC
                CLC
                ADC HDPLT
                STA HDPLT ;NO CARRY
                LDA CRLVD
                BNE CHKLS2
                JSR GETLTU
                JMP UNCPADDL
                ;
CHKLS2          LDY #0
                LDA LDIREC
                BEQ UNCM1A
                CMP #$02
                BEQ UNCM1A
                CMP #$FE
                BNE UNCM1AA ;USE PREVIOUS END TYPE
                LDY #2
UNCM1A          STA OLDLDC
                ;
UNCM1AA         TXA
                BMI UNCMX1 ;BRANCH IF NOT ON SCREEN
                LDA OLDLDC
                STA LEBUF,X
                TYA
                CLC
                ADC HDPLT
                STA LLBUF,X
                ;
UNCMX1          DEC CRLVD
                BNE UNCPADDR
                JSR GETLTU
                ;
UNCPADDR        LDA RDIREC
                CLC
                ADC HDPRT
                STA HDPRT ;NO CARRY
                LDA CRRVD
                BNE CHKRS2
                JSR GETRTU
                JMP UNCPADDR
                ;
CHKRS2          LDY #0
                LDA RDIREC
                BEQ UNCM1B
                CMP #$02
                BEQ UNCM1B
                CMP #$FE
                BNE UNCM1BB ;USE PREVIOUS END TYPE
                LDY #2
UNCM1B          STA OLDRDC
                ;
UNCM1BB         TXA
                BMI UNCMX2 ;BRANCH IF NOT ON SCREEN
                LDA OLDRDC
                STA REBUF,X
                TYA
                CLC
                ADC HDPRT
                STA RLBUF,X
                ;
UNCMX2          DEC CRRVD
                BNE UNCPBP
                JSR GETRTU
                ;
UNCPBP          INC VDPL
                BNE UNCMXL
                INC VDPH
                ;
                ;...LOOP
UNCMXL          CPX #22
                BEQ UNCOMPEXIT
                JMP UNCOMPLP
;UNCOMPEXIT      RTS ; Share with RTS of GETLTU below!
                ;
                ;
GETLTU          LDY DTPTL
                LDA (LEDSTL),Y
                STA CRLVD
                LDA (LEDRCL),Y
                STA LDIREC
                INY
                STY DTPTL
UNCOMPEXIT      RTS

GETRTU          LDY DTPTR
                LDA (REDSTL),Y
                STA CRRVD
                LDA (REDRCL),Y
                STA RDIREC
                INY
                STY DTPTR
                RTS
                ;
                ;
                ;
                ;DRAW VARIOUS SCAPE EDGES...
                ;
                ;
DRAWSC          ;
BOTSCP          ;SET UP SCREEN POINTER
                LDX #$0FC0&255
                LDY #$0FC0/256
                STX DAT1
                STY DAT1+1
                LDX #21 ;BOTTOM LINE
                ;
                ;CLEAR LINE FIRST
CLRIT           LDY #39
                LDA #SPACE
CLRLN1          STA (DAT1),Y
                DEY
                BPL CLRLN1
                ;
                ;DO LHS BEFORE RHS
BOTHSC          LDA LLBUF,X
                SEC
                SBC PLAYX
                BCC RTOPSC
                BEQ RTOPSC
                TAY
                DEY
                CPY #41
                BCC CHKTS1
                LDY #39
                BNE DOSEC ;JMP DOSEC
CHKTS1          CPY #40
                BNE LTLSEC
                DEY
                BNE LTPSEC ;JMP LTPSEC
                ;
LTLSEC          LDA LEBUF,X
                BEQ DOSEC
                CMP #68
                BEQ LTLS2
                CMP #$FE
                BEQ LTLS1
                LDA #60
                BNE LTLS2 ;JMP LTLS2
LTLS1           LDA #66
LTLS2           STA (DAT1),Y
                DEY
                BMI RTOPSC
                ;
LTPSEC          LDA LEBUF,X
                BEQ DOSEC
                CMP #68
                BEQ DOSEC
                CMP #$FE
                BEQ LTPS1
                LDA #59
                BNE LTPS2 ;JMP LTPS2
LTPS1           LDA #65
LTPS2           STA (DAT1),Y
                DEY
                BMI RTOPSC
                ;
DOSEC           LDA #67
DOSEC1          STA (DAT1),Y
                DEY
                BPL DOSEC1
                ;
                ;
RTOPSC          LDA #$80 ;**PLAY WIDTH**
                SEC
                SBC PLAYX
                SBC RLBUF,X ;CARRY ALREADY SET
                ;THIS GIVES Y OFFSET TO DISPAY
                ;LEFTMOST CHAR OF RIGHT 'BAR'
                TAY
                BCS CHKBS1 ;+VE
                CPY #$FF ;=-1
                BNE CHKBS2 ;<-1
                INY ;LDY #0
                BEQ RTSCS ;JMP RTSCS
CHKBS2          LDY #0
                BEQ DOSEC2 ;JMP DOSEC2
CHKBS1          CPY #$28
                BCS ENDDLN ;>$27
                ;
RTSCF           LDA REBUF,X
                BEQ DOSEC2
                CMP #69
                BEQ RTSC3
                CMP #$FE
                BEQ RTSC2
                LDA #61
                BNE RTSC3 ;JMP RTSC3
RTSC2           LDA #63
RTSC3           STA (DAT1),Y
                INY
                CPY #40
                BCS ENDDLN
                ;
RTSCS           LDA REBUF,X
                BEQ DOSEC2
                CMP #69
                BEQ DOSEC2
                CMP #$FE
                BEQ RTSC4
                LDA #62
                BNE RTSC5 ;JMP RTSC5
RTSC4           LDA #64
RTSC5           STA (DAT1),Y
                INY
                CPY #40
                BCS ENDDLN
                ;
DOSEC2          LDA #67
DSEC2A          STA (DAT1),Y
                INY
                CPY #40
                BCC DSEC2A
                ;
ENDDLN          RTS
                ;
                ;
                ;
                ;DO VERTICAL SIDES...
LHSSCP          LDA PLAYX
                STA TEMP1
                LDX #$0C78&255
                LDA #$0C78/256
                STX DAT1 ;SCR
                STA DAT1+1
                BNE DOVERT ;JMP DOVERT
                ;
RHSSCP          LDA PLAYX
                CLC
                ADC #39
                STA TEMP1
                LDX #$0C9F&255
                LDA #$0C9F/256
                STX DAT1
                STA DAT1+1
                ;
DOVERT          LDX #0
DVLP            LDY #0
                LDA #SPACE
                STA (DAT1),Y
                ;CHECK LHS FIRST
CHKLHS          LDA LLBUF,X
                SEC
                SBC TEMP1
                BCC CHKRHS
                BEQ CHKRHS
                CMP #3
                BCS LMIDBT ;>2
                CMP #1
                BEQ LEND1
                ;MUST BE 2 SO DROP THROUGH
                ;
                LDA LEBUF,X
                BEQ LMIDBT
                CMP #2
                BEQ LPEND1
                LDA #65
                BNE LSTORE ;JMP LSTORE
LPEND1          LDA #59
                BNE LSTORE ;JMP LSTORE
                ;
LEND1           LDA LEBUF,X
                BEQ LMIDBT
                CMP #2
                BEQ LEND2
                LDA #66
                BNE LSTORE ;JMP LSTORE
LEND2           LDA #60
                BNE LSTORE ;JMP LSTORE
                ;
LMIDBT          LDA #67
LSTORE          STA (DAT1),Y
                ;
CHKRHS          LDA #$80 ;**PLAY WIDTH**
                SEC
                SBC RLBUF,X
                SBC TEMP1
                BEQ REND1
                BCS VLDSLP
                CMP #$FF ;=-1
                BNE RMDBIT
                ;
                LDA REBUF,X
                BEQ RMDBIT
                CMP #2
                BEQ RPEND1
                LDA #64
                BNE RSTORE ;JMP RSTORE
RPEND1          LDA #62
                BNE RSTORE ;JMP RSTORE
                ;
REND1           LDA REBUF,X
                BEQ RMDBIT
                CMP #2
                BEQ REND2
                LDA #63
                BNE RSTORE ;JMP RSTORE
REND2           LDA #61
                BNE RSTORE ;JMP RSTORE
                ;
RMDBIT          LDA #67
RSTORE          STA (DAT1),Y
                ;
VLDSLP          CLC
                LDA DAT1
                ADC #40
                STA DAT1
                STA DAT2
                BCC VLDSL1
                INC DAT1+1
                INC DAT2+1
                ;
VLDSL1          INX
                CPX #22
                BCS EVLDIS
                JMP DVLP
                ;
EVLDIS          RTS
                ;
                ;
                ;
                ;DO GATES (SCAPES 3-5)
GATES           LDX SCAPNO
                CPX #3
                BCC EVLDIS
                DEX
                DEX
                DEX ;X=WHICH GATE
                LDA GATCNT
                BPL CLOSING
                LDA #0
                SEC
                SBC GATCNT
CLOSING         CMP MAXNOPOS,X
                BCC GATL1
                LDA MAXNOPOS,X
GATL1           STA GATPOS
                ;
                LDA GATYBTL,X
                SEC
                SBC TPLAYL
                BMI EVLDIS
                STA CORD2+1
                LDA GATYBTH,X
                SBC TPLAYH
                BNE EVLDIS
                STA CORD2 ;=0
                LDA GATHGT,X
                STA NEWTEMP
                LDA SMTHGLDR,X
                STA SGLDRV
                ;
GATELOOP        LDX CORD2+1 ;X=WHICH LINE
                CPX #22 ;NO OF PLAY ROWS
                BCS LOOPRND
                LDA GATPOS
                LSR ; = LSR A ;/2
                PHP ;LSB=END TYPE
                TAY ;Y=WHICH STAGE OF OPENING
                LDA SCAPNO
                CMP #3
                BNE GPGD4X
                LDA #RGTES3
                PHA
                LDA S3GATL,Y
                BNE GTL2 ;JMP
GPGD4X          CMP #5
                BEQ GPGD5
                LDA S4GATR,Y
                PHA
                LDA S4GATL,Y
                BNE GTL2 ;JMP
GPGD5           LDA S5GATR,Y
                PHA
                LDA S5GATL,Y
                BNE GTL2
                LDA #$58
                SEC
                BCS GTL2A ;JMP
                ;
GTL2            CLC
GTL2A           STA LLBUF,X
                PLA
                STA RLBUF,X
                LDA SGLDRV
                BCS FLIPEND ;FORCE END TYPE
                PLP ;END TYPE
                PHP
                BCC NORMEND
FLIPEND         EOR #68
NORMEND         STA LEBUF,X
                PLP
                BCC NRMEND2J
                LDA SCAPNO
                CMP #3
                BNE GTDORGT
NRMEND2J        AND #0
                BEQ NRMEND2 ;JMP
GTDORGT         LDA #69
NRMEND2         STA REBUF,X
                ;
                JSR STCCRD
                LDA #LINBUF&255
                STA DAT1
                LDA #LINBUF/256
                STA DAT1+1
                LDX CORD2+1
                JSR CLRIT ;DRAW HORIZ SCAPE LINE IN LINBUF
                LDY #$27
COPLIN          LDA (DAT1),Y
                STA (SC1),Y
                DEY
                BPL COPLIN
                ;
LOOPRND         DEC CORD2+1
                DEC NEWTEMP
                BNE GATELOOP
                ;
                RTS ;EXIT
                ;
MAXNOPOS        BYTE 15,7,24
GATYBTL         BYTE $DD,$0C,$1D
GATYBTH         BYTE 0,1,1
GATHGT          BYTE 4,5,4
SMTHGLDR        BYTE 0,0,68
                ;
RGTES3          EQU $25 ;CONSTANT
S3GATL          BYTE $5B,$5A,$59,$58,$57,$56,$55,$54
S4GATR          BYTE $2E,$2D,$2C,$2B
S4GATL          BYTE $52,$51,$50,$4F,$4E
S5GATR          BYTE $29,$28,$27,$26,$25,$24,$23
                BYTE $22,$21,$20,$1F,$1E,$1D
S5GATL          BYTE 0,0,0,0,0,0,0
                BYTE 0,0,$59,$5A,$5B,$5C
                ;




;
; File X2.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;'DOTS'
                ;PLOT/UNPLOT POINTS...
                ;BULLETS (YOURS/GUNS')
                ;STARS,EXPLOSIONS
                ;
FRSTDT          EQU 154 ;FIRST BULLET CHAR
                ;
                ;
UNDOT           LDX #0
UNDTL1          LDA BULLFG,X
                BPL NORBLL
                JSR UNPLBL
NORBLL          INX
                CPX #32
                BCC UNDTL1
                RTS
                ;
                ;
                ;UNPLOT BULLET
                ;
UNPLBL          LDA BLSPL,X
                STA DAT1
                STA DAT2
                LDA BLSPH,X
                STA DAT1+1
                SEC
                SBC #4
                STA DAT2+1
                ;
                LDY #0
                LDA BLCHS,X
                STA (DAT1),Y
                TAY
                LDA COLTAB,Y
                LDY #0
                STA (DAT2),Y
                ;
                LDA BULLFG,X
                AND #$7F
                STA BULLFG,X
                RTS
                ;
                ;
DOTS            JSR STARS
                ;
                LDX #31
                ;
BULLLP          LDY BULLFG,X
                BEQ NXTPLT
                DEY
                TYA
                STA BULLFG,X
                ;
                JSR ADBLVL
                JSR PRXTST
                LDA BULLFG,X
                BEQ NXTPLT
                ;
                JMP PLTPNT
                ;
NXTPLT          DEX
                BPL BULLLP
                RTS
                ;
DOTMASKS        BYTE $C0,$C0,$30,$30,$0C,$0C,$03,$03
DOTDAT          BYTE $80,$40,$20,$10,$08,$04,$02,$01
                ;
PLTPNT          ;MY ROUTINE
                JSR STCD2
                LDY #0
                STY COLFLG
                LDA (DAT1),Y
                STA BLCHS,X
                LDY #SHIPCOL
                CMP #59 ;1ST SCAPE CHAR
                BCC PLTPT1
                CMP #72 ;(LAST SCAPE CHAR)+1
                BCS PLTPT1
                LDY #SHIPCOL+8
PLTPT1          STY DTCLMD+1
                LDY #FRSTDT ;1ST BULL CHR
                STY TEMP1
                JSR SETCHR ;DOESN'T CORRUPT XR
                LDA BLXLO,X
                ROL ; = ROL A
                ROL ; = ROL A
                ROL ; = ROL A
                ROL ; = ROL A
                AND #$07
                TAY
                LDA DOTDAT,Y
                PHA
                LDA DOTMASKS,Y
                PHA
                LDA BLYLO,X
                ROL ; = ROL A
                ROL ; = ROL A
                ROL ; = ROL A
                ROL ; = ROL A
                AND #$07
                TAY
                PLA
                AND (CH2),Y
                BEQ PLPT2
                INC COLFLG
PLPT2           PLA
                ORA (CH2),Y
                STA (CH2),Y
                ;
                ;PLOT ON SCREEN
                ;
                TXA
                CLC
                ADC #FRSTDT ;1ST BULL CHR
                LDY #0
                STA (DAT1),Y
DTCLMD          LDA #SHIPCOL ;SMC
                STA (DAT2),Y
                ;
                LDA BULLFG,X
                ORA #$80
                STA BULLFG,X
                ;
                LDA BLTYPE,X
                BEQ CHKSHB ;CHECK FOR SHIP BULLET HIT OBJECT
                STX TEMPY
TSTCDT          LDX TEMPY
                LDA COLFLG
                BEQ BLCRON
REMBUL          LDX TEMPY
RMBUL2          AND #0
                STA BULLFG,X
                JSR UNPLBL
BLCRON          JMP NXTPLT
                ;
CHKSHB          STX TEMPY
                TXA
                CLC
                ADC #FRSTDT
                STA CHAR
                LDX SCAPNO ;CHECK THROUGH ALL GUNS ON THIS SCAPE
                LDA GUNTOT,X
                TAX
CKSHBL          DEX
                BMI TSTCDT
CKSHB1          JSR PLOTSUB
                LDX TEMPX ;SAVED IN PLOTSUB
                BCS CKSHBL ;BRANCH IF OBJECT NOT ON SCREEN
                LDA GSTAT
                BEQ CKSHBL ;BRANCH IF OBJECT DOESNT EXIST (DESTROYED)
CKSHB3          LDY NEWTEMP
                LDA (DAT2),Y
                TAY
                LDA (SC1),Y
                CMP CHAR
                BEQ CKSHB5
                DEC NEWTEMP
                BPL CKSHB3
                BMI CKSHBL ;JMP
                ;
CKSHB5          LDY GOR
                CPY #5 ;BALL & PEDISTAL
                BEQ REMBUL
                BCS NOTDES
                AND #0 ;DESTROY
                STA GUNFGS,X
                CPY #4 ;FUEL
                BEQ BLSTFU
                ;
                LDA #1 ;GUN
                JSR PREEXP
                LDA #GUNSCR
                JSR ADDSCR
                JMP REMBUL
                ;
BLSTFU          LDA #1 ;FUEL
                JSR PREEXP
                LDA #BFLSCR
                JSR ADDSCR
                JMP REMBUL
                ;
                ;
NOTDES          CPY #7 ;LHS DOME
                BEQ DOMHIT
                CPY #8 ;RHS DOME
                BNE TRYRCT
                ;
DOMHIT          LDA GATCNT
                BNE DMHTL
                ORA #$7F ;=$7F,($FF BELOW)
DMHTL           ORA #$80 ;OPENING
                STA GATCNT
                LDA #4
                JSR PREEXP
                JMP REMBUL
                ;
TRYRCT          ;MUST BE REACTOR
                LDA #4
                JSR PREEXP
                ;
                JSR RANDOM
                AND #31
                CLC
                ADC REACDM
                STA REACDM
                STA REACNT
                BCC NORKLL
                ;
                ;REACTOR DAMAGED TO DEATH
                LDA CNTDWN
                BPL NORKLL ;ALREADY DEAD
                ORA #$FF
                STA REACNT
                LDA #10
                STA CNTDWN
                LDA #SECLEN
                STA FLSHCNT
                ;
NORKLL          JMP REMBUL
                ;
                ;
                ;
PRXTST          LDA SHPX+1
                SEC
                SBC #7 ;MODAL VALUE
                STA CORD2
                LDA LOADX+1
                SEC
                SBC #BWID
                STA CORD3
                LDA BLXHI,X
                SEC
                SBC #4
                STA CORD1
                LDA BLXLO,X
                LDY #3
BXCDC1          ASL ; = ASL A
                ROL CORD1
                DEY
                BNE BXCDC1
                BCC BXCDC2
                RTS
                ;
BXCDC2          LDA SHPY+1
                SEC
                SBC #7 ;MODAL VALUE
                STA CORD2+1
                LDA LOADY+1
                SEC
                SBC #BHGT
                STA CORD3+1
                LDA BLYHI,X
                STA CORD1+1
                LDA BLYLO,X
                LDY #3
BYCDC1          ASL ; = ASL A
                ROL CORD1+1
                DEY
                BNE BYCDC1
                ;
                LDA BLTYPE,X
                CMP #3
                BNE BALPRX ;ONLY CHECK GUN BULLETS
                ;
                ;CHECK FOR SHIP
                LDA SHST
                BNE BALPRX
                ;
                LDY TEMPY
                LDA CORD1
                SEC
                SBC CORD2
                BCC BALPRX
                CMP #15 ;MODAL VALUE
                BCS BALPRX
                ;
                LDA CORD1+1
                SEC
                SBC CORD2+1
                BCC BALPRX
                CMP #15 ;MODAL VALUE
                BCS BALPRX
                LDA #$80
                ORA COLDET
                STA COLDET
                ;
                ;CHECK FOR BALL
BALPRX          LDA BLTYPE,X
                BEQ BALPR1A
                CMP #3
                BNE NOTPRX
BALPR1A         LDA PICKUP
                BEQ NOTPRX
                LDA CORD1
                SEC
                SBC CORD3
                BCC NOTPRX
                CMP #BWID*2+1
                BCS NOTPRX
                LDA CORD1+1
                SEC
                SBC CORD3+1
                BCC NOTPRX
                CMP #BHGT*2+1
                BCS NOTPRX
                LDA #$40
                ORA COLDET
                STA COLDET
                ;
NOTPRX          RTS
                ;
                ;
ADBLVL          LDY #0
                LDA BLXV,X
                ASL ; = ASL A
                BCC ADBLV1
                DEY
                CLC
ADBLV1          ADC BLXLO,X
                STA BLXLO,X
                TYA
                ADC BLXHI,X
                STA BLXHI,X
                CMP #39
                BCC ADBLV2
                LDA BULLFG,X
                AND #$80
                STA BULLFG,X
                ;
ADBLV2          LDY #0
                LDA BLYV,X
                ASL ; = ASL A
                BCC ADBLV3
                DEY
                CLC
ADBLV3          ADC BLYLO,X
                STA BLYLO,X
                TYA
                ADC BLYHI,X
                STA BLYHI,X
                CMP #21
                BCC ADBLV4
                LDA BULLFG,X
                AND #$80
                STA BULLFG,X
                ;
ADBLV4
NOSTRS          RTS
                ;
                ;
STARS           LDA LOPCNT
                AND #1
                ORA TPLAYH
                BNE NOSTRS
                LDA TPLAYL
                BMI NOSTRS
                JSR FNDSPC
                JSR RANDOM
STRSL1          CMP #21
                BCC STRSL2
                SBC #21
                BNE STRSL1
STRSL2          STA BLYHI,Y
                JSR RANDOM
STRSL3          CMP #39
                BCC STRSL4
                SBC #39
                BNE STRSL3
STRSL4          STA BLXHI,Y
                LDA #$80
                STA BLYLO,Y
                STA BLXLO,Y
                AND #0
                STA BLYV,Y
                STA BLXV,Y
                JSR RANDOM
                AND #$3F
                STA BULLFG,Y
                AND #1
                CLC
                ADC #1
                STA BLTYPE,Y
                RTS
                ;
                ;
FNDSPC          LDY #31
FNDLP1          LDA BULLFG,Y
                BEQ FOUNDSP
                DEY
                BPL FNDLP1
                ;
                LDY #31
FNDLP2          LDA BULLFG,Y
                CMP #10
                BCC FOUNDSP
                DEY
                BPL FNDLP2
                ;
                JSR RANDOM
                AND #15
                ADC #4
                TAY
                ;
FOUNDSP         RTS
                ;
                ;
KPRANG          ASL ; = ASL A
                BCC KPRNG1
                ORA #$C0
                BNE KPRNG2 ;JMP
KPRNG1          AND #$4F
KPRNG2          ROR ; = ROR A
                RTS
                ;
                ;
PREEXP          STA CHAR
                LDX TEMPY
                LDA BLXLO,X
                STA CORD1
                LDA BLXHI,X
                STA CORD1+1
                LDA BLYLO,X
                STA CORD2
                LDA BLYHI,X
                STA CORD2+1
                LDA BLXV,X
                STA CORD3
                LDA BLYV,X
                STA CORD3+1
                ;FALLS THRU TO EXPLSN
                ;
                ;EXPLOSIONS
                ;X/Y CO-ORDS IN CORD1,2
                ;VELS IN CORD3.TYPE IN CHAR
EXPLSN          LDA #$80
                STA V2SNDI
                ;
                LDX #8
                LDA CHAR
                CMP #4 ;REACTOR
                BNE EXPLP1
                LDA #1
                STA CHAR
                LDX #3
                ;
EXPLP1          AND #0
                SEC
                SBC CORD3
                STA CORD3
                AND #0
                SEC
                SBC CORD3+1
                STA CORD3+1 ;FLIP DIRECTIONS
                ;
EXPLP2          STX TEMPX
                JSR FNDSPC
                LDA CORD1
                STA BLXLO,Y
                LDA CORD1+1
                STA BLXHI,Y
                LDA CORD2
                STA BLYLO,Y
                LDA CORD2+1
                STA BLYHI,Y
                ;
                JSR RANDOM
                LDX CHAR
                CPX #2
                BEQ EXPLP3
                ASL ; = ASL A
                BCC EXPLP4
                ORA #$E0
                BNE EXPLP5 ;JMP
EXPLP4          AND #$10
EXPLP5          ROR ; = ROR A
                ADC CORD3 ;CRY CLR ^
EXPLP3          JSR KPRANG
                STA BLXV,Y
                ;
                JSR RANDOM
                LDX CHAR
                CPX #2
                BEQ EXPLP6
                ASL ; = ASL A
                BCC EXPLP7
                ORA #$E0
                BNE EXPLP8 ;JMP
EXPLP7          AND #$10
EXPLP8          ROR ; = ROR A
                ADC CORD3+1 ;CRY CLR ^
EXPLP6          JSR KPRANG
                STA BLYV,Y
                ;
                LDA CHAR
                STA BLTYPE,Y
                ;
                LDA BULLFG,Y
                ASL ; = ASL A
                PHP
                JSR RANDOM
                AND #$3F
                PLP
                ROR ; = ROR A
                STA BULLFG,Y
                ;
                TYA
                TAX
                JSR ADBLVL
                ;
                LDX TEMPX
                DEX
                BEQ OTOFEX
                JMP EXPLP2
;OTOFEX          RTS ; Share RTS with FIRE below!
                ;
                ;
                ;FIRE (SHOOT). MAX 4 BULLS @ A TIME
                ;
FIRE            LDA SHST
                BEQ FIRE1
FIREEX          AND #0
                STA FIRFLG
OTOFEX          RTS
                ;
FIRE1           LDX BLLCNT
                LDA BLTYPE,X
                BNE MKBULL
                LDA BULLFG,X
                BNE FIREEX
                ;
MKBULL          AND #0
                STA FIRFLG
                STA BLTYPE,X
                LDA SHPX+1
                ROR ; = ROR A
                ROR ; = ROR A
                ROR ; = ROR A
                ROR ; = ROR A
                TAY
                AND #$E0
                STA BLXLO,X
                TYA
                ROL ; = ROL A
                AND #$1F
                CLC
                ADC #4
                STA BLXHI,X
                LDA SHPY+1
                ROR ; = ROR A
                ROR ; = ROR A
                ROR ; = ROR A
                ROR ; = ROR A
                TAY
                AND #$E0
                STA BLYLO,X
                TYA
                ROL ; = ROL A
                AND #$1F
                STA BLYHI,X
                ;
                LDA SHPOR
                JSR SINE
                LDA RESULT
                STA BLXV,X
                LDA SHPOR
                JSR COSINE
                AND #0
                SEC
                SBC RESULT
                STA BLYV,X
                ;
                JSR ADBLVL
                ;
                LDA #$40 ;RANGE
                STA BULLFG,X
                INX
                TXA
                AND #3
                STA BLLCNT
                ;
                ;FIRE SOUND
                LDA #2
                STA V1SNDI
                ;
                RTS
                ;






;
; File X3.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;'RASIRQ'
                ;
                ;
                ;Y-SMOOTH SCROLL TABLES
                ;
                ; Corrected values here found emprically!
                ;
                ; NB: It's important to test this on a real machine or a _good_
                ; emulator, such as YAPE (http://yape.plus4.net/).  As The Plus/4
                ; mode of the VICE editor for example does not emulate the TED
                ; chip properly when it comes to vertical scrolling.
                ;
                ; Position of "bottom of visible screen" raster interrupt
TAB1            BYTE $C3,$C1,$BF,$C5
                ; Value to push raster forward to at raster line $17
TAB2            BYTE $1D,$1B,$19,$1F
                ; Value to push raster back to at raster line $D0
TAB4            BYTE $CA,$CC,$CE,$C8



VECTORS         .WORD RESET,RESET,IRQIN

                ;DO RASTER IRQS
                ;+SCROLLING & GRAPHICS UPDATE
                ;
IRQIN           SEI
                CLD
                PHA
                TYA
                PHA
                TXA
                PHA
                STY $FF3F ;RAM SELECT

                LDA $FF09 ; Interrupt request register
                AND #$02  ; Bit 1 = raster-counter
                STA $FF09 ;CLEAR FLAG

                ;
                ; 2009-11: MY: This block of lines is just to push the video
                ;              changes we make off the side of the screen to
                ;              decrease flicker.  The JSR is original.  The
                ;              LDA/STA I added after testing on WinVICE.
                ;
                ;              Swap the commenting on the last two lines below
                ;              to show where the raster interrupts occur by
                ;              changing the border colour.
                ;
                JSR RASDELAY ;HELPS PUSH RASTER ONTO NEXT LINE
                LDA $FF1D ; vertical line
                STA DUMMY
;                STA $FF19 ;BORDER

                JMP (JMPVEC)

DUMMY           EQU $0200

                ;
                ; Process raster interrupt #1, at the bottom of the visible
                ; screen.  Exact raster line depends on y scroll position (see
                ; TAB1)
                ;
RSNM1           ;(VARIABLE POS).SCREEN OFF
                LDA #$78 ; Blank screen
                STA $FF06
                ;
                LDA #RSNM2_POS
                LDX #RSNM2&255
                LDY #RSNM2/256
                ;
                ;
IRQEX           STX JMPVEC
                STY JMPVEC+1
                STA $FF0B ; Raster interrupt register
                PLA
                TAX
                PLA
                TAY
                PLA
                RTI ;EXIT IRQ


                ;
                ; Process raster interrupt #2, at raster line $D0
                ;
RSNM2_POS       EQU $D0
RSNM2           ;40COLS/NORMAL SCRPS/SCREEN ON
                LDA #$18
                STA $FF06 ;25ROWS/SMTHYPOS0/SCREEN ON
                LDA #$98
                STA $FF07 ;40COLS
                LDX FRAMEYSMTH
                LDA TAB4,X
                STA $FF1D
                ;
                LDA MC1COL
                STA $FF16
                LDA MC2COL
                STA $FF17
                ;
                INC SNDCTR ;UPDATE SOUND TIMER

                ; If the game is on and the reactor is counting down to
                ; destruction, display the seconds left on either side of the
                ; "lives" number.
                LDA PLAYING
                BNE EXRAS2A
                LDX #$25
                LDY CNTDWN
                BMI EXRAS2
                BEQ EXRAS2A
                DEC FLSHCNT
                BNE EXRAS2XTR
                LDA #SECLEN
                STA FLSHCNT
                DEY
                STY CNTDWN
                BEQ EXRAS2A
EXRAS2XTR       TYA
                CLC
                ADC #$2F
                TAX
EXRAS2          STX $0C60
                STX $0C66 ;COL. ALREADY OK
EXRAS2A         LDA #RSNM0_POS
                LDX #RSNM0&255
                LDY #RSNM0/256
                JMP IRQEX

                ;
                ; Process raster interrupt #0, at raster line $17
                ;
RSNM0_POS       EQU $17
RSNM0           ;SCREEN ON/DO SMOOTH/DO SOUND
                LDA SMTHY
                LSR ; = LSR A
                STA FRAMEYSMTH
                TAX
                LDA SMTHX
                ORA #$90
                STA $FF07 ; Bits 0-2 = horizontal smooth scrolling position X
                LDA TAB1,X
                PHA
                LDA TAB2,X
                STA $FF1D ; Actual position of vertical scanning
                AND #$06
                STA $FF1F ; Bits 0-2: actual vertical scanning line in a character row
                ;
                ;
                ;DO MULTI-COL. FOR PLAY AREA
                LDA MCONFLG
                BNE MCLSOFF
                LDA VISFLG
                BEQ DOSOUND
                AND SHST
                BNE DOSOUND
MCLSOFF         AND #0
                STA $FF16
                STA $FF17
                ;
                ;
                ;DO SOUND
DOSOUND         LDA PLAYING
                BNE NOV2SOUND
                LDA V2SNDI
                BMI EXPLSNDINI
                BNE EXPLOSSD
THRUSTSD        LDA SHIFT
                BEQ NOV2SOUND
                LDA COLDET
                BNE NOV2SOUND
                LDA #THRSND&255
                STA $FF0F
                LDA #THRSND/256
                STA $FF10
                BNE V2SOUNDON ;JMP
                ;
NOV2SOUND       LDA $FF11
                AND #$1F
                STA $FF11
                BPL DOV1SOUND ;JMP
                ;
EXPLSNDINI      DEC V2SNDI
                LDA #EX1SND&255
                STA $FF0F
                LDA #EX1SND/256
                STA $FF10
                ;
EXPLOSSD        SEC
                LDA $FF0F
                SBC #EXSSND
                TAX
                STA $FF0F
                LDA $FF10
                AND #$03
                SBC #0
                STA $FF10
                BNE V2SOUNDON
                CPX #EX2SND
                BCC V2SOUNDON
                STA V2SNDI ;0-SEE^
                ;
                ;
V2SOUNDON       LDA $FF11
                ORA #$4F
                STA $FF11
                ;
DOV1SOUND       LDA PLAYING
                BNE NV1SNDJ
                LDA CNTDWN
                BMI DV1SNDL
                BEQ NV1SNDJ
                ;
                LDA FLSHCNT
                CMP #SECLEN-4
                BCC NV1SNDJ
                BCS BEEPON ;JMP
                ;
DV1SNDL         LDA V1SNDI
                BPL NOTPICK
PICKSOUND       CMP #$FE
                BNE NOTINIPICK
                INC V1SNDI
                AND #0
                STA SNDCTR
NOTINIPICK      LDA SNDCTR
                CMP #4
                BCC BEEPON
                CMP #6
                BCS PICKSCHK
NV1SNDJ         JMP NOV1SOUND
PICKSCHK        CMP #10
                BCC BEEPON
                INC V1SNDI
                ;
BEEPON          LDA #BEEPSND&255
                STA $FF0E
                LDA #BEEPSND/256+$C8
                STA $FF12
                JMP V1SOUNDON
                ;
                ;
NOTPICK         LDA SHST
                BEQ NOTSHIELD
                AND #0
                STA V1SNDI
                LDA #$C8
                STA $FF12
                LDX #SH1SND
                LDA SNDCTR
                AND #$02
                BEQ DOSHIELD1
                LDX #SH2SND
DOSHIELD1       STX $FF0E
                JMP V1SOUNDON
                ;
NOTSHIELD       LDA V1SNDI
                BEQ NOV1SOUND
                CMP #3
                BCS GUNFRSOUND
                ;
FIRESOUND       CMP #2
                BNE FIRSOUNDDO
FIRSNDINI       LDA #FR1SND/256+$C8
                STA $FF12
                LDA #FR1SND&255
                STA $FF0E
                DEC V1SNDI
                ;
FIRSOUNDDO      SEC
                LDA $FF0E
                SBC #FRSSND
                TAX
                STA $FF0E
                LDA $FF12
                AND #$03
                SBC #0
                BNE FIRSNDCON
                CPX #FR2SND
                BCS FIRSNDCON
FIRSNDFIN       STA V1SNDI ;0-SEE^
FIRSNDCON       ORA #$C8
                STA $FF12
                BNE V1SOUNDON ;JMP
                ;
                ;
GUNFRSOUND      CMP #5
                BNE GUNFRSNDDO
GNFRSDINI       LDA #GN1SND&255
                STA $FF0E
                LDA #GN1SND/256+$C8
                STA $FF12
                DEC V1SNDI
GUNFRSNDDO      SEC
                LDA $FF0E
                SBC #GNSSND
                STA $FF0E
                TAY
                LDA $FF12
                AND #3
                SBC #0
                BNE GUNFRSNDOK
                CPY #GN2SND ;(<256)
                BCS GUNFRSNDOK
                STA V1SNDI ;=0
GUNFRSNDOK      ORA #$C8
                STA $FF12
                ;JMP V1SOUNDON FALLS THRU ANYWAY
                ;
                ;
V1SOUNDON       LDA $FF11
                ORA #$1F
                STA $FF11
                BNE ENDOFSOUND ;JMP
                ;
NOV1SOUND       LDA $FF11
                AND #$6F
                STA $FF11
                ;
ENDOFSOUND      PLA
                LDX #RSNM1&255
                LDY #RSNM1/256
                JMP IRQEX
                ;
                ;
                ;
                ;CALCULATE OFFSET FROM # OF LINES
                ;USED IN SCROLLING
                ;SEND # OF LINES IN AR
                ;OFFSET IS PUT IN CH2 (SO F. TEMP)
RSXTR0          TAX
                LDA OFFSETLO,X
                STA CH2
                LDA OFFSETHI,X
                AND #$03
                STA CH2+1
RSXTR1          RTS
                ;
                ;
                ;ENSURE 'FLICKER-FREE' DISPLAY
WAITRS          LDA $FF1D
                CMP #$16
                BNE WAITRS
WTRS1           LDA $FF1D
                CMP #$CE
                BNE WTRS1
                RTS
                ;
                ;
RCBLOWUP        INC MCONFLG
                JSR BLWSHP
                JSR BLWBAL
                LDA REACX
                BEQ DTHLP0
                STA CORD1+1
                LDA REACY
                BEQ DTHLP0
                STA CORD2+1
                AND #0
                STA CORD1
                STA CORD2
                ORA #2
                STA CHAR
                JSR EXPLSN
                ;
DTHLP0          LDA #$01
DTHLP1          STA DVAR1 ;SCREEN COL
                STA $FF15
                LDA #5
                STA DVAR2
DTHLP2          JSR WAITRS
                JSR UNDOT
                JSR DOTS
                DEC DVAR2
                BNE DTHLP2
                LDA DVAR1
                CLC
                ADC #$10
                BPL DTHLP1
                ;
                AND #0
                STA $FF15
                JMP DIE
                ;
                ;
                ;UNDISPLAY/REDISPLAY GRAPHICS
UNDRD           JSR WAITRS
                LDA COLDET
                AND #$0C
                EOR #$0C
                BEQ UNDRDC
                JSR UNPLTB
UNDRDC          JSR UNDOT
                LDA COLDET
                AND #$08
                BNE UNDRDA
                JSR UNSHDS
UNDRDA          LDA COLDET
                AND #$04
                BNE UNDRDB
                JSR UNBLDS
UNDRDB          AND #0
                STA UNPLOT
                JSR PLOTOB
                JSR SCRLSC
                JSR GATES
                ;
                ;CHECK FOR ESCAPED PLANET...
                LDA TPLAYH
                BNE UNDRD1
                ORA TPLAYL
                CMP #90 ;ESCAPE HEIGHT
                BCS UNDRD1
                JMP LPLAN
                ;
UNDRD1          LDX COLDET
                BEQ UNDRD2
                AND #0
                STA V1SNDI ;KILL V1 SOUND
                INC KLLSHP
                LDA KLLSHP
                CMP #$40
                BCC NTDYET
                JMP DIE
NTDYET          CMP #$20
                BNE NTBLBT
                ;MAKE SURE BOTH EXPLODED
                TXA
                AND #$08
                BNE NTSHPA
                JSR BLWSHP
NTSHPA          TXA
                AND #$04
                BNE NTBLBT
                JSR BLWBAL
                ;
NTBLBT          TXA
                AND #$88
                EOR #$80
                BNE NTSHPD
                JSR BLWSHP
NTSHPD          TXA
                AND #$44
                EOR #$40
                BNE UNDRD2
                JSR BLWBAL
                ;
UNDRD2          LDX CNTDWN
                BMI NOTCOUNT
                ;
WTCHK           LDA $FF1D
                CMP #$B0
                BCS WTCHK
                TXA
                BNE FLREAC
                JMP RCBLOWUP
FLREAC          LDA LOPCNT
                AND #2
                STA GUNFGS+1 ;ALWAYS REACTOR
                ;
NOTCOUNT        LDA FIRFLG
                BEQ UNDRD2A
                JSR FIRE
UNDRD2A         ORA #$FF
                STA UNPLOT
                JSR PLOTOB
                LDA COLDET
                AND #$04
                BNE UNDRD3
                JSR BALL
UNDRD3          LDA COLDET
                AND #$08
                BNE UNDRD4
                JSR SHIP
UNDRD4          JSR DOTS
                LDA COLDET
                AND #$0C
                EOR #$0C
                BEQ UNDRDD
                JMP TOWBAR
;UNDRDD          RTS ; Share RTS with BLWSHP below
                ;
                ;
BLWSHP          TXA
                PHA
                LDA COLDET
                ORA #$08
                STA COLDET
                LDA SHPX
                STA CORD1
                LDA SHPX+1
                STA CORD1+1
                LDA SHPY
                STA CORD2
                LDA SHPY+1
                STA CORD2+1
                ;
BLWCON          LDX #3
BLWCN1          LSR CORD2+1
                ROR CORD2
                LSR CORD1+1
                ROR CORD1
                DEX
                BNE BLWCN1
                CLC
                LDA CORD1+1
                ADC #4
                STA CORD1+1
                LDA #2
                STA CHAR
                JSR EXPLSN
                PLA
                TAX
UNDRDD
BLUPEX          RTS
                ;
BLWBAL          LDA PICKUP
                BEQ BLUPEX
                TXA
                PHA
                LDA COLDET
                ORA #$04
                STA COLDET
                LDA LOADX
                STA CORD1
                LDA LOADX+1
                STA CORD1+1
                LDA LOADY
                STA CORD2
                LDA LOADY+1
                STA CORD2+1
                JMP BLWCON
                ;
                ;
                ;******************
                ;***SCROLL SCAPE***
                ;******************
SCRLSC          AND #0
                STA WRAPFLG
                LDA SCXVL
                BNE SCRLXX
                JMP SCRLY
SCRLXX          LDA SMTHX
                SEC
                SBC SCXVL
                TAY
                AND #$07
                STA SMTHX
                TYA
                EOR #$FF
                CLC
                ADC #$08
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A ;/8
                BNE SCXCN
                JMP SCRLY
SCXCN           CMP #$10
                BCC SCRLX1
                JMP SCRLXM
SCRLX1          STA CHAR
                TAY
                CLC
                ADC PLAYX
                CMP #86 ;128-42
                BCC SSCX1 ;WRAP IF NEC.
                SBC #86 ;CARRY SET ^
                INC WRAPFLG
SSCX1           STA PLAYX
                ;SCROLL (DATA RIGHT TO LEFT)
                LDA WRAPFLG
                BNE CHANGECDJ
                LDX #0
SCXLFT          JSR SCROLL
                INX
                INY
                CPY #$28
                BCC SCXLFT
                ;
                LDA CHAR
                STA TEMP2
                ;
                LDA #40
                SEC
                SBC TEMP2
                CLC
                ADC PLAYX
                STA TEMP1
                ;
LSCNDT          LDA #$0CA0&255
                LDY #$0CA0/256
                SEC
                SBC TEMP2
                STA DAT1
                STY DAT1+1
                ;
                JSR DOVERT ;CORRUPTS DAT1
                INC TEMP1
                DEC TEMP2
                BNE LSCNDT
                ;
                ;CHANGE CO-ORDS
CHANGECDJ       JMP CHNGCD
                ;
                ;
                ;MOVE SCREEN DATA FROM Y OFFSET
                ;TO X OFFSET... (COLOURS ASWELL)
SCROLL          LDA $0C78,Y ;1
                STA $0C78,X
                LDA $0CA0,Y ;2
                STA $0CA0,X
                LDA $0CC8,Y ;3
                STA $0CC8,X
                LDA $0CF0,Y ;4
                STA $0CF0,X
                LDA $0D18,Y ;5
                STA $0D18,X
                LDA $0D40,Y ;6
                STA $0D40,X
                LDA $0D68,Y ;7
                STA $0D68,X
                LDA $0D90,Y ;8
                STA $0D90,X
                LDA $0DB8,Y ;9
                STA $0DB8,X
                LDA $0DE0,Y ;10
                STA $0DE0,X
                LDA $0E08,Y ;11
                STA $0E08,X
                LDA $0E30,Y ;12
                STA $0E30,X
                LDA $0E58,Y ;13
                STA $0E58,X
                LDA $0E80,Y ;14
                STA $0E80,X
                LDA $0EA8,Y ;15
                STA $0EA8,X
                LDA $0ED0,Y ;16
                STA $0ED0,X
                LDA $0EF8,Y ;17
                STA $0EF8,X
                LDA $0F20,Y ;18
                STA $0F20,X
                LDA $0F48,Y ;19
                STA $0F48,X
                LDA $0F70,Y ;20
                STA $0F70,X
                LDA $0F98,Y ;21
                STA $0F98,X
                LDA $0FC0,Y ;22
                STA $0FC0,X
EXSCRL          RTS
                ;
                ;
                ;SCROLL HORIZONTALLY OTHER WAY
SCRLXM          CLC
                ADC #$07 ;$27-$20
                TAY
                CLC
                ADC #$D9 ;CONVERT TO TRUE NEGATIVE
                STA CHAR
                ADC PLAYX ;WRAP IF NEC.
                BEQ SSCX3
                BCS SSCX2
SSCX3           CLC
                ADC #86 ;128-42
                INC WRAPFLG
SSCX2           STA PLAYX
                ;SCROLL (DATA LEFT TO RIGHT)
                LDA WRAPFLG
                BNE CHNGCD
                LDX #$27
SCXRT           JSR SCROLL
                DEX
                DEY
                BPL SCXRT
                ;
                TXA
                CLC
                ADC PLAYX
                STA TEMP1
                INX
                STX TEMP2
                ;
RSCNDT          LDA #$0C77&255
                LDY #$0C77/256
                CLC
                ADC TEMP2
                STA DAT1
                STY DAT1+1
                ;
                JSR DOVERT
                DEC TEMP1
                DEC TEMP2
                BNE RSCNDT
                ;
                ;CHANGE CO-ORDS
CHNGCD          LDX #31
CHCRRS          LDA BLXHI,X
                SEC
                SBC CHAR ;# OF CHARS JUST SCROLLED
                STA BLXHI,X
                DEX
                BPL CHCRRS
                ;
                ASL CHAR
                ASL CHAR
                ASL CHAR ;*8=PIXELS
                ;
                LDA SHPX+1
                SEC
                SBC CHAR
                STA SHPX+1
                LDA CMX+1
                SEC
                SBC CHAR
                STA CMX+1
                ;
                LDA LOADX+1
                SEC
                SBC CHAR
                STA LOADX+1
                ;
                ;
SCRLY           ;AND #0
                ;STA SCXVL ;RESET X VEL.
                ;
                LDA SCYVL
                BNE SCRLY1
SCRLY0          LDA WRAPFLG
                BEQ SCRLY0A
                JSR WHOLED
SCRLY0A         RTS ;DONE X & Y
SCRLY1          LDA SMTHY
                SEC
                SBC SCYVL
                TAY
                AND #$07
                STA SMTHY
                TYA
                EOR #$FF
                CLC
                ADC #$08
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                BNE SCRLY2A
                JMP SCYEXT ; Done
SCRLY2A         CMP #$10
                BCC SCRLX2
                JMP SCRLUP
SCRLX2          STA CHAR
                TAY
                CLC
                ADC TPLAYL
                STA TPLAYL
                BCC SCRLY2
                INC TPLAYH
SCRLY2          STY TEMP1 ;# OF CHARS
                TYA
                DEY
                BNE SCRLYBG
                JMP SCRLYD1 ;FAST SCROLL
SCRLYBG         JSR RSXTR0
                ;
                LDA #$0C78&255
                STA PS1DAT1+1
                CLC
                ADC CH2
                STA PS1DAT2+1
                LDA #$0C78/256
                STA PS1DAT1+2
                ADC CH2+1
                STA PS1DAT2+2
                ;
                LDA #22 ;NUMBER OF PLAY ROWS
                SEC
                SBC TEMP1
                TAX
SCRLY5          ;MOVE 1 LINE OF SCREEN DATA
                ;FROM DAT2 TO DAT1
                ;DOESN'T CORRUPT XR
                LDY #$27 ;=`39
PS1DAT2         LDA PS1DAT2,Y
PS1DAT1         STA PS1DAT1,Y
                DEY
                BPL PS1DAT2
                ;
                LDA PS1DAT2+1
                CLC
                ADC #$28 ;=`40
                STA PS1DAT2+1
                BCC SCRLY6
                INC PS1DAT2+2
SCRLY6          LDA PS1DAT1+1
                CLC
                ADC #$28 ;=`40
                STA PS1DAT1+1
                BCC SCRLY7
                INC PS1DAT1+2
SCRLY7          DEX
                BNE SCRLY5
                LDA PS1DAT1+1
                STA DAT1
                LDA PS1DAT1+2
                STA DAT1+1
                ;
                ;DAT1 IS NOW SET UP OK
                ;FOR SCAPE DISPLAY
                ;BUT FIRST REDO SCAPE BUFFERS
                ;
SCRLYRJ         JSR UNCOMP ;DOESN'T CORRUPT DAT1
                LDA WRAPFLG
                BNE CHANGCDJ2
                ;
                LDA #22
                SEC
                SBC TEMP1
                TAX
SCRLY8          JSR CLRIT
                LDA DAT1
                CLC
                ADC #$28 ;=`40
                STA DAT1
                BCC SCRLY9
                INC DAT1+1
SCRLY9          INX
                DEC TEMP1
                BNE SCRLY8
                ;
                ;CHANGE CO-ORDS
CHANGCDJ2       JMP CHNCD2
                ;
                ;
SCRLUP          ORA #$E0 ;TRUE NEGATIVE
                STA CHAR
                TAY
                CLC
                ADC TPLAYL
                STA TPLAYL
                BCS SCRUP1 ;FLAGS REVERSED AS NEG
                DEC TPLAYH
SCRUP1          TYA
                EOR #$FF
                CLC
                ADC #1 ;2'S COMPLEMENT
                STA TEMP1 ;# OF CHARS
                INY
                BNE SCRLYUBG
                JMP SCRLYU1 ;FAST SCROLL
SCRLYUBG        JSR RSXTR0
                ;
                LDA #$0FC0&255
                STA PS2DAT1+1
                SEC
                SBC CH2
                STA PS2DAT2+1
                LDA #$0FC0/256
                STA PS2DAT1+2
                SBC CH2+1
                STA PS2DAT2+2
                ;
                LDA #22 ;NO. OF PLAY ROWS
                SEC
                SBC TEMP1
                TAX
SCYB1           ;MOVE 1 LINE OF SCREEN DATA
                ;FROM DAT2 TO DAT1
                ;DOESN'T CORRUPT XR
                LDY #$27 ;=`39
PS2DAT2         LDA PS2DAT2,Y
PS2DAT1         STA PS2DAT1,Y
                DEY
                BPL PS2DAT2
                ;
                LDA PS2DAT2+1
                SEC
                SBC #$28 ;=`40
                STA PS2DAT2+1
                BCS SCYB2
                DEC PS2DAT2+2
SCYB2           LDA PS2DAT1+1
                SEC
                SBC #$28 ;=`40
                STA PS2DAT1+1
                BCS SCYB3
                DEC PS2DAT1+2
SCYB3           DEX
                BNE SCYB1
                LDA PS2DAT1+1
                STA DAT1
                LDA PS2DAT1+2
                STA DAT1+1
                ;
                ;DAT1 OK FOR DISPLAY
                ;FIRST REDO BUFFERS
                ;
SCRLYURJ        JSR UNCOMP ;DOESN'T CORRUPT DAT1
                LDA WRAPFLG
                BNE CHNCD2
                ;
                LDX TEMP1
                DEX
SCYB4           JSR CLRIT
                LDA DAT1
                SEC
                SBC #$28 ;=`40
                STA DAT1
                BCS SCYB5
                DEC DAT1+1
SCYB5           DEX
                BPL SCYB4
                ;
                ;CHANGE CO-ORDS
CHNCD2          LDX #31
CHCRYU          LDA BLYHI,X
                SEC
                SBC CHAR ;# OF CHARS JUST SCROLLED
                STA BLYHI,X
                DEX
                BPL CHCRYU
                ;
                LDA CHAR
                ASL ; = ASL A
                ASL ; = ASL A
                ASL ; = ASL A ;*8=PIXELS
                STA CHAR
                ;
                LDA SHPY+1
                SEC
                SBC CHAR
                STA SHPY+1
                LDA CMY+1
                SEC
                SBC CHAR
                STA CMY+1
                ;
                LDA LOADY+1
                SEC
                SBC CHAR
                STA LOADY+1
                ;
                ;
SCYEXT          AND #0
                STA SCYVL ;RESET Y VEL.
                JMP SCRLY0 ;FINISHED X & Y
                ;
                ;
                ;FAST COPY FOR ONE LINE SCROLL (MOVE DATA UP)
SCRLYD1         LDY #$20
SCYD1A          LDA $0C80,Y
                STA $0C58,Y
                INY
                BPL SCYD1A
                LDY #$27
SCYD1B          LDA $0D00,Y
                STA $0CD8,Y
                DEY
                BPL SCYD1B
                LDY #$00
SCYD1C          LDA $0D28,Y
                STA $0D00,Y
                INY
                BPL SCYD1C
                LDY #$28
SCYD1D          LDA $0D80,Y
                STA $0D58,Y
                INY
                BPL SCYD1D
                LDY #$27
SCYD1E          LDA $0E00,Y
                STA $0DD8,Y
                DEY
                BPL SCYD1E
                LDY #$00
SCYD1F          LDA $0E28,Y
                STA $0E00,Y
                INY
                BPL SCYD1F
                LDY #$28
SCYD1G          LDA $0E80,Y
                STA $0E58,Y
                INY
                BPL SCYD1G
                LDY #$27
SCYD1H          LDA $0F00,Y
                STA $0ED8,Y
                DEY
                BPL SCYD1H
                LDY #$00
SCYD1I          LDA $0F28,Y
                STA $0F00,Y
                INY
                BPL SCYD1I
                LDY #$40
SCYD1J          LDA $0F68,Y
                STA $0F40,Y
                INY
                BPL SCYD1J
                LDA #$C0
                STA DAT1
                LDA #$0F
                STA DAT1+1
                JMP SCRLYRJ ;REJOIN COPY ROUTINE
                ;
                ;FAST COPY FOR ONE LINE SCROLL (MOVE DATA DOWN)
SCRLYU1         LDY #$BF
SCYU1A          LDA $0F00,Y
                STA $0F28,Y
                DEY
                BNE SCYU1A
                LDA $0F00
                STA $0F28
                LDY #$27
SCYU1B          LDA $0ED8,Y
                STA $0F00,Y
                DEY
                BPL SCYU1B
                LDY #$D7
SCYU1C          LDA $0E00,Y
                STA $0E28,Y
                DEY
                BNE SCYU1C
                LDA $0E00
                STA $0E28
                LDY #$27
SCYU1D          LDA $0DD8,Y
                STA $0E00,Y
                DEY
                BPL SCYU1D
                LDY #$D7
SCYU1E          LDA $0D00,Y
                STA $0D28,Y
                DEY
                BNE SCYU1E
                LDA $0D00
                STA $0D28
                LDY #$27
SCYU1F          LDA $0CD8,Y
                STA $0D00,Y
                DEY
                BPL SCYU1F
                LDY #$5F
SCYU1G          LDA $0C78,Y
                STA $0CA0,Y
                DEY
                BPL SCYU1G
                LDA #$0C78&255
                STA DAT1
                LDA #$0C78/256
                STA DAT1+1
                JMP SCRLYURJ
                ;
                ;
                ;'PLOTOB'
                ;PLOT OBJECTS
                ;
                ;
PLTSBX          SEC
PLTSBX1         RTS
PLOTSUB         STX TEMPX
                LDA GUNFGS,X
                STA GSTAT
SMDGNX          LDA GUNX0,X
                SEC
                SBC PLAYX
                BCC PLTSBX
                STA CORD2
SMDGYL          LDA GUNYL0,X
                SEC
                SBC TPLAYL
                STA CORD2+1
SMDGOR          LDA GUNOR0,X
                TAY
                PHP
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                PLP
                SBC TPLAYH
                BNE PLTSBX
                TYA
                AND #$0F
                STA GOR
                TAY
                LDA CORD2
                CMP WIDTHS,Y
                BCS PLTSBX1
                LDA CORD2+1
                CMP HEIGHTS,Y
                BCS PLTSBX1
                ;
PLOTIT          JSR STCCRD ;SETUP SC1
                LDA SC1
                STA CL1
                LDA SC1+1
                SEC
                SBC #4
                STA CL1+1
                LDY GOR
                LDA GDATLT,Y
                STA DAT1
                LDA GDATHT,Y
                STA DAT1+1
                LDA GADDLT,Y
                STA DAT2
                LDA GADDHT,Y
                STA DAT2+1
                LDA GDATLEN,Y
                STA NEWTEMP
                CLC
PLTLPEX         RTS
                ;
                ;
PLOTOB          LDA UNPLOT
                BNE DONEGAT
                LDY #BUBCHR
                LDA CNTDWN
                BPL BUBSOFF
                LDA REACNT
                BEQ YEBUBS
BUBSOFF         LDY #32
YEBUBS          STY REACTD
                ;
                LDA LOPCNT
                AND #3
                BEQ NOBUBS
                LDX 8*BUBCHR+$3800
                LDY #0
BUBSCR          LDA 8*BUBCHR+$3801,Y
                STA 8*BUBCHR+$3800,Y
                INY
                CPY #7
                BNE BUBSCR
                STX 8*BUBCHR+$3807
                ;
NOBUBS          LDA LOPCNT
                AND #15
                BEQ DGOUT
                LDA REACNT
                BEQ DGOUT
                DEC REACNT
                ;
DGOUT           LDA GATCNT
                BEQ DONEGAT
                DEC GATCNT
                ;
DONEGAT         LDX #0
                STX TBST
                STX REACX
                STX REACY
                DEX
PLTLP1          INX
                CPX NUMGUN ;NO OF OBJECTS
                BCS PLTLPEX
PLTLP2          JSR PLOTSUB
                BCS PLTLP1
                ;
                LDA UNPLOT
                AND GSTAT
                PHA
                BNE PLTLP3

                LDY GOR
                CPY #4 ;IS OBJECT A GUN ?
                BCS UNPLTLP ;NO
                LDX RPSCCH,Y ;CHAR TO REPLACE BIT OF GUN IN SCAPE
                LDY #4 ;LENGTH OF GUN DATA
                BNE UNPLT1 ;JMP

UNPLTLP         LDY NEWTEMP
                LDX #SPACE
UNPLT1          LDA (DAT2),Y
                TAY
                LDA #SHIPCOL+8 ;COLOUR
                STA (CL1),Y
                TXA
                STA (SC1),Y
                DEC NEWTEMP
                BPL UNPLTLP
                BMI PLTLP4 ;JMP

PLTLP3          LDY NEWTEMP
                LDA (DAT1),Y
PLTXT1          TAX
                LDA (DAT2),Y
                TAY
                LDA COLTAB,X
                STA (CL1),Y
                TXA
                STA (SC1),Y
                DEC NEWTEMP
                BPL PLTLP3
                ;
PLTLP4          LDA GOR
                CMP #6
                BNE PLTLPXTR
                LDX CORD2
                INX
                STX REACX
                LDX CORD2+1
                INX
                STX REACY
                ;
PLTLPXTR        PLA
                BNE PLTLP4A
                JMP PLTLP7
PLTLP4A         LDA GOR
                CMP #5 ;PEDISTAL
                BNE PLTLP5
                ;SET UP BALL CO-ORDS & TEST FOR PICKUP
                LDA CORD2
                SEC
                SBC #3
                BCC PLTLP5
                CMP #32
                BCS PLTLP5
                ASL ; = ASL A
                ASL ; = ASL A
                ASL ; = ASL A ;*8
                STA LOADX+1
                LDX CORD2+1
                INX
                TXA
                ASL ; = ASL A
                ASL ; = ASL A
                ASL ; = ASL A ;*8
                STA LOADY+1
                LDA SHST
                BNE PIKTS1
                STA CONNECT
                BEQ PLTLP6 ;JMP,NOT FUEL
PIKTS1          JSR PKUPTST
                LDA PICKUP
                BEQ PLTLP6 ;NOT FUEL
                LDA #$FE
                STA V1SNDI ;SOUND
                LDX TEMPX
                AND #0
                STA GUNFGS,X
                JMP PLTLP2 ;UNPLOT
                ;
PLTLP5          ;TEST FOR FUEL PICKUP
                LDA GOR
                CMP #4
                BNE PLTLP6
                LDA PICKUP
                BNE PLTLP6
                LDA SHST
                BEQ PLTLP6
                ;CONVERT CO-ORDS
                LDA CORD2 ;X
                SEC
                SBC #4
                CMP #32
                BCS PLTLP6
                ASL ; = ASL A
                ASL ; = ASL A
                ASL ; = ASL A ;*8
                SEC
                SBC SHPX+1
                BCS PLTLP6
                CMP #$EA
                BCC PLTLP6
                CMP #$FE
                BCS PLTLP6
                LDA CORD2+1 ;Y
                ASL ; = ASL A
                ASL ; = ASL A
                ASL ; = ASL A ;*8
                SEC
                SBC SHPY+1
                CMP #$0A
                BCC PLTLP6
                CMP #$1F
                BCS PLTLP6
                STA TBST
                LDX TEMPX
                DEC TANKS,X
                BNE NEMPTY
                LDA #$FE
                STA V1SNDI ;SOUND
                AND #0
                STA GUNFGS,X
                LDA #FUELSCR
                JSR ADDSCR
NEMPTY          JSR INFUEL
                ;
                ;
PLTLP6          LDX GOR
                CPX #4
                BCS PLTLP7
                LDA REACNT
                BNE PLTLP7
                LDA CNTDWN
                BPL PLTLP7
                JSR RANDOM
                CMP DIFFC2
                BCS PLTLP7
                ;
                LDA #5
                STA V1SNDI ;SOUND
                ;
                JSR FNDSPC
                LDA GBLXOF,X
                STA BLXLO,Y
                LDA GBLYOF,X
                STA BLYLO,Y
                LDA CORD2
                CLC
                ADC GBHXOF,X
                STA BLXHI,Y
                LDA CORD2+1
                CLC
                ADC GBHYOF,X
                STA BLYHI,Y
                LDA #3
                STA BLTYPE,Y
                JSR RANDOM
                BPL GNFIR1
                ORA #$E0
                BNE GNFIR2 ;JMP
GNFIR1          AND #$1F
GNFIR2          CLC
                ADC GBXMV,X
                STA BLXV,Y
                JSR RANDOM
                BPL GNFIR3
                ORA #$E0
                BNE GNFIR4 ;JMP
GNFIR3          AND #$1F
GNFIR4          CLC
                ADC GBYMV,X
                STA BLYV,Y
                JSR RANDOM
                AND #$3F
                ORA #$0F
                STA BULLFG,Y
                ;
PLTLP7          LDX TEMPX
                JMP PLTLP1
                ;
                ;
GBLXOF          BYTE 0,0,0,0
GBLYOF          BYTE 0,0,0,0
GBHXOF          BYTE 2,1,1,2
GBHYOF          BYTE 0,0,2,2
GBXMV           BYTE $20,$E0,$E0,$20
GBYMV           BYTE $E0,$E0,$20,$20
                ;
                ;CHARS IN SCAPE...REPLACE LAST BYTE OF GUNS DATA
RPSCCH          BYTE 60,61,63,66
                ;
                ;
WIDTHS          BYTE 38,38,38,38 ;GUNS
                BYTE 38 ;FUEL
                BYTE 39 ;PEDISTAL(+BALL)
                BYTE 38 ;REACTOR
                BYTE 40,40 ;GATE CONTROLS
                ;
                ;
HEIGHTS         BYTE 20,20,20,20 ;GUNS
                BYTE 20 ;FUEL
                BYTE 19 ;PEDISTAL(+BALL)
                BYTE 19 ;REACTOR
                BYTE 20,20 ;GATE CONTROLS
                ;
                ;
GDATLT          BYTE GUNP1D&255
                BYTE GUNP2D&255
                BYTE GUNP3D&255
                BYTE GUNP4D&255
                BYTE FUELD&255
                BYTE PDSTLD&255
                BYTE REACTD&255
                BYTE LHSDMD&255
                BYTE RHSDMD&255
GDATHT          BYTE GUNP1D/256
                BYTE GUNP2D/256
                BYTE GUNP3D/256
                BYTE GUNP4D/256
                BYTE FUELD/256
                BYTE PDSTLD/256
                BYTE REACTD/256
                BYTE LHSDMD/256
                BYTE RHSDMD/256
GADDLT          BYTE GUNP1A&255
                BYTE GUNP2A&255
                BYTE GUNP3A&255
                BYTE GUNP4A&255
                BYTE FUELA&255
                BYTE PDSTLA&255
                BYTE REACTA&255
                BYTE LHSDMA&255
                BYTE RHSDMA&255
GADDHT          BYTE GUNP1A/256
                BYTE GUNP2A/256
                BYTE GUNP3A/256
                BYTE GUNP4A/256
                BYTE FUELA/256
                BYTE PDSTLA/256
                BYTE REACTA/256
                BYTE LHSDMA/256
                BYTE RHSDMA/256
                ;
                ;(LENGTH OF DATA FOR EACH OBJECT)-1
GDATLEN         BYTE 4,4,4,4,5,5,6,1,1
                ;
GUNP1D          BYTE 75,76,77,79,78 ;,0
GUNP1A          BYTE 0,1,2,42,41
GUNP2D          BYTE 82,83,84,80,81 ;,0
GUNP2A          BYTE 0,1,2,40,41
GUNP3D          BYTE 90,92,93,94,91 ;,0
GUNP3A          BYTE 0,40,41,42,1
GUNP4D          BYTE 89,85,86,87,88 ;,0
GUNP4A          BYTE 2,40,41,42,1
FUELD           BYTE 101,102,103,104,105,106 ;,0
FUELA           BYTE 0,1,2,40,41,42
PDSTLD          ;+BALL
                BYTE 95,96,97,98,99,100 ;,0
PDSTLA          BYTE 0,1,40,41,80,81
REACTD          BYTE BUBCHR,207,208,73,209,72,73 ;,0
REACTA          BYTE 1,40,41,42,80,81,82
LHSDMD          BYTE 96,98 ;,0
LHSDMA          BYTE 0,40
RHSDMD          BYTE 95,97 ;,0
RHSDMA          BYTE 0,40
                ;
                ;
                ;# OF GUNS ON EACH SCAPE
GUNTOT          BYTE 4,5,13,12,19,17
                ;
                ;DATA FOR EACH LANDSCAPE
GUNX0           BYTE 67 ;PED
                BYTE 75 ;REACTOR
                ;FUEL
                BYTE 50
                ;GUNS
                BYTE 58
                ;
GUNYL0          BYTE 165
                BYTE 161 ;REACTOR
                ;FUEL
                BYTE 163
                ;GUNS
                BYTE 165
                ;
GUNOR0          BYTE $05 ;PED
                BYTE $06 ;REACTOR
                ;FUEL
                BYTE $04
                ;GUNS
                BYTE $00
                ;
                ;
                ;
GUNX1           BYTE 65 ;PED
                BYTE 52 ;REACTOR
                ;FUEL
                BYTE 70
                ;GUNS
                BYTE 61,81
                ;
GUNYL1          BYTE 195 ;PED
                BYTE 162 ;REAC
                ;FUEL
                BYTE 196
                ;GUNS
                BYTE 187,185
                ;
GUNOR1          BYTE $05 ;PED
                BYTE $06 ;REAC
                ;FUEL
                BYTE $04
                ;GUNS
                BYTE $03,$02
                ;
                ;
                ;
GUNX2           BYTE 31 ;PED
                BYTE 82 ;REAC
                ;FUEL
                BYTE 60,74
                BYTE 77,80
                BYTE 59,49
                ;GUNS
                BYTE 84,64
                BYTE 44,39
                BYTE 24
                ;
GUNYL2          BYTE 231 ;PED
                BYTE 164 ;REAC
                ;FUEL
                BYTE 159,186
                BYTE 186,186
                BYTE 201,214
                ;GUNS
                BYTE 185,181
                BYTE 196,216
                BYTE 208
                ;
GUNOR2          BYTE $05 ;PED
                BYTE $06 ;REAC
                ;FUEL
                BYTE $04,$04
                BYTE $04,$04
                BYTE $04,$04
                ;GUNS
                BYTE $01,$03
                BYTE $03,$01
                BYTE $03
                ;
                ;
                ;
GUNX3           BYTE 75 ;PED
                BYTE 46 ;REAC
                BYTE 90,90 ;DOMES
                ;FUEL
                BYTE 76
                ;GUNS
                BYTE 58,44,44,60
                BYTE 54,75,84
                ;
GUNYL3          BYTE 245 ;PED
                BYTE 204 ;REAC
                BYTE 211,227 ;DOMES
                ;FUEL
                BYTE 212
                ;GUNS
                BYTE 174,189,195,199
                BYTE 207,229,239
                ;
GUNOR3          BYTE $05 ;PED
                BYTE $06 ;REAC
                BYTE $08,$08 ;DOMES
                ;FUEL
                BYTE $04
                ;GUNS
                BYTE $03,$00,$03,$02
                BYTE $00,$03,$01
                ;
                ;
                ;
GUNX4           BYTE 81 ;PED
                BYTE 72 ;REAC
                BYTE 85,78 ;DOMES
                ;FUEL
                BYTE 63,76,79,52
                BYTE 55,58,69,72
                ;GUNS
                BYTE 58,80,66,50
                BYTE 72,64,87
                ;
GUNYL4          BYTE 28 ;PED
                BYTE 193 ;REAC
                BYTE 1,18 ;DOMES
                ;FUEL
                BYTE 170,194,194,216
                BYTE 250,250,5,5
                ;GUNS
                BYTE 186,186,215,249
                BYTE 248,4,17
                ;
GUNOR4          BYTE $15 ;PED
                BYTE $06 ;REAC
                BYTE $18,$17 ;DOMES
                ;FUEL
                BYTE $04,$04,$04,$04
                BYTE $04,$04,$14,$14
                ;GUNS
                BYTE $03,$02,$01,$00
                BYTE $02,$10,$12
                ;
                ;
                ;
                ;
GUNX5           BYTE 78 ;PED
                BYTE 84 ;REAC
                BYTE 82,94 ;DOMES
                ;FUEL
                BYTE 76,98
                ;GUNS
                BYTE 86,78,84,74
                BYTE 62,85,85,87
                BYTE 102,78,76
                ;
GUNYL5          BYTE 55 ;PED
                BYTE 60 ;REAC
                BYTE 35,19 ;DOMES
                ;FUEL
                BYTE 251,210
                ;GUNS
                BYTE 47,42,31,8
                BYTE 5,237,230,214
                BYTE 204,203,175
                ;
GUNOR5          BYTE $15 ;PED
                BYTE $16 ;REAC
                BYTE $17,$18 ;DOMES
                ;FUEL
                BYTE $04,$04
                ;GUNS
                BYTE $11,$13,$13,$12
                BYTE $13,$01,$02,$01
                BYTE $02,$03,$02
                ;
                ;
                ;
                ;**ABOVE NEEDS TUNING**


;
; File X4.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;'SCPDAT'
                ;SCAPE DATA
                ;
SCPDAT
                ;
                ;
                ;
LEDIS0          BYTE 161,0,4
                BYTE 0,3,0,64
                ;
                ;
LEDIR0          BYTE $00,$26,$02
                BYTE $0A,$02,$0D,$00
                ;
                ;
REDIS0          BYTE 161
                BYTE 0,3,0,64
                ;
                ;
REDIR0          BYTE $00
                BYTE $2A,$02,$06,$00
                ;
                ;
                ;
LEDIS1          BYTE 162,0,3
                BYTE 0,5,14,6
                BYTE 5,3,0,64
                ;
                ;
LEDIR1          BYTE $00,$28,$02
                BYTE $0D,$02,$00,$FE
                BYTE $00,$02,$0B,$00
                ;
                ;
REDIS1          BYTE 162,0,6,15
                BYTE 4,5,6,64
                ;
                ;
REDIR1          BYTE $00,$26,$02,$00
                BYTE $FE,$00,$02,$00
                ;
                ;
                ;
LEDIS2          BYTE 161,0,19
                BYTE 3,12,0,3
                BYTE 9,0,3,21
                BYTE 3,0,64
                ;
                ;
LEDIR2          BYTE $00,$44,$00
                BYTE $FE,$00,$F2,$FE
                BYTE $00,$F2,$FE,$00
                BYTE $02,$0A,$00
                ;
                ;
REDIS2          BYTE 161,0,6
                BYTE 0,14,0,4
                BYTE 3,0,15,0
                BYTE 13,0,3,64
                ;
                ;
REDIR2          BYTE $00,$26,$00
                BYTE $0C,$00,$F5,$00
                BYTE $02,$0A,$00,$0E
                BYTE $00,$0F,$02,$00
                ;
                ;
                ;
LEDIS3          BYTE 161,0,5
                BYTE 0,6,11,6
                BYTE 3,2,5,8
                BYTE 0,7,0,12
                BYTE 7,9,2,128
                ;
                ;
LEDIR3          BYTE $00,$2D,$02
                BYTE $09,$00,$FE,$00
                BYTE $02,$00,$FE,$00
                BYTE $0E,$02,$11,$00
                BYTE $FE,$00,$02,$00
                ;
                ;
REDIS3          BYTE 161,0,30,0
                BYTE 5,8,0,34
                BYTE 6,4,0,128
                ;
                ;
REDIR3          BYTE $00,$3A,$00,$0F
                BYTE $00,$FE,$EC,$00
                BYTE $02,$00,$06,$00
                ;
                ;
                ;
LEDIS4          BYTE 161,0,6,5
                BYTE 0,13,0,3
                BYTE 8,0,10,4
                BYTE 0,22,5,3
                BYTE 0,9,3,0
                BYTE 8,3,0,22
                BYTE 2,0,64
                ;
                ;
LEDIR4          BYTE $00,$2C,$02,$00
                BYTE $0B,$00,$FB,$FE
                BYTE $00,$05,$00,$FE
                BYTE $F6,$00,$02,$00
                BYTE $FB,$00,$02,$08
                BYTE $00,$02,$0A,$00
                BYTE $02,$06,$00
                ;
                ;
REDIS4          BYTE 161,0,24,0
                BYTE 3,8,0,10
                BYTE 0,9,3,0
                BYTE 9,8,12,0
                BYTE 3,6,0,16
                BYTE 3,9,3,64
                ;
                ;
REDIR4          BYTE $00,$36,$00,$FB
                BYTE $FE,$00,$12,$00
                BYTE $FC,$00,$02,$0E
                BYTE $00,$FE,$00,$FC
                BYTE $FE,$00,$F7,$00
                BYTE $FE,$00,$02,$00
                ;
                ;
                ;
LEDIS5          BYTE 156,0,16
                BYTE 0,20,10,0
                BYTE 3,41,0,14
                BYTE 3,5,14,4
                BYTE 3,8,3,14
                BYTE 0,64
                ;
                ;
LEDIR5          BYTE $00,$29,$00
                BYTE $0B,$02,$00,$F6
                BYTE $FE,$00,$F6,$00
                BYTE $FE,$00,$02,$00
                BYTE $FE,$00,$FE,$00
                BYTE $06,$00
                ;
                ;
REDIS5          BYTE 156,0,11
                BYTE 5,14,16,5
                BYTE 5,0,7,9
                BYTE 4,5,3,13
                BYTE 0,10,11,6
                BYTE 2,4,2,13,6
                BYTE 8,0,4,0,64
                ;
                ;
REDIR5          BYTE $00,$23,$02
                BYTE $00,$FE,$00,$FE
                BYTE $00,$0D,$02,$00
                BYTE $FE,$00,$02,$00
                BYTE $0B,$00,$FE,$00
                BYTE $FE,$00,$02,$00,$02
                BYTE $00,$FB,$00,$06,$00
                ;
                ;
                ;
                ;***ABOVE NEEDS FINE-TUNING***
                ;'MESDAT'
                ;0:MESDAT
                ;MESSAGES ETC
                ;
                ;
SCREEN_ORIGIN   EQU $0C00
MST
MIMESS          BYTE $0D,$EB ;$0DEB=(9+3,11)
                BYTE $63 ;CYAN
                ;'MISSION '
                BYTE 13,9,19,19,9,15,14,32
                ;BRK *SEE NEXT LINE
                ;
INMESS          BYTE 0,0 ;DEFAULT - CARRY ON
                BYTE $63 ;CYAN
                ;'IN'
                BYTE 9,14
                ;BRK *SNL
                ;
CMMESS          BYTE 0,0 ;DEF.
                BYTE $63 ;CYAN
                ;'COMPLETE'
                BYTE 3,15,13,16,12,5,20,5
                BRK
                ;
BOSCRN          EQU $0E3F ;$0E3F=(11+3,15)
BOMESS          BYTE BOSCRN/256,BOSCRN&255
                BYTE $77 ;YELLOW
                ;'BONUS .000'
                BYTE 2,15,14,21,19,32,32,48,48,48
                ;BRK *SNL
                ;
FAMESS          BYTE 0,0 ;DEF.
                BYTE $63 ;CYAN
                ;'FAILED'
                BYTE 6,1,9,12,5,4
                BRK
                ;
NBMESS          BYTE $0E,$40 ;$0E40=(11+3,16)
                BYTE $67 ;YELLOW
                ;'NO BONUS'
                BYTE 14,15,32,2,15,14,21,19
                BRK
                ;
PDMESS          BYTE $0D,$9C ;$0D9C=(7+3,12)
                BYTE $65 ;GREEN
                ;'PLANET DESTROYED'
                BYTE 16,12,1,14,5,20,32,4,5,19,20,18,15,25,5,4
                BRK
                ;
RGMESS          BYTE $0E,$14 ;$0E14=(10+3,12)
                BYTE $76 ;BLUE
                ;'REVERSE GRAVITY'
                BYTE 18,5,22,5,18,19,5,32,7,18,1,22,9,20,25
                BRK
                ;
ILMESS          BYTE $0D,$C2 ;$0DC2=(8+3,10)
                BYTE $74 ;PURPLE
                ;'INVISIBLE LANDSCAPE'
                BYTE 9,14,22,9,19,9,2,12,5,32,12,1,14,4,19,3,1,16,5
                BRK
                ;
GOMESS          BYTE $0E,$18 ;$0E18=(10+3,16)
                BYTE $33 ;CYAN
                ;'GAME OVER'
                BYTE 7,1,13,5,32,15,22,5,18
                BRK
                ;
FUMESS          BYTE $0D,$C7 ;$0DC7=(8+3,15)
                BYTE $47 ;YELLOW
                ;'OUT OF FUEL'
                BYTE 15,21,20,32,15,6,32,6,21,5,12
                BRK
                ;
                ;
MS2ST
SPMESS_POS      EQU SCREEN_ORIGIN+[15*40]+6
SPMESS          ;BYTE $0E,$62 ;$0E62=(12+3,10)
                BYTE >SPMESS_POS, <SPMESS_POS
                BYTE $71 ;COL.
                ;'PRESS SPACE TO START'
                ;BYTE 16,18,5,19,19,32,19,16,1,3,5,32
                ;BYTE 20,15,32,19,20,1,18,20
                DV.B ASCII2CBM 'P, 'R, 'E, 'S, 'S, ' , 'S, 'P, 'A, 'C, 'E
                DV.B ASCII2CBM ' , 'T, 'O, ' , 'S, 'T, 'A, 'R, 'T
                DV.B ASCII2CBM ' , 'S, 'C, 'A, 'P, 'E, ' , '1
SPMESS_END      BYTE 0
SPMESS_LEN      EQU [SPMESS_END - SPMESS - 3]

                ;
                ; 2009-11: MY: Swap the commenting on the next two lines to
                ;              move the top text line on the title page up to
                ;              immediately under the score box.  Useful for
                ;              testing smooth scrolling.
                ;
TIMESS          BYTE $0C,$AB ;$0CAB=(1+3,11)
                ;BYTE $0C,$83 ;$0CAB=(1+3,11)
                BYTE $77 ;YELLOW
                ;'COMMODORE 16 THRUST'
                BYTE 3,15,13,13,15,4,15,18,5
                BYTE 32,40,41,32,20,8,18,21,19,20
                BRK
                ;
T1MESS_POS      EQU SCREEN_ORIGIN+[6*40]+5
T1MESS          ;BYTE $0C,$F5 ;$0CF5=(3+3,5)
                BYTE >T1MESS_POS, <T1MESS_POS
                BYTE $53 ;CYAN
                ;'PROGRAMMED BY MATTHEW F. YOUNG'
                ;BYTE 16,18,15,7,18,1,13,13,5,4,32,2,25,32
                ;BYTE 13,1,20,20,8,5,23,32,6,46,32,25,15,21,14,7
                DV.B ASCII2CBM 'P, 'R, 'O, 'G, 'R, 'A, 'M, 'M, 'E, 'D
                DV.B ASCII2CBM ' , 'B, 'Y, ' , 'M, 'A, 'T, 'T, 'H, 'E, 'W
                DV.B ASCII2CBM ' , 'F, ' , 'Y, 'O, 'U, 'N, 'G
                BYTE 0
                ;
T2MESS          BYTE $0D,$4B ;$0D4B=(5+3,11)
                BYTE $53 ;CYAN
                ;'FOR LYNSOFT UK LTD'
                BYTE 6,15,18,32,12,25,14,19,15,6,20
                BYTE 32,21,11,32,12,20,4
                BYTE 0
                ;
T3MESS          BYTE $0D,$96 ;$0D96=(7+3,6)
                BYTE $73 ;CYAN
                ;'WITH GRAPHICS BY CHRIS YOUNG'
                BYTE 23,9,20,8,32,7,18,1,16,8,9,3,19,32
                BYTE 2,25,32,3,8,18,9,19,32,25,15,21,14,7
                BYTE 0
                ;
T4MESS          BYTE $0D,$EC ;$0DEC=(10+3,12)
                BYTE $75 ;GREEN
                ;'@ FIREBIRD 1987'...@=(C)
                BYTE 210,32,6,9,18,5,2,9,18,4,32,40,206,74,34
                BYTE 0
                BYTE $FF
                ;
                ;
                ;
                ;CONSTANTS FOR PRINT ROUTINES
MIOFF           EQU MIMESS-MST
INOFF           EQU INMESS-MST
CMOFF           EQU CMMESS-MST
BOOFF           EQU BOMESS-MST
FAOFF           EQU FAMESS-MST
NBOFF           EQU NBMESS-MST
PDOFF           EQU PDMESS-MST
RGOFF           EQU RGMESS-MST
ILOFF           EQU ILMESS-MST
GOOFF           EQU GOMESS-MST
FUOFF           EQU FUMESS-MST
                ;
BOSCRP          EQU BOSCRN+6
                ;
                ;
                ;'BALL/SHIP/ROPE'
                ;0:BALL/SHIP/ROPE
                ;DISPLAY BALL
                ;
                ;SEND X/Y POS. OF MIDDLE
                ;OF BALL IN BALLPS
                ;
                ;
                ;OFFSETS FROM BALL CHAR ORIGIN
BALTAB          BYTE 0,40,80
                BYTE 1,41,81
                BYTE 2,42,82
                ;
                ;COPY & ADJUST CO-ORDS
BALL            LDA PICKUP
                BNE BALL1
                RTS
BALL1           LDA LOADY+1
                SEC
                SBC #BHGT
                STA CORD1+1
                SEC
                LDA LOADX+1
                SBC #BWID
                STA CORD1
                ;
                ;SET UP TABLE
                ;
                ;SET UP SC1
                JSR SETCRD
                ;
                ;SAVE ORIGIN FOR BALL POSITION
                STA BLSL
                STX BLSH
                ;
                ;BUILD TABLE...
BBTAB           LDX #8
CBCD1L          LDY BALTAB,X
                LDA (SC1),Y
                STA BLCS,X
                ;
                ;...COPY CHAR DATA...
                LDY #107 ;1ST BALL CHAR
                STY TEMP1
                JSR SETCHR
                ;
                ;...AND LOOP
                DEX
                BPL CBCD1L
                ;
                ;SET UP PTR FOR BALL 'SPRITE DAT'
                LDX #BALDAT&255
                LDY #BALDAT/256
                STX DAT1
                STY DAT1+1
                ;
                ;GET OFFSETS
                LDA CORD1
                AND #$07
                STA XDIST
                LDA CORD1+1
                AND #$07
                STA YDIST
                ;
                ;LOOP COUNTER
                LDA #11
                STA TEMP2
                ;
                ;DO SMOOTH BIT...
SMTHBL          AND #0
                STA BUFFER+2
                LDY #1 ;COPY CURR PIXEL LINE
                LDA (DAT1),Y
                STA BUFFER,Y
                DEY
                LDA (DAT1),Y
                ;
                LDX XDIST
                BEQ CBBTC
BSMRT1          LSR ; = LSR A
                ROR BUFFER+1
                ROR BUFFER+2
                DEX
                BNE BSMRT1
                ;CRRY CLR AS LAST BUF BYT CLRD @ STRT
                ;
CBBTC           STA BUFFER
                LDY YDIST
                CLC
                LDA $3B58,Y
                TAX
                ORA BUFFER
                STA $3B58,Y
                TXA
                AND BUFFER
                BEQ CC2OK1
                SEC
CC2OK1          LDA $3B58+24,Y ;24=3*8
                TAX
                ORA BUFFER+1
                STA $3B58+24,Y
                TXA
                AND BUFFER+1
                BEQ CC2OK2
                SEC
CC2OK2          LDA $3B58+48,Y ;48=6*8
                TAX
                ORA BUFFER+2
                STA $3B58+48,Y
                TXA
                AND BUFFER+2
                BEQ CC2OK3
                SEC
CC2OK3          BCC CC2OK4
                LDA #$40
                ORA COLDET
                STA COLDET
                ;
CC2OK4          INY
                STY YDIST
                LDA DAT1
                CLC
                ADC #2
                STA DAT1
                BCC BMPB1
                CLC
                INC DAT1+1
BMPB1           DEC TEMP2
                BNE SMTHBL
                ;
                ;DISPLAY BALL
                ;BLDISP LDA PICKUP
                ;BEQ BLDSPX
                ;
                JSR BLSTPT
BLDS1L          LDA BLCS,X
                CMP #59
                BCC BLNOMC
                CMP #72
                BCS BLNOMC
                LDA #SHIPCOL+8
                BNE BLPLCL ;JMP
BLNOMC          LDA #SHIPCOL
BLPLCL          LDY BALTAB,X
                STA (CL1),Y
                TXA
                CLC
                ADC #107 ;1ST BALL CHAR
                STA (BLSL),Y
                DEX
                BPL BLDS1L
BLDSPX          RTS
                ;
                ;UNDISPLAY BALL
UNBLDS          LDA PICKUP
                BEQ UNBLDX
                ;
                JSR BLSTPT
UNBD1L          LDY BALTAB,X
                LDA BLCS,X
                STA (BLSL),Y
                TAY
                LDA COLTAB,Y
                LDY BALTAB,X
                STA (CL1),Y
                DEX
                BPL UNBD1L
UNBLDX          RTS
                ;
                ;
                ;SET UP PTRS...
BLSTPT          LDA BLSL
                STA CL1
                LDA BLSH
                SEC
                SBC #4
                STA CL1+1
                LDX #8
                RTS
                ;
                ;
                ;DISPLAY SHIP (AND SHIELD,TBEAM)
                ;
                ;SEND SHIP CO-ORDS IN SHPX,SHPY
                ;(MIDDLE OF SHIP,FRCS. IGNORED)
                ;SEND SHIELD ON/OFF IN SHST
                ;SEND TRACTOR BEAM ON/OFF IN TBST
                ;
                ;
                ;OFFSETS FROM SHIP CHAR ORIGIN
PSPTAB          BYTE 0,40,80,120,160,200,240
                BYTE 1,41,81,121,161,201,241
                BYTE 2,42,82,122,162,202,242
                BYTE 3,43,83,123,163,203,243
                ;
                ;
                ;COPY SHST TO TEMP2
SHIP            LDA SHST
                STA TEMP2
                ;
                ;COPY & CHANGE CO-ORDS
                LDA SHPY+1
                SEC
                SBC #SHHGT
                STA CORD1+1
                SEC
                LDA SHPX+1
                SBC #SHWID
                STA CORD1
                ;
                ;SET UP TABLE
                ;
                ;SET UP SC1
                JSR SETCRD
                ;
                ;SAVE ORIGIN FOR SHIP POSITION
                STA SHSL
                STX SHSH
                ;
                ;BUILD TABLE...
BSTAB           LDX #27
CSCDL1          LDY PSPTAB,X
                LDA (SC1),Y
                STA SHCS,X
                ;
                ;...COPY CHAR DATA...
                LDY #116 ;1ST SHIP/TRCTR BEAM CHAR
                STY TEMP1
                JSR SETCHR
                ;
                ;...AND LOOP
                DEX
                BPL CSCDL1
                ;
                ;SET UP PTR FOR SHIP 'SPRITE DATA'
                LDA #SHPBUF&255
                STA DAT1
                LDA #SHPBUF/256
                STA DAT1+1
                ;
                ;SET UP PTR FOR SHIP SHIELD DATA
                LDX #SHLDDT&255
                LDY #SHLDDT/256
                STX DAT2
                STY DAT2+1
                ;
                ;GET OFFSETS
NOSHDAT         LDA CORD1
                AND #$07
                STA XDIST
                LDA CORD1+1
                AND #$07
                STA YDIST
                ;
                ;LOOP COUNTER
                LDA #48
                STA TEMP1
                ;
                ;DO SMOOTH BIT!
SMOOTH          AND #0
                STA BUFFER+3
                LDY #2
SMCP1           LDA (DAT1),Y ;SHIP DATA
                LDX TEMP2
                BEQ SMCP2
                ORA (DAT2),Y ;SHIELD DATA
SMCP2           STA BUFFER,Y
                DEY
                BPL SMCP1 ;COPY CURR PIXEL LINE
                ;
                LDX XDIST
                BEQ CPBTC
SMRT1           LSR ; = LSR A
                ROR BUFFER+1
                ROR BUFFER+2
                ROR BUFFER+3
                DEX
                BNE SMRT1
                STA BUFFER
                ;
CPBTC           LDY YDIST
                CLC
                LDA $3BA0,Y
                TAX
                ORA BUFFER
                STA $3BA0,Y
                TXA
                AND BUFFER
                BEQ CCKOK1
                SEC
CCKOK1          LDA $3BA0+56,Y ;56=7*8
                TAX
                ORA BUFFER+1
                STA $3BA0+56,Y
                TXA
                AND BUFFER+1
                BEQ CCKOK2
                SEC
CCKOK2          LDA $3BA0+112,Y ;112=14*8
                TAX
                ORA BUFFER+2
                STA $3BA0+112,Y
                TXA
                AND BUFFER+2
                BEQ CCKOK3
                SEC
CCKOK3          LDA $3BA0+168,Y ;168=21*8
                TAX
                ORA BUFFER+3
                STA $3BA0+168,Y
                TXA
                AND BUFFER+3
                BEQ CCKOK4
                SEC
CCKOK4          BCC CCKOK5
                LDA TEMP1 ;IGNORE TBM COLLISIONS
                CMP #28
                BCC CCKOK5

                LDA CHEAT_MODE
                AND #CHEAT_INVULNERABLE
                BNE CCKOK5

                LDA #$80
                ORA COLDET
                STA COLDET
                ;
CCKOK5          INY
                STY YDIST
                LDA DAT1
                CLC
                ADC #3
                STA DAT1
                ;BCC BMPS1-ALWAYS ON SAME PAGE
                ;CLC
                ;INC DAT1+1
BMPS1           LDA DAT2
                ADC #3
                STA DAT2
                ;BCC BMPS2-ALWAYS ON SAME PAGE
                ;INC DAT2+1
BMPS2           DEC TEMP1
                BEQ EXSHIP
                LDA TEMP1
                CMP #27
                BEQ CHDPTS
                JMP SMOOTH
CHDPTS          LDA TBST
                BEQ EXSHIP
                LDX #TRBMDT&255
                LDY #TRBMDT/256
                STX DAT1
                STY DAT1+1
                AND #0
                STA TEMP2
                JMP SMOOTH
EXSHIP
                ;DISPLAY SHIP
SHDISP          JSR SHDUSB
                ;
SHDSL1          LDY PSPTAB,X
                LDA TBST
                STA OTBST
                BNE SHDSL2
                CPY #125
                BCS SHDSL3
SHDSL2          LDA SHCS,X
                CMP #59
                BCC SHNOMC
                CMP #72
                BCS SHNOMC
                LDA #SHIPCOL+8 ;MCM ON
                BNE SHPLCL ;JMP
SHNOMC          LDA #SHIPCOL
SHPLCL          STA (CL1),Y
                TXA
                CLC
                ADC #116 ;1ST SHIP UDG
                STA (SHSL),Y
SHDSL3          DEX
                BPL SHDSL1
                RTS
                ;
                ;UNDISPLAY SHIP
UNSHDS          JSR SHDUSB
                ;
UNSDL1          LDY PSPTAB,X
                LDA OTBST
                BNE UNSDL2
                CPY #125
                BCS UNSDL3
UNSDL2          LDA SHCS,X
                STA (SHSL),Y
                TAY
                LDA COLTAB,Y
                LDY PSPTAB,X
                STA (CL1),Y
UNSDL3          DEX
                BPL UNSDL1
                RTS
                ;
SHDUSB          LDA SHSL
                STA CL1
                LDA SHSH
                SEC
                SBC #4
                STA CL1+1
                LDX #27
                RTS
                ;
                ;
                ;COPY SHIP DATA TO BUFFER AND DO
                ;TRANSFORMATIONS IF NECESSARY
SHIPDTABL       BYTE        SHPDAT&255,  [SHPDAT+$40]&255, [SHPDAT+$80]&255
                BYTE  [SHPDAT+$C0]&255, [SHPDAT+$100]&255, [SHPDAT+$140]&255
                BYTE [SHPDAT+$180]&255, [SHPDAT+$1C0]&255, [SHPDAT+$200]&255
SHIPDTABH       BYTE        SHPDAT/256,  [SHPDAT+$40]/256, [SHPDAT+$80]/256
                BYTE  [SHPDAT+$C0]/256, [SHPDAT+$100]/256, [SHPDAT+$140]/256
                BYTE [SHPDAT+$180]/256, [SHPDAT+$1C0]/256, [SHPDAT+$200]/256
                ;
DSBNMCP         LDA (DAT1),Y
                STA SHPBUF,Y
                DEY
                BPL DSBNMCP
                BMI RJDSHBF ;JMP
                ;
DOSHBF          LDA SHPOR
                TAX
                CMP #$09
                BCC DSHBF0
                CMP #$10
                BEQ DSHBFY
                BCC DSHBFZ
                CMP #$18
                BCC DSHBF0
DSHBFZ          EOR #$07
                CLC
                ADC #$01
DSHBF0          CMP #$09
                BCC DSHBFY
                SBC #$08 ;CARRY SET ^
                BNE DSHBF0
DSHBFY          AND #$0F
                TAY
                LDA SHIPDTABL,Y
                STA DAT1
                LDA SHIPDTABH,Y
                STA DAT1+1
                LDY #$3E
                TXA
                PHA
                CLC
                ADC #$08
                AND #$10
                BEQ DSBNMCP ;NORMAL COPY
                LDX #0 ;COPY WITH MIRROR (^)
DSBCM1          LDA (DAT1),Y
                STA SHPBUF+2,X
                DEY
                LDA (DAT1),Y
                STA SHPBUF+1,X
                DEY
                LDA (DAT1),Y
                STA SHPBUF,X
                INX
                INX
                INX
                DEY
                BPL DSBCM1
RJDSHBF         PLA ;REJOIN HERE
                AND #$10
                BEQ DOSHBX
                ;
MIRR1           ;<
                LDX #$3E
MIRR1A          LDA SHPBUF,X
                LDY #8
MIRR1B          ASL ; = ASL A
                ROR TEMP1
                DEY
                BNE MIRR1B
                LDA TEMP1
                STA SHPBUF,X
                DEX
                BPL MIRR1A
                LDX #$3C
MIRR1C          LDA SHPBUF+2,X
                TAY
                LDA SHPBUF,X
                STA SHPBUF+2,X
                TYA
                STA SHPBUF,X
                DEX
                DEX
                DEX
                BPL MIRR1C
                ;
                ;MOVE DATA 1 PIXEL TO THE LEFT
                LDX #$3E
                CLC
SHPDTF          ROL SHPBUF,X ;WRAP DOESN'T MATTER
                DEX
                BPL SHPDTF
                ;
DOSHBX          RTS
                ;
                ;
                ;
                ;TOWBAR SUBROUTINE
                ;START CO-ORDS IN SHPX/SHPY
                ;END CO-ORDS IN LOADX/LOADY
                ;CORD3 IS WORKSPACE
                ;
TOWBAR          LDA PICKUP
                ORA CONNECT
;                BNE TOWBR1
;                RTS
                BEQ DOSHBX ; RTS
                ;RESET TOWBAR BUFFER POINTER
TOWBR1          ORA #$FF
                STA TBBPTR
                ;
                ;CHECK X DIRECTION
CHKX            LDX #0
                LDA LOADX+1
                CMP SHPX+1
                BCS XGRT
                INX ;=1
XGRT            STX XFLG
                ;
                ;CHECK Y DIRECTION
CHKY            LDY #0
                LDA LOADY+1
                CMP SHPY+1
                BCS YGDN
                INY ;=1
YGDN            STY YFLG
                ;
                ;COPY START INTO CORD3
                ;& CLEAR FRACTIONS
                LDA #$80 ;START IN MIDDLE OF PIXEL
                STA CORD3+3
                STA CORD3+2
                LDA SHPX+1
                STA CORD3
                LDA SHPY+1
                STA CORD3+1
                ;
                ;FIND XDIST
                SEC
                LDA XFLG
                BNE XDST1
                LDA LOADX+1
                SBC SHPX+1
                JMP XDST2
XDST1           LDA SHPX+1
                SBC LOADX+1
XDST2           STA XDIST
                ;
                ;FIND YDIST
                SEC
                LDA YFLG
                BNE YDST1
                LDA LOADY+1
                SBC SHPY+1
                JMP YDST2
YDST1           LDA SHPY+1
                SBC LOADY+1
YDST2           STA YDIST
                ;
                ;CONVERT DISTS TO LARGEST FRACS
DTFIN           LDA XDIST
                BMI DTFEX
                LDA YDIST
                BMI DTFEX
                ASL XDIST
                ASL YDIST
                JMP DTFIN
DTFEX
                ;
                ;SET UP NEXT CHAR
PNTC            JSR NXTCHR
                ;
                ;CALC POS OF AND SET NEXT BITS
PNTA            LDA CORD3+1
                AND #$07
                TAY
                LDA CORD3
                AND #$07
                TAX
                AND #0
                SEC
RPBIT           ROR ; = ROR A
                DEX
                BPL RPBIT
                ORA (CH2),Y
                STA (CH2),Y
                ;
                ;COMPARE CURX/Y WITH ENDX/Y
                ;IF EQUAL, THEN EXIT
                LDA CORD3
                CMP LOADX+1
                BNE EXCOM
                LDA CORD3+1
                CMP LOADY+1
                BNE EXCOM
                JMP PLTTBR
EXCOM
                ;
                ;ADD (SUBTRACT) FRACTIONS
                AND #0
                STA PLTFLG
                STA NEWCHR
DOFRC           LDA CORD3+2
                LDX XFLG
                BNE SUBX
                CLC
                ADC XDIST
                STA CORD3+2
                BCC DOFRY
                INC CORD3
                LDA CORD3
                AND #$07
                BNE DPLFLG
                INC NEWCHR
                BNE DPLFLG
SUBX            SEC
                SBC XDIST
                STA CORD3+2
                BCS DOFRY
                DEC CORD3
                LDA CORD3
                AND #$07
                EOR #$07
                BNE DPLFLG
                INC NEWCHR
DPLFLG          INC PLTFLG
DOFRY           LDA CORD3+3
                LDY YFLG
                BNE SUBY
                CLC
                ADC YDIST
                STA CORD3+3
                BCC CHECKS
                INC CORD3+1
                LDA CORD3+1
                AND #$07
                BNE DPFLG2
                INC NEWCHR
                BNE DPFLG2
SUBY            SEC
                SBC YDIST
                STA CORD3+3
                BCS CHECKS
                DEC CORD3+1
                LDA CORD3+1
                AND #$07
                EOR #$07
                BNE DPFLG2
                INC NEWCHR
DPFLG2          INC PLTFLG
                ;
                ;CHECK FLAGS AND LOOP ACCORDINGLY
CHECKS          LDA PLTFLG
                BEQ DOFRC
                LDA NEWCHR
                BNE GPC
                JMP PNTA
GPC             JMP PNTC
                ;
                ;SET UP NEXT CHARACTER FOR TOWBAR
NXTCHR          ;CALC SCREEN POS
                LDA CORD3
                STA CORD1
                LDA CORD3+1
                STA CORD1+1
                JSR SETCRD
                ;
                ;PUT VALUE & LOC IN BUFFER
                LDY #0
                LDX TBBPTR
                INX ;BUMP TABLE POINTER
                STX TBBPTR
                LDA SC1
                STA TBSL,X
                LDA SC1+1
                STA TBSH,X
                LDA (SC1),Y
                STA TBCS,X
                ;
                ;COPY CHAR DATA AND EXIT
                LDY #144 ;1ST ROPE CHAR
                STY TEMP1
                JMP SETCHR
                ;
                ;
                ;PLOT TOWBAR ON SCREEN
PLTTBR          LDX TBBPTR
                LDY #0
PLTLP           JSR TBPTST
                TXA
                CLC
                ADC #144 ;1ST ROPE CHAR
                STA (SC1),Y
                LDA #SHIPCOL
                STA (CL1),Y
                DEX
                BPL PLTLP
PLTBEX          RTS
                ;
                ;UNPLOT TOWBAR FROM SCREEN
UNPLTB          LDA PICKUP
                ORA CONNECT
                BEQ UPLTBX
                LDX TBBPTR
UPTBLP          JSR TBPTST
                LDA TBCS,X
                LDY #0
                STA (SC1),Y
                TAY
                LDA COLTAB,Y
                LDY #0
                STA (CL1),Y
                DEX
                BPL UPTBLP
UPLTBX          RTS
                ;
                ;
TBPTST          LDA TBSL,X
                STA SC1
                STA CL1
                LDA TBSH,X
                STA SC1+1
                SEC
                SBC #$04
                STA CL1+1
                RTS



;
; File X5.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.41
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ;'MATHS'
                ;MATHS ROUTINES - NOT BY ME
                ;
                ;
                ;FIRST, SOME XTRA VARS/CONSTANTS
                ;
DINT            EQU 40          ;SHIP/LOAD DIST. (PIXELS)
SDFCM           EQU 20
SHPMS           EQU 8           ;CONSTANT
LDMASS          EQU 8
FINT            EQU 40
SHPCON          EQU 5           ;40/8:SEE^
LDCON           EQU 3           ;40/16:SEE^
                ;CON2=1 ASSUMED
;CON3            EQU 1/2 ;8/(8+8)
;CON4            EQU 1/8 ;APPROX.((32/2*PI)*(1/40))
                ;
                ;NOW THE ROUTINES
                ;
THRUST          ;DOES ACC DUE TO THRUSTING
                LDA #SHPCON ;F%/SHIPMS
                LDX PICKUP
                BEQ WEIGHT
                LDA #LDCON ;F%/(SHIPMS+LDMASS)
WEIGHT          STA CON1
                ;
                LDA THCTR
                ADC #100
                STA THCTR
                BCS DOTHR
                RTS
DOTHR
                ;LINEAR X
                LDA SHPOR
                JSR SINE
                LDA RESULT
                ASL ; = ASL A
                STA ACC2
                LDA RESULT+1
                ROL ; = ROL A
                STA ACC2+1 ;SIN(SHPOR)
                LDA CON1
                STA ACC1
                JSR MUL1X2 ;CON1*SIN(SHPOR)
                JSR DIVRES
                CLC
                LDA CMXVL
                ADC RESULT
                STA CMXVL
                LDA CMXVL+1
                ADC RESULT+1
                STA CMXVL+1
                ;
                ;LINEAR Y
                LDA SHPOR
                JSR COSINE
                LDA RESULT
                ASL ; = ASL A
                STA ACC2
                LDA RESULT+1
                ROL ; = ROL A
                STA ACC2+1 ;COS(SHPOR)
                LDA CON1
                STA ACC1
                JSR MUL1X2 ;CON1*COS(SHPOR)
                JSR DIVRES
                SEC
                LDA CMYVL
                SBC RESULT
                STA CMYVL
                LDA CMYVL+1
                SBC RESULT+1
                STA CMYVL+1
                ;
                ;ANGULAR
                SEC
                LDA PROR+1
                SBC SHPOR
                LDX PROR ;LOBYTE OF PROR-SHPOR
                JSR SINHRS
                LDA HIRESL+1
                ROL ; = ROL A
                PHP
                ROR ; = ROR A
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL
                PLP
                ROR ; = ROR A
                ROR HIRESL ;DIV 8 TO GET SINE
                STA RESULT+1
                LDA HIRESL
                STA RESULT
                ;RESULT=CON2*SIN(PROR-SHPOR) (CON2=1)
                JSR DIVRES
                SEC
                LDA PRANVL
                SBC RESULT
                STA PRANVL
                LDA PRANVL+1
                SBC RESULT+1
                STA PRANVL+1
                RTS
                ;
DIVRES          LDA RESULT+1
                ROL ; = ROL A
                PHP
                ROR ; = ROR A ;GET HI BIT
                PLP
                PHP
                ROR ; = ROR A
                ROR RESULT
                PLP
                PHP
                ROR ; = ROR A
                ROR RESULT
                PLP
                PHP
                ROR ; = ROR A
                ROR RESULT
                PLP
                ROR ; = ROR A
                ROR RESULT
                STA RESULT+1
                BCC DIVOUT
                INC RESULT
                BNE DIVOUT ;ROUNDING
                INC RESULT+1
DIVOUT          RTS
                ;
                ;
                ;MOVMNT SUBS
CSADDX          CLC
                LDA CMX
                ADC CMXVL
                LDA CMX+1
                ADC CMXVL+1
                RTS
                ;
CSADDY          CLC
                LDA CMY
                ADC CMYVL
                LDA CMY+1
                ADC CMYVL+1
                RTS
                ;
MOVMNT          ;ADD VELS & DO THRUST INDEP ACCS
                ;1ST CHECK FOR SCRL
CSCRLX          LDA CMXVL+1
                BMI CSCRLB
                ;
CSCRLA          JSR CSADDX
                BCS CSCRLA2
                CMP #$E0
                BCC CSSCRLA
CSCRLA2         ADC #$21 ;ONLY APPROX.
                BPL CSCRLA3
                LDA #$7F
CSCRLA3         STA SCXVL
                JMP CSCRLY ;JMP
CSSCRLA         CMP #$C0
                BCC CSSCRLA1
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                SEC
                SBC #$16
                BNE CSCRLA3 ;JMP
CSSCRLA1        CMP #$A0
                BCC NOSSCRLX
                BCS CSCRLY
NOSSCRLX        AND #0
                BEQ CSCRLA3 ;JMP
                ;
CSCRLB          JSR CSADDX
                BCC CSCRLB2
                CMP #$20
                BCS CSSCRLB
CSCRLB2         SBC #$21
                BMI CSCRLB3
                LDA #$80
CSCRLB3         STA SCXVL
                BNE CSCRLY ;JMP
CSSCRLB         CMP #$40
                BCS CSSCRLB1
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                SEC
                SBC #9
                BNE CSCRLB3 ;JMP
CSSCRLB1        CMP #$60
                BCS NOSSCRLX
                ;
CSCRLY          LDA CMYVL+1
                BMI CSCRLD
                ;
CSCRLC          JSR CSADDY
                BCS CSCRLE
                CMP #$70
                BCC CSSCRLC
CSCRLE          SEC
                SBC #$6F
                BPL CSXTR2
                LDA #$7F
CSXTR1          STA SCYVL
                BNE CSFIN ;JMP
CSSCRLC         CMP #$60
                BCC CSFIN
                LDA #$01
                BNE CSXTR2 ;JMP
                ;
CSSCRLD         CMP #$48
                BCS CSFIN
                ORA #$FF
                BNE CSXTR1 ;JMP
                ;
CSCRLD          JSR CSADDY
                BCC CSCRLF
                CMP #$38
                BCS CSSCRLD
CSCRLF          SEC
                SBC #$39
                BMI CSXTR2
                LDA #$80
CSXTR2          STA SCYVL
                ;
CSFIN           JSR UNDRD
                CLC ;KEEPS BRANCH IN RANGE IF HERE
                LDA KLLSHP
                BNE EXMVMNT
                ;
                ;ADD VELOCITIES
                LDA CMX
                ADC CMXVL
                STA CMX
                LDA CMX+1
                ADC CMXVL+1
                STA CMX+1
                CLC
                LDA CMY
                ADC CMYVL
                STA CMY
                LDA CMY+1
                ADC CMYVL+1
                STA CMY+1
                ;
                ;ANGULAR VELOCITY
                LDA PICKUP
                BEQ MVMTS1 ;ONLY ANGLE IF LOADED
                CLC
                LDA PROR
                ADC PRANVL
                STA PROR
                LDA PROR+1
                ADC PRANVL+1
                AND #31
                STA PROR+1
MVMTS1
                ;
                CLC
                LDA CMYVL
                ADC GRAV
                STA CMYVL
                LDA CMYVL+1
                ADC GRAV+1
                STA CMYVL+1
                ;
                ;X FRICTION
                LDY #0 ;HI BYTE
                LDA CMXVL+1
                STA TEMPORARY
                BPL XFRSK
                DEY ;$FF
XFRSK
                STY TEMPORARY+1
                ORA TEMPORARY+1
                BNE FRXSK
                INC TEMPORARY
FRXSK           ;FORCE NON-0 RESULT
                SEC
                LDA CMXVL
                SBC TEMPORARY
                STA CMXVL
                LDA CMXVL+1
                SBC TEMPORARY+1
                STA CMXVL+1
NOXFR
                ;ANGULAR FRICTION
                LDA PFRCTR
                ADC #128
                STA PFRCTR
                BCC NPFR ;REDUCE FRIC
                LDY #0 ;HI BYTE
                LDA PRANVL
                ROL ; = ROL A
                LDA PRANVL+1
                ADC #0 ;ROUND
                STA TEMPORARY
                BPL PFRSK
                DEY ;$FF
PFRSK           STY TEMPORARY+1
                SEC
                LDA PRANVL
                SBC TEMPORARY
                STA PRANVL
                LDA PRANVL+1
                SBC TEMPORARY+1
                STA PRANVL+1
NPFR            JSR POSNS
EXMVMNT         RTS
POSNS
                ;SETS UP SHPX ETC. ACCORDING TO CM AND PROR
                ;
                LDA PICKUP
                BNE POSSK1
                ;
                ;IF NOT LOADED, SHIPPOS=CMPOS
                LDA CMX
                STA SHPX
                LDA CMX+1
                STA SHPX+1
                LDA CMY
                STA SHPY
                LDA CMY+1
                STA SHPY+1
                RTS
POSSK1
                ;IS LOADED
                LDX PROR
                LDA PROR+1 ;LO,HI
                JSR SINHRS ;HIRESL=8*SIN(PROR) INT,FRAC
                LDA HIRESL+1
                ROL ; = ROL A
                PHP
                ROR ; = ROR A ;GET HI BIT INTO CARRY
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL ;DIV 4 (TO KEEP MULTIPLY IN RANGE)
                STA HIRESL+1
                STA ACC2+1
                LDA HIRESL
                STA ACC2 ;ACC2=2*SIN(PROR) INT,FRAC
                LDA #SDFCM
                STA ACC1
                JSR MUL1X2 ;RESULT=2*SDFCM*SIN(PROR) INT,FRAC
                PLP
                PHP
                ROR RESULT+1
                ROR RESULT ;FINAL FACTOR OF 2
                LDA RESULT
                ADC #0
                STA RESULT
                LDA RESULT+1
                ADC #0
                STA RESULT+1 ;ROUNDING
                CLC
                LDA CMX
                ADC RESULT
                STA SHPX
                LDA CMX+1
                ADC RESULT+1
                STA SHPX+1
                ;
                ;LOAD X
                LDA HIRESL
                STA ACC2
                LDA HIRESL+1
                STA ACC2+1
                ;LDA #DINT:SEC:SBC#SDFCM
                LDA #SDFCM
                STA ACC1
                JSR MUL1X2 ;RESULT=DINT*2*SIN(PROR) INT,FRAC
                PLP
                ROR RESULT+1
                ROR RESULT
                LDA RESULT
                ADC #0
                STA RESULT
                LDA RESULT+1
                ADC #0
                STA RESULT+1 ;ROUNDING
                SEC
                LDA CMX
                SBC RESULT
                STA LOADX
                LDA CMX+1
                SBC RESULT+1
                STA LOADX+1
                ;
                ;Y
                LDX PROR
                LDA PROR+1 ;LO,HI
                JSR COSHRS ;HIRESL=8*COS(PROR) INT,FRAC
                LDA HIRESL+1
                ROL ; = ROL A
                PHP
                ROR ; = ROR A ;GET HI BIT INTO CARRY
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL
                PLP
                PHP
                ROR ; = ROR A
                ROR HIRESL
                STA HIRESL+1
                STA ACC2+1
                LDA HIRESL
                STA ACC2 ;ACC2=2*COS(PROR) INT,FRAC
                LDA #SDFCM
                STA ACC1
                JSR MUL1X2 ;RESULT=2*SDFCM*COS(PROR) INT,FRAC
                PLP
                PHP
                ROR RESULT+1
                ROR RESULT
                LDA RESULT
                ADC #0
                STA RESULT
                LDA RESULT+1
                ADC #0
                STA RESULT+1 ;ROUNDING
                SEC
                LDA CMY
                SBC RESULT
                STA SHPY
                LDA CMY+1
                SBC RESULT+1
                STA SHPY+1
                ;
                ;LOAD Y
                LDA HIRESL
                STA ACC2
                LDA HIRESL+1
                STA ACC2+1
                ;LDA #DINT:SEC:SBC#SDFCM
                LDA #SDFCM
                STA ACC1
                JSR MUL1X2
                ;RESULT=DINT*SIN(PROR) INT,FRAC
                PLP
                ROR RESULT+1
                ROR RESULT
                LDA RESULT
                ADC #0
                STA RESULT
                LDA RESULT+1
                ADC #0
                STA RESULT+1 ;ROUNDING
                CLC
                LDA CMY
                ADC RESULT
                STA LOADY
                LDA CMY+1
                ADC RESULT+1
                STA LOADY+1
                ;
                RTS
                ;
                ; 'ARITH'
                ;0:ARITH
                ;**ARITHMETIC**
MUL1X2
                ;1BYTE(0-255)*2BYTES(+/-$7FFF)
                ;RESULT=ACC1*ACC2
                ;NO OVERFLOW CHECKING
                ;QUICKEST FOR SMALL ACC1
                AND #0
                STA RESULT
                STA RESULT+1
                LDX #8
                LDY RESULT+1
M1X2LP          LSR ACC1
                BCC M12NAD
                CLC
                PHP
                LDA ACC2
                ADC RESULT
                STA RESULT
                TYA
                ADC ACC2+1
                TAY
                PLP
M12NAD          BEQ M12OUT ;ACC1=0, NO POINT IN CONTINUING
                ROL ACC2
                ROL ACC2+1
                DEX
                BNE M1X2LP
M12OUT          STY RESULT+1
                RTS
                ;
                ;*TRIG FUNCTIONS*
                ;
                ;LO-RES ANGLES. 32 DEGREES/CIRCLE
                ;AR=NO OF DEGREES
                ;RESULT=-127 TO +127 (HI BYTE=0 OR $FF).
                ;WHICH IS +/- 0.5 IN INT/FRAC TERMS.
                ;CORRUPTS P,YR
                ;
SINE            PHA
                JMP RDSIN
                ;
COSINE          PHA
                CLC
                ADC #8 ;USING COS(X)=SIN(X+90)
                ;
RDSIN           AND #31
                TAY
                LDA SINTAB,Y
                STA RESULT
                BPL HISIN0
                ORA #$FF
RDSIN_EXIT      STA RESULT+1
                PLA
                RTS
HISIN0          AND #0
                ;STA RESULT+1
                ;PLA
                ;RTS
                BEQ RDSIN_EXIT ; = JMP

                ;
                ;
                ;HI-RES ANGLES
                ;AR = INT,XR = FRAC
                ;4 EXTRA BITS (TAKEN FROM X)
                ;=> 512 DEGREES
                ;RESULT IS 16*THAT FOR LO-RES,
                ;WHICH GIVES +/- $07F0
                ;(+/-8.0 APPROX)
                ;CORRUPTS AR,XR,YR
                ;
COSHRS          CLC
                ADC #8
                ;
SINHRS          JSR SINE
                TAY ;ANGLE ROUNDED DOWN
                LDA RESULT
                STA HIRESL
                LDA RESULT+1
                STA HIRESL+1
                TXA
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                LSR ; = LSR A
                STA TEMPORARY ;THE FOUR EXTRA BITS
                LDX #15
HRSLP           CPX TEMPORARY
                BNE NINANG
                INY
                TYA
                JSR SINE ;ANGLE ROUNDED UP
NINANG          CLC
                LDA RESULT
                ADC HIRESL
                STA HIRESL
                LDA RESULT+1
                ADC HIRESL+1
                STA HIRESL+1
                DEX
                BNE HRSLP
                RTS
                ;
                ;
NEGAC2          SEC
                AND #0
                SBC ACC2
                STA ACC2
                AND #0
                SBC ACC2+1
                STA ACC2+1
                AND #0
                SEC
                SBC ACC1
                STA ACC1
                RTS
FINDDST         ;BETWEEN SHIP & POD
                SEC
                LDA SHPY+1
                SBC LOADY+1
                BCS PTPOS1
                EOR #$FF
                ADC #1 ;CARRY CLEAR^
                INC CHAR ;OR#1
PTPOS1          STA LENY
                ;
                SEC
                LDA SHPX+1
                SBC LOADX+1
                BCS PTPOS2
                EOR #$FF
                ADC #1
                INC CHAR
                INC CHAR ;OR#2
PTPOS2          STA LENX
                ;
FINDLEN         LDX LENX
                CPX LENY
                BCS XLENBT
                LDY LENY
                STX LENY
                STY LENX
                LDA CHAR
                ORA #4
                STA CHAR
XLENBT          LDA LENY
                CLC
                ADC LENX
                BCS PDOUT
                ADC LENX
                BCS PDOUT
                ADC LENX
                BCC PDOUT2
PDOUT           ORA #$FF
PDOUT2          RTS
                ;
                ;
PKUPTST         ;CALLED FROM PLOTOB
                AND #0
                STA PROR
                STA CHAR ;FLAG
                JSR FINDDST
                CMP #117
                BCC DOCNECT
                CMP #132
                BCS TESTCON
                RTS
DOCNECT         LDA #1
                STA CONNECT
PTOUT           RTS
                ;
TESTCON         LDA CONNECT
                BEQ PTOUT
                ;
                ;PICKUP
                STA PICKUP
                ;
                ;FIND PROR
                ;MAX/MIN
                LDA LENY ;MIN
                BNE DIVIDE
                ORA #$FF
                BNE DIVIDE3 ;JMP
DIVIDE          AND #0
                LDX #12 ;RES*8
                CLC
DIVIDE1         ROL ACC1
                ASL LENX
                ROL ; = ROL A
                CMP LENY
                BCC DIVIDE2 ;0
                SBC LENY ;CSET^
                SEC ;1
DIVIDE2         DEX
                BNE DIVIDE1
                LDA ACC1
                BCC DIVIDE3
                CLC
                ADC #1 ;ROUNDING
DIVIDE3         STA PROR+1
                ;
                LDX #9
CHKTAN          DEX
                CMP TANTAB-4,X
                BCC CHKTAN
                STX PROR+1
                LDA CHAR
                AND #4
                BEQ DOPRR1
                LDA #8
                SEC
                SBC PROR+1
                STA PROR+1
                ;
DOPRR1          LDA CHAR
                AND #3
                BEQ DOPRR2
                CMP #1
                BEQ DOPRRF
                CMP #2
                BEQ DOPRR3
                ;1:Y-VE:16-ANG
                LDA #32
                SEC
                SBC PROR+1
                BNE DOPRRPF ;JMP
                ;2:X-VE:32-ANG
DOPRR2          LDA #16
                SEC
                SBC PROR+1
                BNE DOPRRPF
                ;3:X-VE,Y-VE:16+ANG
DOPRR3          LDA #16
                CLC
                ADC PROR+1
DOPRRPF         AND #$1F
                STA PROR+1
DOPRRF          ;FIND COFM (1/2 WAY BETWEEN)
                ;
                CLC
                LDA SHPX
                ADC LOADX
                STA CMX
                LDA SHPX+1
                ADC LOADX+1
                ROR ; = ROR A
                STA CMX+1
                ROR CMX
                ;
                CLC
                LDA SHPY
                ADC LOADY
                STA CMY
                LDA SHPY+1
                ADC LOADY+1
                ROR ; = ROR A
                STA CMY+1
                ROR CMY
                ;
                ;FIND ANG VEL
                LDA PROR+1
                JSR COSINE
                JSR DIVRES ;<<??
                LDA RESULT
                STA ACC2
                LDA RESULT+1
                STA ACC2+1
                LDA CMXVL+1
                STA ACC1
                BPL FNAVL1
                JSR NEGAC2 ;SIGN(ACC1) TO ACC2
FNAVL1          JSR MUL1X2 ;(COS(PROR)*CMXVL)/4
                LDA RESULT
                STA HIRESL
                LDA RESULT+1
                STA HIRESL+1
                ;
                LDA PROR+1
                JSR SINE
                JSR DIVRES ;:<<??
                LDA RESULT
                STA ACC2
                LDA RESULT+1
                STA ACC2+1
                LDA CMYVL+1
                STA ACC1
                BPL FNAVL2
                JSR NEGAC2
FNAVL2          JSR MUL1X2
                ;
                CLC
                LDA HIRESL
                ADC RESULT
                STA PRANVL
                LDA HIRESL+1
                ADC RESULT+1
                BMI KPINRNG2
KPINRNG1        AND #$07
                BPL INRANGE ;JMP
KPINRNG2        ORA #$F8
INRANGE         STA PRANVL+1 ;(1/8)*(VXCOSA+VYSINA)
                ;
                ;
                ;LINEAR VELS
                LDA CMXVL+1
                ASL ; = ASL A
                PHP
                ROR ; = ROR A
                PLP
                ROR ; = ROR A
                STA CMXVL+1
                ROR CMXVL
                ;
                LDA CMYVL+1
                ASL ; = ASL A
                PHP
                ROR ; = ROR A
                PLP
                ROR ; = ROR A
                STA CMYVL+1
                ROR CMYVL
                ;
                RTS
                ;
TANTAB          BYTE 0,10,15,26,82
                ;
                BYTE 0,0 ;END MARKER



;
; File X6.C16
; Starts Here
;
;  PDS Pc4.a :0000:  (C)  1985-7 P.D Systems
;
;
;       File  "REVISED C-16 THRUST"
;    Version          00.49
;         By  "**MATTHEW F YOUNG**"
;
;     Created on Fri the 21st of Aug 1987
;        Last update 20:19 on 25/09/87
;
                ORG $3800
                ;CHARSET
                HEX 0000030000030000 ;0:BUBBLES
                HEX 007E667E66660000 ;A
                HEX 007E667E667E0000 ;B
                HEX 007C6060607C0000 ;C
                HEX 007C6666667C0000 ;D
                HEX 007C6078607C0000 ;E
                HEX 007C607860600000 ;F
                HEX 007C60606C7C0000 ;G
                HEX 0066667E66660000 ;H
                HEX 007E1818187E0000 ;I
                HEX 007E181818780000 ;J
                HEX 0066667866660000 ;K
                HEX 00606060607C0000 ;L
                HEX 00667E6666660000 ;M
                HEX 007E666666660000 ;N
                HEX 007E6666667E0000 ;O
                HEX 007E66667E600000 ;P
                HEX 007E6666667E0600 ;Q
                HEX 007E666678660000 ;R
                HEX 007C607C0C7C0000 ;S
                HEX 007E181818180000 ;T
                HEX 00666666667E0000 ;U
                HEX 0066666666180000 ;V
                HEX 0066667E7E660000 ;W
                HEX 0066661866660000 ;X
                HEX 0066667E067E0000 ;Y
                HEX 007E067E407E0000 ;Z
                HEX 00000000A8080808 ;27:SCORE BOX
                HEX 00000000AA000000 ;28
                HEX 2020202020202020 ;29
                HEX 000000AA55000000 ;30
                HEX 000000002A202020 ;31
                HEX 0000000000000000 ;SPACE
                HEX 0018181800180000 ;~!
                HEX 00FC0C3030300000 ;34:~7 NOT UNDERLINED
                HEX 2020202020280A00 ;35:SCORE BOX
                HEX 080808080828A000 ;36
                HEX 000000000000AA00 ;37
                HEX 0808080808080808 ;38
                HEX 0300000000000000 ;39
                HEX 00F0303030FC0000 ;40:~1 NOT UNDERLINED
                HEX 00FCC0FCCCFC0000 ;41:~6
                HEX 6666186666000000 ;~*
                HEX 18187E1818000000 ;~+
                HEX 0000001818780000 ;~,
                HEX 0000F80000000000 ;~-
                HEX 0000000018180000 ;~.
                HEX 000C0C3C30300000 ;~/
                HEX FCCCCCCCFC00AA00 ;~0
                HEX F0303030FC00AA00 ;~1
                HEX FC0CFCC0FC00AA00 ;~2
                HEX FC0C3C0CFC00AA00 ;~3
                HEX C0CCFC0C0C00AA00 ;~4
                HEX FCC0FC0CFC00AA00 ;~5
                HEX FCC0FCCCFC00AA00 ;~6
                HEX FC0C30303000AA00 ;~7
                HEX FCCCFCCCFC00AA00 ;~8
                HEX FCCCFC0CFC00AA00 ;~9
                HEX 0010000010000000 ;~:
                HEX 8050A855AA55AA55 ;59:SLOPE ENDS
                HEX 000000008050A855 ;60
                HEX 0000000000010A15 ;61
                HEX 00010A15AA55AA55 ;62
                HEX AA150A0100000000 ;63
                HEX AA55AA55AA150A01 ;64
                HEX AA55AA55AA54A040 ;65
                HEX AA54A04000000000 ;66
                HEX AA55AA55AA55AA55 ;67:LANDSCAPE BLOCK
                HEX A050A050A050A050 ;68:EXTRA END BITS
                HEX 0A050A050A050A05 ;69
                HEX 00000000AA55AA55 ;70
                HEX AA55AA5500000000 ;71
                HEX 04040404FC040404 ;72:REACTOR (PART)
                HEX 8080808080808080 ;73
                HEX 00FCCCFCCCFC0000 ;74:~8 NOT UNDERLINED
                HEX 06196080FF000000 ;75:GUNS
                HEX 7886611806E11806 ;76
                HEX 0000008080806010 ;77
                HEX 030000008050A855 ;78
                HEX 1090909050503000 ;79
                HEX 080909090A0A0C00 ;80
                HEX C000000000010A15 ;81
                HEX 0000000101010608 ;82
                HEX 1E61861860871860 ;83
                HEX 60980601FF000000 ;84
                HEX 000000FF80601906 ;85
                HEX 0618E10618618678 ;86
                HEX 1060808080000000 ;87
                HEX AA54A04000000003 ;88
                HEX 0030505090909010 ;89
                HEX 000C0A0A09090908 ;90
                HEX AA150A01000000C0 ;91
                HEX 0806010101000000 ;92
                HEX 601887601886611E ;93
                HEX 000000FF01069860 ;94
                HEX 0000030C10102020 ;95:BALL & PEDISTAL
                HEX 0000C03008080404 ;96
                HEX 202010100C030000 ;97
                HEX 0404080830C00000 ;98
                HEX 040B080601010102 ;99
                HEX 20D0106080808040 ;100
                HEX 00030C1010171416 ;101:FUEL
                HEX 00FF000000575456 ;102
                HEX 00C0300808484848 ;103
                HEX 1414101010101010 ;104
                HEX 5477000000000000 ;105
                HEX 4868080808080808 ;106
                HEX 0000030C10102020 ;107:BALL-REDEFINED
                HEX 0000C03008080404 ;108
                HEX 202010100C030000 ;109
                HEX 0404080830C00000 ;110
                HEX 007F7F7F7F7F7F7F ;111
                HEX 007F7F7F7F7F7F7F ;112
                HEX 007F7F7F7F7F7F7F ;113
                HEX 007F7F7F7F7F7F7F ;114
                HEX 007F7F7F7F7F7F7F ;115
                HEX 0000000000010102 ;116:SHIP & TRACTOR BEAM-REDEFINED
                HEX 2050508888040402 ;117
                HEX 0000000000000000 ;118
                HEX 020C100804040201 ;119
                HEX 0201000001718A04 ;120
                HEX 0080408000000000 ;121
                HEX 0000000000000000 ;122
                HEX 0000000000000000 ;123
                HEX 0000000000000000 ;124
                HEX 0000000000000000 ;125
                HEX 0000000000000000 ;126
                HEX 0000000000000000 ;127
                HEX 0000000000000000 ;128
                HEX 0000000000000000 ;129
                HEX 0000000000000000 ;130
                HEX 0000000000000000 ;131
                HEX 0000000000000000 ;132
                HEX 0000000000000000 ;133
                HEX 0000000000000000 ;134
                HEX 0000000000000000 ;135
                HEX 0000000000000000 ;136
                HEX 0000000000000000 ;137
                HEX 0000000000000000 ;138
                HEX 0000000000000000 ;139
                HEX 0000000000000000 ;140
                HEX 0000000000000000 ;141
                HEX 0000000000000000 ;142
                HEX 0000000000000000 ;143
                HEX 81C2FCE4E4FCFEFF ;144:TOWBAR CHARS
                HEX 81C2FCE4E4FCFEFF ;145
                HEX 81C2FCE4E4FCFEFF ;146
                HEX 81C2FCE4E4FCFEFF ;147
                HEX 81C2FCE4E4FCFEFF ;148
                HEX 81C2FCE4E4FCFEFF ;149
                HEX 81C2FCE4E4FCFEFF ;150
                HEX 81C2FCE4E4FCFEFF ;151
                HEX 81C2FCE4E4FCFEFF ;152
                HEX 81C2FCE4E4FCFEFF ;153
                ;
                ; The following 32 characters are redefined at run-time.
                ; So at load time, they can hold some code we want to put on
                ; the stack.
                ;

STACK
;                HEX 0202020202020202 ;154:DOT CHARS-REDEFINED
;                HEX 0202020202020202 ;155
;                HEX 0202020202020202 ;156
;                HEX 0202020202020202 ;157
;                HEX 0202020202020202 ;158
;                HEX 0202020202020202 ;159
;                HEX 0202020202020202 ;160
;                HEX 0202020202020202 ;161
;                HEX 0202020202020202 ;162
;                HEX 0202020202020202 ;163
;                HEX 0202020202020202 ;164
;                HEX 0202020202020202 ;165
;                HEX 0202020202020202 ;166
;                HEX 0202020202020202 ;167
;                HEX 0202020202020202 ;168
;                HEX 0202020202020202 ;169
;                HEX 0202020202020202 ;170
;                HEX 0202020202020202 ;171
;                HEX 0202020202020202 ;172
;                HEX 0202020202020202 ;173
;                HEX 0202020202020202 ;174
;                HEX 0202020202020202 ;175
;                HEX 0202020202020202 ;176
;                HEX 0202020202020202 ;177
;                HEX 0202020202020202 ;178
;                HEX 0202020202020202 ;179
;                HEX 0202020202020202 ;180
;                HEX 0202020202020202 ;181
;                HEX 0202020202020202 ;182
;                HEX 0202020202020202 ;183
;                HEX 0202020202020202 ;184
;                HEX 0202020202020202 ;185

                ;ROUTINES TO SIT ON STACK
                ;
                ORG STACK
                BYTE $A9
PRASDELAY       RTS
                ;
PRESMESS        ;*C-16 THRUST IS COPYRIGHT FIREBIRD 1987* (reversed,back to front)
                BYTE 170,183,184,185,177,160,132,146,137,130
                BYTE 133,146,137,134,160,148,136,135,137,146
                BYTE 153,144,143,131,160,147,137,160,148,147
                BYTE 149,146,136,148,160,182,177,173,131,170
                ;
PRESET          SEI
                CLD
                LDA #$04
                STA $FF12
                AND #0
                STA $FF11
                STA $FF19 ;BORDER
                TAY
RST1            STA $0002,Y
                STA $0200,Y
                STA $0300,Y
                STA $0400,Y
                STA $0500,Y
                STA $0600,Y
                STA $0700,Y
                STA $3EFA,Y
                STA $0800,Y
                STA $0900,Y
                STA $0A00,Y
                STA $0B00,Y
                INY
                BNE RST1
                ;
                LDA #$0C
                STA RESSMC+2
                LDA #$33 ;COUNTER: MORE THAN ENOUGH
                STA $02
RESMA           LDX #$27
RESMB           LDA RESMESS,X
PRESSMC         STA $0C00,Y
                INY
                BNE RESMC
                INC RESSMC+2
                DEC $02
                BEQ RESMD
RESMC           DEX
                BPL RESMB
                BMI RESMA
                ;
RESMD           LDA #$08
                STA $FF07
                LDA #$1B
                STA $FF06
                LDA #$D0
                STA $FF13
                LDA #NEWRES&255 ;LO BYTE OF NEW RESET ADDRESS
                STA $FFFA ;HI BYTE IS SAME
                STA $FFFC
LOOP            DEC $FF15 ;SCREEN BACKGROUND COLOUR
PNEWRES         SEI
                CLC
                BCC LOOP ;JMP
                ;
STKOFFSET       EQU STACK-$0100
RASDELAY        EQU PRASDELAY-STKOFFSET
RESMESS         EQU PRESMESS-STKOFFSET
RESET           EQU PRESET-STKOFFSET
NEWRES          EQU PNEWRES-STKOFFSET
RESSMC          EQU PRESSMC-STKOFFSET

                ORG [STACK + 32 * 8]
                HEX 00000000000000FF ;186:HYPERSPACE CHARS
                HEX 000000000000FFFF ;187
                HEX 0000000000FFFFFF ;188
                HEX 00000000FFFFFFFF ;189
                HEX 000000FFFFFFFFFF ;190
                HEX FF00000000000000 ;191
                HEX FFFF000000000000 ;192
                HEX FFFFFF0000000000 ;193
                HEX FFFFFFFF00000000 ;194
                HEX FFFFFFFFFF000000 ;195
                HEX 0101010101010101 ;196
                HEX 0303030303030303 ;197
                HEX 0707070707070707 ;198
                HEX 0F0F0F0F0F0F0F0F ;199
                HEX 1F1F1F1F1F1F1F1F ;200
                HEX 8080808080808080 ;201
                HEX C0C0C0C0C0C0C0C0 ;202
                HEX E0E0E0E0E0E0E0E0 ;203
                HEX F0F0F0F0F0F0F0F0 ;204
                HEX F8F8F8F8F8F8F8F8 ;205
                HEX 00FCCCFC0CFC0000 ;206:~9 NOT UNDERLINED
                HEX 030C102146489080 ;207:REACTOR (REST OF)
                HEX C7340C0404040404 ;208
                HEX 808040407F405050 ;209
                HEX 3C429DB1B19D423C ;210:COPYRIGHT SIGN

                ;TRACTOR BEAM DATA
TRBMDT          HEX 0200800200800200
                HEX 8004004004004004
                HEX 0040040040080020
                HEX 0800200800200800
                HEX 2010001010001010
                HEX 0010100010200008
                HEX 2000082000082000
                HEX 0840000440000440
                HEX 0004400004800002
                HEX 8000028000028000
                HEX 02

                ;BALL DATA
BALDAT          HEX 0E00318040404040
                HEX 8020802080204040
                HEX 404031800E00FF

                ;COLOUR TABLE
COLTAB          HEX 7171717171717171 ;0-7
                HEX 7171717171717171 ;8-15
                HEX 7171717171717171 ;16-23
                HEX 7171717979797979 ;24-31
                HEX 7971717979797971 ;32-39
                HEX 7171717171717171 ;40-47
                HEX 7979797979797979 ;48-55
                HEX 7979717979797979 ;56-63
                HEX 7979797979797979 ;64-71
                HEX 7171717171717971 ;72-79
                HEX 7179717171717171 ;80-87
                HEX 7971717971717173 ;88-95
                HEX 7373737171717171 ;96-103
                HEX 7171717373737372 ;104-111
                HEX 7374757671717171 ;112-119
                HEX 7171717171717374 ;120-127
                HEX 7577767271717171 ;128-135
                HEX 7171717171717171 ;136-143
                HEX 7171717171717171 ;144-151
                HEX 7171717171717171 ;152-159
                HEX 7171717171717171 ;160-167
                HEX 7171717171717171 ;168-175
                HEX 7171717171717171 ;176-183
                HEX 7171717171717171 ;184-191
                HEX 7171717171717171 ;192-199
                HEX 7171717171717171 ;200-207
                HEX 717171 ;208-210

                ;SINE TABLE
SINTAB          BYTE $00,$19,$31,$47,$5B,$6A,$76,$7E
                BYTE $7F,$7E,$76,$6A,$5B,$47,$31,$19
                BYTE $00,$E7,$CF,$B9,$A5,$96,$8A,$82
                BYTE $81,$82,$8A,$96,$A5,$B9,$CF,$E7

                BYTE 0,0,0,0,0,0,0

                ;VECTORS
                BYTE $00,$01,$00,$01,$00,$00


;
; File X7.C16 was empty
;

PROG_END
               IF PROG_END>$4000
               ECHO "ERROR: PROGRAM TOO LONG BY ",[PROG_END-$4000]," BYTES !!!"
               ERR
               ENDIF

                ; 2009-11: MY: Uncomment this block of code, and the place that
                ;              calls it above, to allow the screen smooth
                ;              scroll position on the title page to be adjusted
                ;              by the arrow keys.  The X and Y smooth scroll
                ;              offsets are shown to the left of the lives
                ;              number.
                ;
                ;              Note that this makes the program Plus/4 only --
                ;              ie too long to fit on a real Commodore 16!
                ;
;XX_SCROLL_TEST  CLC
;                CALL_CHKKY_NB C16_KEY_RIGHT
;                BNE XX_NOT_RIGHT
;                LDY SMTHX
;                INY
;                BNE XX_STORE_SMTHX ; = JMP
;XX_NOT_RIGHT    CALL_CHKKY_NB C16_KEY_LEFT
;                BNE XX_NOT_LEFT
;                LDY SMTHX
;                DEY
;XX_STORE_SMTHX  TYA
;                AND #7
;                STA SMTHX
;XX_NOT_LEFT     LDA SMTHX
;                ADC #$30
;                STA $0C5E
;                CALL_CHKKY_NB C16_KEY_DOWN
;                BNE XX_NOT_DOWN
;                LDY SMTHY
;                INY
;                BNE XX_STORE_SMTHY ; = JMP
;XX_NOT_DOWN     CALL_CHKKY_NB C16_KEY_UP
;                BNE XX_NOT_UP
;                LDY SMTHY
;                DEY
;XX_STORE_SMTHY  TYA
;                AND #7
;                STA SMTHY
;XX_NOT_UP       LDA SMTHY
;                ADC #$30
;                STA $0C5C
;                RTS

